/*
 * @(#)ChessPieceType.java
 *
 * 2002 - 2020 JIDE Software Inc. All rights reserved.
 * Copyright 2005 - 2020 Catalysoft Ltd. All rights reserved.
 */

import java.awt.Image;
import java.awt.Toolkit;

/**
 * The images contained within this class are as used in the excellent Jin chess client and
 * were obtained and adapted from the download at http://ixian.com/chess/jin-piece-sets/
 * 
 * In accordance with the Creative Commons license of those images, you are free to use this
 * Java class under the following conditions:
 *  - you may share and adapt the class
 *  - you must attribute the source of the work to the original Jin Piece Sets and this Java Class
 *    adaptation provided as a demo of JIDE Charts
 *  - you must apply the share-alike principle to your derived work
 *  See http://creativecommons.org/licenses/by-sa/3.0/  
 * 
 * @author Simon White (FICS handle 'swhite')
 */
public enum ChessPieceType {
    bb, bk, bn, bp, bq, br, wb, wk, wn, wp, wq, wr;
    private final int size = 52;
    private Image image;
    
    public Image getImage() {
        if (image == null) {
            byte[] imageBytes;
            if (this == bb) {
                imageBytes = blackBishop();
            } else if (this == bk) {
                imageBytes = blackKing();
            } else if (this == bn) {
                imageBytes = blackKnight();
            } else if (this == bp) {
                imageBytes = blackPawn();
            } else if (this == bq) {
                imageBytes = blackQueen();
            } else if (this == br) {
                imageBytes = blackRook();
            } else if (this == wb) {
                imageBytes = whiteBishop();
            } else if (this == wk) {
                imageBytes = whiteKing();
            } else if (this == wn) {
                imageBytes = whiteKnight();
            } else if (this == wp) {
                imageBytes = whitePawn();
            } else if (this == wq) {
                imageBytes = whiteQueen();
            } else {
                imageBytes = whiteRook();
            }
            image = Toolkit.getDefaultToolkit().createImage(imageBytes);
        }
        return image;
    }

    public int getSize() {
        return size;
    }
    
    public static byte[] blackBishop() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,12,56,
            73,68,65,84,120,-38,-19,-100,121,124,
            20,69,22,-57,103,-90,7,-61,21,22,
            21,92,4,-27,90,69,13,-54,-67,17,
            97,-27,50,-128,44,4,-63,16,3,114,
            10,-126,16,1,-123,-49,-126,-100,126,80,
            89,-3,-128,-36,8,-70,-8,-111,67,-112,
            59,68,14,65,118,87,9,43,16,52,
            10,-78,-128,-72,68,16,-78,-128,-111,5,
            9,16,36,100,-90,-85,118,94,-6,87,
            13,-11,-96,51,-28,88,62,73,118,-22,
            -5,-57,75,-43,116,-41,-15,-86,-21,-43,
            -85,-22,-22,-72,92,-95,16,10,-1,-125,
            -32,97,-15,-102,-112,-23,-112,-89,32,107,
            4,-71,47,20,10,57,-124,51,125,-121,
            65,-2,-61,-95,31,-9,90,-46,125,37,
            -9,124,66,-95,-96,-95,-84,30,117,87,
            -125,-84,-52,46,59,9,-71,-99,93,-113,
            17,-26,-82,-102,123,-66,-95,-112,-33,-32,
            -122,120,69,79,-2,-35,91,-106,76,92,
            7,11,-73,15,114,-109,37,19,48,-126,
            106,29,102,-39,13,-41,-13,13,-123,-126,
            -10,79,127,61,126,87,37,75,30,-5,
            -40,-110,82,90,82,12,-124,-20,-87,-89,
            -1,-40,-61,-110,-107,-101,-79,124,-97,13,
            -23,-74,80,-126,-15,-85,30,31,-71,95,
            -17,-128,-84,-107,-24,-104,-7,-112,-77,-11,
            116,117,-35,-56,-105,89,-66,-105,66,-70,
            45,20,-45,-26,53,-12,-28,-39,-61,117,
            -59,-5,-17,66,28,10,-105,-24,80,-1,
            -19,-6,117,-77,71,-23,-7,-40,-7,-122,
            76,93,1,71,80,-118,30,127,33,-123,
            117,-48,100,-60,51,32,-49,33,125,-102,
            126,-35,11,-65,101,-7,38,-121,116,91,
            -80,80,10,114,-87,-98,28,-10,-109,37,
            -105,37,-80,57,8,-50,-128,56,-83,-89,
            127,-120,57,43,-116,-51,65,-82,15,88,
            57,-95,112,-109,-95,28,-77,116,-17,-61,
            36,117,-43,-45,-21,-62,-92,101,60,-83,
            119,-120,-110,-25,-74,90,50,-126,-115,64,
            111,55,-28,59,61,-9,114,67,-127,-51,
            53,-82,48,-74,-48,60,-128,-97,-113,-24,
            -23,-15,13,-32,4,-44,101,29,-45,2,
            -78,-71,-98,126,-7,-104,37,7,-17,103,
            -59,10,-108,-77,-125,-43,-89,116,104,110,
            -62,36,-64,-30,117,-112,124,64,79,-66,
            99,0,-42,59,111,-21,-118,55,-65,69,
            -68,-61,-115,71,-112,108,-61,-82,67,122,
            -62,120,75,-34,-50,-74,126,-116,93,-8,
            -125,47,104,-1,-17,-74,-120,-68,44,62,
            12,-55,-51,-11,-28,-90,71,45,121,124,
            -99,-82,96,-33,110,-52,53,-55,-84,67,
            126,101,82,-51,77,-40,10,-14,-19,96,
            -21,-93,109,-106,-116,100,15,-118,23,35,
            -47,-43,51,72,-67,75,-4,-120,25,113,
            -29,17,-45,127,8,20,-70,-120,121,107,
            3,117,47,-51,-18,24,-65,-61,8,-30,
            -23,-1,65,62,-79,122,122,54,-26,-74,
            62,79,-78,-22,-86,-51,-41,62,37,125,
            68,113,91,-114,39,-44,104,-81,39,-65,
            -70,-113,117,8,-100,0,115,-109,-99,-18,
            38,100,-90,29,-65,-101,8,116,-60,68,
            34,48,114,44,121,-55,78,119,-70,-49,
            -54,23,94,-100,-39,64,79,31,-5,21,
            -21,40,-75,126,122,-104,117,80,-119,-103,
            -93,-104,119,100,-12,-42,-29,-109,-40,28,
            -31,-101,-29,96,-62,46,59,-116,-108,-68,
            74,110,2,19,116,-45,-87,-46,-57,63,
            -50,-22,61,-95,-92,121,125,-52,102,27,
            127,-44,-29,-49,-83,96,35,-90,12,-30,
            -65,103,10,-51,14,98,-70,-32,70,-117,
            43,-70,-108,41,65,-18,-29,-7,86,65,
            61,-40,94,94,-81,104,-42,-114,-106,65,
            76,119,-79,9,112,-101,-35,-89,-39,-6,
            5,-81,11,50,-57,-34,-8,73,-66,70,
            -95,-27,-120,-85,35,-64,72,33,-28,-65,
            85,-4,-111,-34,-124,-103,-99,122,39,97,
            46,78,-83,66,-104,11,30,-23,75,-104,
            -25,-40,125,105,118,-2,122,-66,62,86,
            -113,-73,-12,-8,121,44,-128,31,24,-52,
            44,119,10,115,-53,-117,-35,-100,-45,17,
            -47,113,-6,-49,-101,126,102,35,-25,53,
            -42,49,62,39,19,-27,-103,76,-88,-55,
            94,-54,-58,27,9,-13,-88,-76,66,119,
            -56,46,-115,55,19,87,-35,107,126,-97,
            -125,100,35,-54,-33,79,-113,-81,-17,-57,
            -102,-71,1,127,60,81,-36,-26,36,12,
            121,79,-110,-98,-36,106,35,123,82,-47,
            32,121,-24,-70,-71,-63,-102,-44,-91,-21,
            28,97,43,48,-85,78,39,66,54,106,
            -70,-112,-112,-57,6,60,74,-120,-122,-24,
            -104,-79,-112,127,26,-48,-100,16,15,-32,
            -70,52,-36,-41,80,-27,99,-25,-85,-54,
            -47,59,74,109,-70,-62,105,17,-9,-23,
            -65,-73,-16,49,-89,110,67,113,51,117,
            -40,69,54,98,-39,-42,87,26,123,66,
            -121,-39,10,-29,79,-76,65,72,97,-4,
            76,-40,-23,95,44,-8,51,97,38,-94,
            35,-92,-65,2,33,51,37,11,60,93,
            -35,-89,-14,-79,-13,69,57,14,35,10,
            115,-103,-1,69,61,-3,-125,37,-20,121,
            -116,-43,-37,93,-12,67,2,-13,21,48,
            114,82,-97,101,35,40,-46,-47,-92,-87,
            14,74,32,-20,-12,-125,-77,22,19,-26,
            27,-48,123,124,-58,-67,-124,124,10,-15,
            43,-112,89,25,-43,9,-39,89,93,-89,
            -18,83,-7,-40,-7,-26,-34,65,-88,-105,
            120,84,79,79,-59,-70,-55,-101,-104,123,
            -69,-117,-18,20,52,71,-113,87,-61,-21,
            -128,75,124,-92,-36,31,-44,45,-114,33,
            -92,-87,-30,21,50,9,-7,68,-7,-89,
            8,57,56,106,36,97,78,83,-90,13,
            114,120,-44,40,-62,-100,-124,-21,-30,113,
            95,27,-89,124,-125,72,86,-49,75,25,
            104,-41,-85,-71,-73,-69,-24,118,-48,39,
            122,-68,-6,47,-40,-36,-100,-21,-80,-71,
            121,-45,-21,24,-49,107,-124,60,99,59,
            9,125,9,83,-96,99,-6,64,-58,53,
            -18,79,-104,-25,-99,-18,-53,-93,100,-11,
            -52,-62,46,120,-11,83,-71,-73,-69,-24,
            -122,116,61,90,22,-17,111,78,-80,21,
            -69,-104,27,-52,107,115,-107,34,-82,113,
            -77,-37,17,-14,-92,-118,55,-23,65,-120,
            82,-24,-104,65,-112,125,-101,-12,34,-52,
            43,78,-9,-15,124,-125,-104,56,-10,96,
            -99,120,16,-19,58,-106,123,-69,-117,110,
            -88,14,-17,-122,109,-17,-81,-118,102,78,
            -62,82,-121,5,-93,-13,8,122,-109,-72,
            -58,-51,62,79,-104,71,-84,126,17,-23,
            -112,105,-115,51,-119,107,-36,108,-21,-66,
            -45,121,28,57,-39,-84,-98,72,95,21,
            -59,-68,-72,-3,122,-69,-117,126,-64,94,
            -107,103,-101,-98,-36,58,-111,-19,-123,-87,
            39,92,29,2,-55,-68,-50,-51,86,-78,
            3,33,125,-18,-27,-124,-4,-59,-98,-109,
            122,17,-14,-15,22,113,-124,-104,-48,-94,
            59,33,-58,87,-24,73,-56,102,-22,58,
            -9,82,34,112,95,123,-30,-102,7,-30,
            -58,110,118,-90,94,47,-109,61,64,-83,
            87,-79,-26,110,43,110,-101,-88,106,-63,
            6,47,-51,-61,86,-38,-85,107,-24,-90,
            -58,-33,-40,-114,-1,-127,-112,23,-115,-90,
            -124,-68,-32,-67,-121,8,40,-72,9,113,
            -115,34,-53,18,55,49,18,-40,117,-18,
            -106,-124,-108,-34,-102,68,-64,-12,69,18,
            -14,-94,42,-41,-79,94,86,-67,-85,-80,
            -114,41,-93,-73,-77,-8,109,-98,-94,99,
            -36,-109,-104,5,92,102,-55,-45,109,-40,
            -100,-44,-39,-114,-105,33,2,-78,26,33,
            97,-21,-27,-24,10,-9,18,-46,117,127,
            53,66,-84,-115,-100,73,-120,-113,91,86,
            34,-52,-71,45,-17,32,-52,119,34,-121,
            18,-30,-93,-5,-114,19,98,93,-8,1,
            66,98,111,80,14,15,24,-93,0,50,
            -51,46,-57,-87,30,86,61,91,-95,-34,
            -117,-39,115,56,-87,-72,110,-11,56,108,
            34,26,117,-11,120,-105,115,-82,9,-124,
            109,-14,82,-61,118,16,-14,-10,-24,40,
            66,-60,79,-33,70,-120,-40,93,123,8,
            -111,116,-10,27,66,62,40,79,-26,48,
            94,-2,-108,-61,14,121,46,-121,-63,-74,
            60,-99,67,-110,60,-104,-61,-124,-77,-15,
            -124,124,96,-41,99,-124,88,51,-67,22,
            33,98,59,101,19,98,116,88,10,33,
            113,-108,-40,-4,-105,94,-81,46,-23,-71,
            -73,-93,-8,110,-106,58,-67,110,-80,-35,
            81,-9,65,98,-52,-109,-22,-119,45,29,
            77,-8,79,30,-6,27,97,-2,-96,118,
            2,68,77,66,30,-109,29,114,-104,114,
            -90,54,33,-85,29,-118,34,-60,63,-9,
            -10,35,100,-21,-67,-3,9,-39,-14,-48,
            64,66,-92,-100,25,65,-56,-22,114,64,
            14,83,69,85,66,-90,-87,124,81,-50,
            -15,-46,93,9,127,-70,109,10,-65,35,
            -58,-76,98,-11,-34,92,-46,94,55,-16,
            112,27,36,94,124,25,11,-19,-67,-122,
            40,98,-2,97,-91,-96,-10,-99,9,95,
            -101,85,-33,17,-30,-82,103,46,19,-94,
            103,-60,-85,-124,88,94,113,4,33,46,
            121,111,35,100,61,87,87,66,-114,-122,
            28,-27,-67,-109,-112,15,85,92,75,-120,
            -53,17,103,9,-79,52,110,18,33,-94,
            87,-41,36,68,-59,-10,125,8,95,-108,
            42,-41,-37,-106,-104,-1,45,-21,-104,69,
            122,-67,-19,118,-108,-72,-64,-68,28,111,
            37,61,-34,127,-110,107,27,17,120,-126,
            39,18,-2,-102,80,-36,41,87,56,97,
            -49,21,34,-97,11,78,-63,-14,57,-27,
            30,77,-8,35,92,73,-124,74,127,-82,
            60,-85,103,120,-18,-19,40,121,1,10,
            48,-20,39,-43,-13,10,49,115,-85,82,
            104,-67,106,68,118,-3,-34,-93,8,115,
            113,-20,21,-62,-36,16,-69,-106,48,87,
            118,123,-113,48,-41,116,-85,78,-104,-21,
            98,-4,-124,-71,94,-55,110,53,8,115,
            93,-73,-9,9,115,77,-20,122,-62,92,
            25,43,8,115,67,-17,-105,8,115,73,
            -67,4,34,-37,-10,42,61,-81,19,-45,
            126,102,35,-24,112,73,53,105,78,-18,
            119,28,-28,-105,-74,10,26,18,95,-107,
            85,-118,-38,116,-128,-16,125,109,-17,78,
            111,37,-92,-102,-109,78,-27,87,-6,119,
            19,118,62,114,-117,65,-8,-20,51,15,
            70,101,34,-71,23,-85,-74,26,81,-99,
            -117,-85,59,-99,87,75,-41,-42,54,30,
            19,-119,122,-104,-100,-3,-115,-54,-49,34,
            124,-107,19,107,19,89,51,62,107,78,
            100,31,-35,-4,54,33,-54,109,-119,33,
            68,-3,45,-49,18,-94,-50,-106,-18,-124,
            120,-56,81,-86,-21,112,-33,-26,25,-124,
            40,-9,89,3,34,-5,72,98,71,34,
            107,106,-7,21,-124,-17,25,-85,30,-66,
            -86,-34,79,-119,-121,-65,96,-11,110,88,
            -46,71,80,4,26,-70,-32,-22,54,29,
            -79,100,-69,61,87,-84,32,-4,115,92,
            83,9,123,-82,48,-79,119,-90,94,23,
            -60,67,-66,12,57,18,-78,63,-109,35,
            -39,117,-15,44,31,-45,53,-123,8,-108,
            -77,-100,-16,-49,-48,23,-72,-117,-39,55,
            -82,-98,-27,-8,-29,-95,-94,-82,104,126,
            -40,124,45,26,-128,47,-38,-116,119,33,
            -15,117,-127,49,19,-29,69,-99,-110,-63,
            36,31,126,57,48,-35,6,-104,-34,-40,
            -110,-17,-30,-52,-12,-4,-57,-116,-87,-60,
            -52,61,-34,-31,-60,59,-33,123,38,16,
            -66,61,-98,81,68,96,-82,24,75,-56,
            121,-98,49,-124,29,-1,4,114,51,75,
            -97,103,-33,55,-111,-16,37,121,-121,18,
            -17,100,25,51,-120,89,48,-83,-13,35,
            80,-113,-25,81,47,120,107,-31,73,-80,
            108,-104,-125,-44,1,75,-43,46,-69,-99,
            104,-73,71,29,-37,-6,40,-120,-34,10,
            -35,-5,98,-74,-41,-125,87,-44,-18,-5,
            10,-70,62,112,-73,37,92,-9,32,-33,
            -71,87,-73,-17,-120,-15,-125,92,-75,-120,
            -64,-109,93,-101,48,-25,90,82,-18,-125,
            68,-70,44,3,89,-106,-91,-29,58,115,
            -70,-99,79,27,98,-20,-101,-84,61,-85,
            -35,47,17,-82,-69,11,103,-67,-25,110,
            -126,124,-49,58,-24,-77,-64,-34,-96,-61,
            100,-24,97,13,11,-5,-44,-110,-51,-79,
            -82,-23,-122,5,94,76,125,-56,-25,33,
            -15,-115,105,-52,75,-112,48,25,-49,-64,
            -101,-21,-119,-9,53,47,-30,125,-53,70,
            126,76,106,16,-92,58,-32,-56,14,-55,
            -89,-29,-9,-12,97,-20,62,117,-35,-41,
            -112,-81,-21,-65,-81,-57,-106,77,-4,65,
            -44,99,15,-22,5,-117,16,-77,-108,-43,
            91,-75,3,103,-58,99,-22,-95,-35,-19,
            -96,-121,10,-48,75,35,-90,-73,55,28,
            -12,-101,111,-89,-93,76,-18,5,116,-124,
            -115,62,82,-118,41,4,54,-6,-70,35,
            -70,-63,-41,41,122,28,35,84,-88,47,
            -24,78,-80,-41,21,11,-11,-8,18,-68,
            119,-6,48,-110,93,-73,-116,-27,-117,-77,
            17,2,-81,7,-28,-107,32,-11,8,38,
            -43,-18,122,-124,-98,-98,-118,14,125,-110,
            -17,126,79,-55,93,-49,55,31,96,-101,
            61,43,-39,-126,114,28,-85,32,70,-120,
            -8,13,20,-126,119,-13,-2,111,32,113,
            98,-45,-97,-52,36,-66,38,-16,-29,124,
            -103,31,54,-37,-9,61,-30,127,-41,21,
            106,-105,7,47,-48,62,-85,-115,-12,-57,
            112,-34,-82,-103,-105,117,80,12,-30,93,
            88,62,-87,-8,29,115,-88,15,-97,-71,
            -8,-105,-77,122,-19,114,-88,-1,110,-42,
            -50,4,93,15,114,-70,94,94,63,118,
            -32,-47,-13,81,1,59,-56,-67,91,-113,
            -41,-66,-45,-110,23,-121,48,5,-4,5,
            -15,51,76,-98,101,35,41,-104,84,-37,
            -3,62,38,-43,102,42,20,-106,-3,87,
            61,125,102,-37,27,-41,127,86,-122,126,
            93,54,20,98,-2,-64,58,-118,-105,119,
            49,-113,-11,62,-53,-92,58,-92,63,89,
            47,-25,2,-68,-41,90,-111,76,-49,73,
            -7,-100,123,12,-10,31,58,-58,-107,-47,
            11,-52,-126,45,55,-15,-67,-115,-119,77,
            79,-77,115,30,101,39,72,60,-23,38,
            70,-126,15,-90,-61,-113,-71,64,124,-84,
            -105,63,-25,2,-85,54,70,-98,123,-109,
            -98,62,111,40,123,-99,-128,-81,-63,-3,
            59,81,-50,5,-67,92,-13,105,86,-81,
            -68,-74,71,-23,1,103,-71,-77,-40,9,
            -38,-79,-109,-39,78,69,86,94,-25,34,
            -27,77,-79,55,-94,27,-113,-80,-111,-77,
            38,-97,123,98,-63,-92,-102,19,92,122,
            122,26,58,32,-114,-101,10,124,71,-92,
            62,103,113,65,33,-98,47,-11,-21,122,
            124,110,-55,19,113,-84,60,120,83,55,
            -1,-22,61,111,-46,-65,90,-113,111,-4,
            -100,-43,127,-69,-82,-9,-32,1,43,102,
            119,45,61,121,65,3,75,-18,67,70,
            59,-95,-120,-28,-9,-14,41,-25,65,-30,
            -52,115,50,-68,-77,36,-100,29,88,8,
            -109,19,7,-81,-79,-84,96,79,-98,-6,
            87,46,-119,14,-19,-40,-62,-82,83,-34,
            48,70,100,119,-52,109,-117,102,89,114,
            59,58,42,25,94,101,-14,16,86,-49,
            124,-74,115,39,-26,-20,125,24,-95,11,
            -102,-80,-111,95,75,-41,123,-2,95,19,
            -36,-22,127,66,-60,-1,-105,14,108,-68,
            -15,-88,-102,20,-39,-17,94,38,-39,14,
            -122,-95,-66,-32,-5,-79,-80,-68,-88,124,
            -122,-16,-62,122,109,-63,-33,20,98,82,
            116,-9,-125,-60,-73,-95,-18,-24,2,74,
            120,87,110,-68,2,119,99,111,-50,-128,
            119,100,-88,45,-106,67,14,15,-50,-51,
            62,96,42,28,71,-66,-8,-76,-46,-128,
            41,116,-97,98,-11,-24,82,72,-19,-21,
            -96,-21,77,-23,-47,81,-49,-7,14,-54,
            -5,80,-97,10,-86,-29,71,109,11,73,
            -74,99,67,61,-20,22,61,-47,97,-84,
            -36,118,-123,-36,-82,40,-90,-73,72,87,
            40,-124,66,40,-36,-126,-16,95,49,-4,
            -63,-112,44,91,34,-9,0,0,0,90,
            122,84,88,116,67,111,109,109,101,110,
            116,0,0,120,-38,115,73,44,73,-75,
            82,8,79,77,-47,81,48,50,87,112,
            44,77,87,48,50,48,-80,80,48,-76,
            -76,50,-78,-80,50,49,81,8,112,9,
            -31,114,44,45,-55,-56,47,-78,82,112,
            45,-54,76,86,112,73,85,-16,45,-51,
            75,81,-80,73,77,76,113,-56,-84,-56,
            76,-52,-45,75,-50,-49,-75,-29,2,0,
            -31,23,21,24,123,-69,10,-77,0,0,
            0,0,73,69,78,68,-82,66,96,-126
        };
      }
    
    private static byte[] blackKing() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,19,80,
            73,68,65,84,120,-38,-19,-36,119,96,
            -43,84,-33,7,-16,-36,-28,118,81,40,
            -76,101,85,64,17,4,42,104,-87,-20,
            33,-123,34,-93,50,4,41,32,-96,47,
            32,-85,50,100,-120,-14,32,-96,-96,-128,
            80,120,-88,32,32,83,-12,97,-54,30,
            50,5,125,113,0,-54,16,69,65,-90,
            -84,66,105,75,-95,20,-80,-19,-51,-51,
            -17,-68,-7,-34,-100,115,74,35,-75,69,
            31,-75,-66,-10,124,-2,56,77,110,-110,
            -77,-110,-109,115,-110,-128,-94,20,-124,-126,
            80,16,-2,121,-63,97,91,46,-59,-29,
            33,-74,-72,84,46,-5,21,-124,63,41,
            -8,-16,-72,-126,45,-10,41,-88,-102,-126,
            -16,-113,14,-102,109,-71,-124,21,121,-83,
            -52,-66,58,-20,-74,21,31,107,-58,-29,
            86,124,61,-53,-66,-99,-36,-81,68,46,
            -23,20,-124,-33,23,-44,-103,-39,-105,107,
            29,-80,98,-58,-78,-57,-75,-114,-2,-6,
            126,5,-31,-9,-122,-42,-74,10,-82,101,
            -59,61,-65,-74,-30,1,85,-84,120,-54,
            -62,-69,55,-48,-108,37,124,-69,-54,-39,
            -9,19,-57,-55,41,-99,-126,-112,-41,-80,
            -35,-42,69,-71,-84,-8,-36,-65,-77,55,
            4,53,-69,123,3,-39,-41,-117,-3,-60,
            113,114,74,-89,32,-28,53,-68,-97,125,
            -47,-71,-36,-118,-65,-98,104,-59,87,-9,
            89,113,82,-20,-35,27,40,105,10,-33,
            110,111,-10,-3,-60,113,114,74,-89,32,
            -28,53,20,-70,123,87,84,-108,-33,-28,
            -99,124,24,-35,100,-34,-35,27,-88,-55,
            50,-66,-99,23,-33,-81,100,14,93,90,
            -95,127,-6,68,-78,52,-113,103,-13,-8,
            -70,-19,-52,45,121,111,19,74,117,-106,
            109,-112,-96,-33,-67,-127,106,-5,-4,-6,
            126,-71,-26,-69,-92,45,-97,34,-33,-77,
            -2,110,19,97,-5,-124,-16,18,31,-67,
            94,-32,21,115,-26,-41,39,-114,42,-65,
            39,104,-13,-7,-118,113,-74,13,-68,-77,
            15,-121,-75,49,-39,127,-82,-67,32,-121,
            6,-38,105,27,77,-1,75,-12,-107,-74,
            -29,-118,-64,-69,64,109,17,-49,87,-6,
            -81,95,-39,-22,21,-66,-3,9,-66,-2,
            120,46,-11,-14,-89,5,-89,109,-71,18,
            -49,104,29,-77,-56,-112,40,126,-88,19,
            2,-118,-46,116,51,104,-59,66,-85,-125,
            127,100,-51,82,-96,93,11,-19,5,119,
            28,-55,23,28,124,-12,-91,-18,-67,-29,
            -36,-124,96,37,13,-100,-105,28,101,-63,
            92,-97,10,-115,42,104,-3,-127,49,-83,
            39,-104,13,116,10,30,-33,-23,-16,1,
            115,-69,43,-32,76,-32,-57,9,-80,-99,
            32,71,20,63,112,-120,43,92,9,125,
            9,20,-91,-26,27,-96,-91,-122,-74,0,
            -1,-120,-90,-53,65,83,-21,70,-63,-99,
            -29,23,15,55,-81,-121,71,-8,-38,-112,
            63,123,-66,-27,107,91,62,-21,24,3,
            90,21,-17,121,96,-74,92,50,20,46,
            -31,87,19,122,30,-43,90,-64,-90,22,
            -42,-26,-89,74,-101,-99,-122,41,97,-82,
            -75,124,58,-55,89,19,-42,-97,-13,61,
            9,-47,124,-62,-23,-100,-87,28,-14,48,
            -17,-21,-96,-107,113,-4,0,-38,28,-17,
            -1,-128,89,-91,-113,65,-25,117,106,127,
            72,-98,108,-65,-126,-44,-95,-112,20,-85,
            86,-122,-114,83,-68,35,-64,108,-25,109,
            -96,-83,82,126,2,-83,-76,122,20,-28,
            121,-105,-23,87,24,58,126,-28,12,-126,
            13,-123,121,62,-109,121,-66,-25,88,-53,
            39,-81,106,61,96,-29,120,-65,72,-24,
            81,-52,-71,14,-4,3,-67,-121,-125,-103,
            78,63,-48,-62,-8,-127,-9,-1,-47,87,
            86,-42,1,67,-64,-31,-25,12,6,-11,
            74,80,11,48,79,-96,98,-16,-116,67,
            125,23,46,94,-79,13,119,111,6,-24,
            -64,-12,114,19,-128,-79,-62,61,-127,-15,
            62,-99,78,-101,119,41,-109,121,5,-68,
            8,-33,126,-28,92,3,-11,-5,23,27,
            11,102,-127,-57,-127,25,47,-128,-104,8,
            -57,110,-112,-57,-1,-66,99,121,-96,54,
            -47,-95,64,-43,-8,122,-73,-29,20,48,
            -26,-104,9,125,67,29,61,65,81,2,
            83,-64,108,-110,35,-48,112,-103,-42,25,
            -114,60,-90,20,1,121,-36,51,-123,87,
            1,-69,81,-18,-33,-64,88,-64,62,96,
            -68,75,-90,-21,-14,-124,88,14,-25,-45,
            -68,-54,67,-12,-77,-59,-62,-63,-68,100,
            -114,120,20,-75,-54,-89,-36,-8,111,55,
            -108,122,-57,109,30,46,20,-22,8,14,
            126,69,-88,-31,-38,-5,48,55,-50,-47,
            25,100,-63,-30,-98,-82,1,-82,51,-69,
            -6,-125,-21,100,74,40,24,97,-82,-86,
            96,-84,75,-116,5,35,108,125,28,-72,
            70,52,-100,15,122,57,89,-32,-11,96,
            54,-40,44,120,-95,-75,-43,69,53,-20,
            -90,-116,4,-10,-67,-75,-99,49,-11,-3,
            99,-96,-49,103,86,88,-64,-29,-91,-17,
            -75,1,-3,5,-66,-35,106,-27,61,96,
            -115,-108,-81,-96,-63,121,-19,93,24,56,
            70,-99,14,89,87,94,-60,92,-48,-61,
            55,-50,2,-41,-70,-60,91,96,-44,113,
            61,2,-58,-70,-85,41,96,84,-35,-39,
            5,92,123,-38,53,1,23,31,-92,-48,
            102,-57,-85,96,-26,123,13,-52,-36,-63,
            107,-17,35,-33,-42,-32,-104,-53,-21,113,
            95,-114,-11,124,-113,-63,-113,-57,67,11,
            125,10,74,6,-65,33,-44,-45,38,-61,
            -22,109,-94,96,-91,94,-123,-52,-93,-37,
            125,-63,-83,-16,-118,58,78,-87,96,-2,
            117,-62,-29,7,-66,-2,29,-74,-44,-29,
            12,91,-26,33,-62,-72,-71,-81,-125,-2,
            -88,79,8,-48,76,-27,5,112,-3,-92,
            -99,-122,84,62,108,102,-33,-66,-67,2,
            -36,23,-7,126,113,125,-53,3,41,49,
            47,3,-107,21,-21,-89,-51,3,55,31,
            -84,-80,117,-38,109,-72,49,68,-71,15,
            -12,-42,-66,83,-127,-70,47,-36,4,122,
            5,-66,95,44,123,-61,-125,-79,-75,30,
            34,-99,-103,-20,-88,-123,92,32,-13,-3,
            -3,-106,73,-96,39,-106,56,15,-103,55,
            69,-67,104,-45,97,-7,-92,-27,-93,-63,
            -68,100,24,56,-8,-77,66,-91,-69,-83,
            -98,-17,-7,-54,41,-87,12,3,117,-83,
            -93,20,-104,-105,-20,36,-104,-106,38,50,
            80,-27,115,-56,72,58,-73,3,-88,-110,
            -56,-79,-66,4,-40,-32,101,-83,-128,90,
            70,125,12,52,52,124,52,-48,-53,29,
            -74,1,-75,-3,100,44,-48,115,124,-73,
            -99,98,-1,47,-82,-127,81,59,-48,5,
            -18,46,60,61,103,-92,47,-72,59,-15,
            -51,-90,46,-100,7,-60,-69,10,-74,-104,
            -57,-77,-33,-69,9,-28,-49,-73,-5,79,
            -109,-26,-32,46,-62,-49,-12,79,-125,-37,
            -128,59,102,-17,49,48,-38,-80,-20,97,
            -57,78,111,-96,-25,-98,62,14,52,32,
            124,44,-48,-88,39,-69,1,13,95,49,
            11,-88,-126,123,52,-80,-111,98,-57,-77,
            -127,64,-59,43,87,-126,-52,82,-78,-95,
            -26,-64,-28,-111,14,79,48,87,-11,7,
            115,112,98,-123,98,-9,122,37,-15,81,
            -102,58,87,-82,24,11,17,-73,68,-126,
            65,27,-63,-3,-4,-119,-35,96,-92,-120,
            12,-90,-100,4,74,-117,-54,4,10,-28,
            -37,-17,112,76,7,-77,43,-15,100,-116,
            101,40,103,64,118,45,47,-68,-110,14,
            70,50,-101,-32,-15,-90,56,-34,39,27,
            -64,88,107,-19,79,-125,-67,39,1,-21,
            -10,-35,90,-96,81,23,50,-128,126,40,
            -44,10,-40,3,-2,71,-128,41,-15,19,
            -127,54,29,86,-127,-122,122,13,4,-42,
            67,125,28,104,-40,-89,-37,-63,-8,66,
            -92,67,-19,-128,117,24,86,30,12,113,
            5,-116,86,12,48,-13,-39,7,88,-122,
            35,22,100,-66,-41,-75,125,28,-88,72,
            106,67,32,121,-126,29,95,4,-58,-111,
            -64,67,-32,-2,70,-44,-101,115,19,52,
            -8,-64,118,61,-68,-102,-61,-24,56,-57,
            80,-125,-17,-40,88,54,80,52,108,-72,
            36,18,122,119,38,-72,-66,18,25,-70,
            25,10,-108,-39,32,30,-88,-67,-40,-50,
            43,16,-104,91,-101,9,-20,-106,-38,22,
            -40,117,45,25,-40,77,103,87,-112,5,
            62,57,-32,107,-96,-121,-7,97,95,-30,
            -15,-49,-79,79,-127,-111,-54,-73,75,111,
            -104,4,-44,-118,-1,30,-10,82,34,24,
            -6,43,39,-127,-68,-8,-6,-78,-11,98,
            -128,-94,-7,126,-87,83,63,5,35,-111,
            -1,46,58,-87,-105,-6,-50,0,-125,-33,
            -60,-39,119,-50,113,96,-98,-7,-15,96,
            -26,-69,13,-80,84,109,62,-80,91,94,
            33,-112,117,-17,106,-4,26,24,-119,-73,
            -33,4,22,40,-22,101,-10,18,112,-67,
            38,27,104,36,-84,89,101,107,32,-2,
            -69,82,53,-81,15,2,-28,68,48,96,
            61,-124,-16,71,35,105,15,-107,-98,8,
            -116,93,11,2,35,83,100,-28,95,109,
            -63,-32,55,111,118,-47,-69,12,48,-73,
            121,-15,-102,-60,-88,-57,44,-112,15,-36,
            49,-95,92,11,-20,-74,87,5,-112,-21,
            -105,78,-39,15,-58,49,-39,101,-42,1,
            22,86,-1,7,-96,62,124,-69,-35,27,
            -85,0,-67,-2,115,56,-48,-66,-97,15,
            3,29,94,-37,25,-88,34,-33,110,115,
            -61,18,64,-111,-6,82,96,-95,-30,-72,
            111,109,3,35,-98,111,-73,-58,-85,57,
            -104,-7,-104,11,-20,118,-114,-7,46,11,
            44,-45,-69,56,48,-30,-21,-49,-116,-119,
            5,-29,-128,-20,81,-66,5,-29,70,-23,
            40,16,-5,95,79,43,86,6,74,-90,
            -39,-22,-67,79,94,-33,-109,-55,9,85,
            -111,86,16,57,65,100,-84,-35,90,112,
            63,-55,-45,-97,-101,-38,1,-104,-49,125,
            37,-128,-60,61,-128,57,-38,2,75,-77,
            38,-128,-10,25,-66,25,-85,112,-57,114,
            18,-80,-101,-54,94,96,-52,-69,4,80,
            -26,-95,111,-128,-70,-119,2,111,56,13,
            70,18,-33,-17,-3,-56,-106,64,61,-8,
            -49,-94,107,28,18,49,21,-88,43,-33,
            110,-42,-90,-3,96,-52,20,-57,57,56,
            31,-88,-121,87,40,-80,64,37,29,-52,
            124,28,5,118,43,-57,124,-118,-40,51,
            -95,54,-53,-7,34,48,113,-27,-79,-78,
            91,-128,22,-92,-11,2,22,-60,-109,-101,
            -45,110,54,-72,71,-118,-19,-118,44,-122,
            -58,-121,109,87,-46,-83,-68,54,-48,102,
            -15,-105,-1,113,-24,-14,-107,56,-16,-80,
            -127,-96,43,-78,-96,95,3,-119,10,-21,
            -83,-68,2,102,1,122,-128,121,-27,120,
            102,-18,119,41,-96,-120,53,-56,90,-42,
            -58,67,-42,-103,27,29,4,-44,-120,39,
            -73,-123,-113,-98,30,-81,27,9,52,-126,
            111,-9,-36,-119,121,64,-45,127,52,-128,
            -90,-14,-11,-49,-42,-81,3,52,-112,-83,
            -14,104,-58,-113,-77,-14,-23,-51,32,-81,
            48,-90,13,-126,60,52,-116,-120,69,-71,
            122,-127,89,-50,110,32,127,-17,116,-24,
            42,-112,-20,1,-122,13,5,93,21,-5,
            -5,39,66,-105,86,-74,122,-33,-104,-41,
            6,90,44,-2,42,28,5,-99,75,-117,
            3,15,127,3,-12,-102,34,-31,3,-73,
            -127,46,-117,-101,61,31,4,48,-91,47,
            -36,-47,-75,-27,22,-13,51,-110,-33,-116,
            51,-51,1,-88,-55,60,67,61,51,127,
            86,121,127,26,80,79,-111,-18,-62,53,
            64,124,-90,-49,-42,-65,-67,0,40,48,
            -18,109,32,81,17,43,63,-88,2,-28,
            45,-10,-37,-9,9,-48,51,74,125,96,
            97,-54,6,48,-45,-19,10,102,-70,-10,
            -82,44,-73,88,-108,115,16,-56,-11,-67,
            15,92,2,58,41,-46,29,62,17,-12,
            70,98,-65,-62,-35,-96,-45,66,91,-67,
            -25,-11,-75,-121,122,82,-4,85,-44,7,
            30,63,34,14,-36,-87,58,-72,23,-15,
            116,-33,79,-22,4,-108,17,-108,12,-12,
            -83,-20,-30,-126,-64,60,35,-99,112,15,
            5,-10,60,-53,51,-49,-24,121,-64,-36,
            98,125,-1,-77,64,-30,-54,-43,-81,-44,
            4,-38,-31,51,2,-40,99,-75,42,-125,
            113,-87,102,99,48,-82,22,-102,1,-84,
            114,-14,24,-96,-3,124,-65,-44,62,107,
            64,54,44,-77,-26,115,76,23,-23,-26,
            57,-97,-68,92,-78,-100,98,116,-101,2,
            116,56,-87,11,80,-70,-104,64,119,-86,
            6,-18,-35,98,-69,-94,93,-96,-95,-19,
            21,-68,-6,77,94,7,9,61,-59,95,
            -59,23,64,-55,-23,-42,82,-22,19,15,
            46,0,-58,-46,31,2,10,-31,25,-40,
            -37,-66,16,80,37,89,-16,122,96,102,
            -36,-42,-123,-27,26,123,30,58,-102,5,
            111,2,89,5,47,53,0,104,125,114,
            117,-96,75,60,-35,-81,58,-43,6,18,
            87,-8,32,30,-9,-24,-74,10,-88,26,
            -33,-18,116,-126,23,-48,15,-63,-85,-127,
            -66,-112,39,-110,-25,-95,-85,-39,-91,-14,
            116,-13,-100,79,94,46,89,78,-66,-66,
            -67,47,80,69,-98,-18,-98,-12,112,-96,
            34,15,-50,2,57,72,8,40,-15,-71,
            71,119,91,-67,119,-52,-21,48,-69,46,
            111,81,-7,126,67,-115,-123,109,-14,-55,
            -63,-42,-107,-112,-43,-43,-19,-120,5,-86,
            -57,127,-113,86,95,2,51,67,-107,-64,
            -20,2,10,-61,61,84,64,41,96,-122,
            86,14,-28,-6,15,23,-19,0,42,46,
            -46,93,28,1,20,44,70,107,60,94,
            -2,-31,64,-96,-14,98,59,-21,9,5,
            -35,-49,127,-33,-94,69,-125,121,-36,32,
            96,70,-98,-13,37,-54,-63,-53,37,-53,
            -55,-53,-67,99,42,80,93,-111,-18,-42,
            45,-96,63,40,31,97,-67,3,91,-38,
            -38,-82,-100,-121,-77,79,111,-14,60,81,
            -43,-28,-117,0,-97,51,-48,45,69,36,
            -44,114,10,-72,-36,60,31,125,121,-4,
            -31,-77,43,-128,2,-28,-72,-65,54,-80,
            116,89,-64,66,-112,-121,-118,-32,-9,2,
            -83,52,100,13,26,90,15,4,-86,46,
            110,-127,23,-21,3,-19,-14,13,1,86,
            -82,112,111,96,1,9,-117,-128,-60,-93,
            -91,-113,91,-108,3,106,40,43,-22,6,
            -104,103,126,94,-69,54,91,-66,-99,117,
            32,-85,92,-49,110,0,10,22,-11,-64,
            -29,94,45,71,-125,46,-17,-31,-66,-27,
            -96,75,113,-37,-67,-89,-45,-67,78,84,
            -59,35,-121,77,-78,-127,-90,-126,119,99,
            -21,12,-8,126,-81,72,112,-27,116,-48,
            -27,-109,-124,-60,40,-96,61,21,-65,0,
            26,35,43,100,53,-80,-44,95,12,10,
            114,25,37,57,42,-64,29,125,124,121,
            -96,-35,-55,117,-128,-82,-118,103,100,-113,
            -114,2,-102,82,-13,20,-48,36,113,-127,
            37,52,1,74,8,-72,14,-12,-93,-20,
            -38,52,48,27,62,-73,81,-90,45,-97,
            -10,114,84,-36,3,52,38,-79,21,-48,
            94,81,15,43,-25,-127,126,66,-18,55,
            31,-66,-21,-22,-73,3,-100,39,108,-11,
            -67,-4,-73,62,52,-11,-78,10,-96,-55,
            -9,64,-50,0,-24,40,63,-46,8,113,
            -127,-2,-22,-123,21,64,47,-118,12,126,
            -67,22,40,-94,80,83,32,121,-27,57,
            14,2,-69,-98,-29,4,-48,30,-41,3,
            -77,43,-15,4,-71,-66,-33,-50,69,64,
            125,68,122,-35,27,-126,113,-85,-9,77,
            32,-89,88,-65,-19,5,-96,-2,124,-65,
            17,106,43,48,-113,-13,24,48,61,-73,
            43,56,-89,124,-117,114,-15,114,-118,105,
            0,-69,-80,10,40,38,36,13,-12,37,
            -14,-118,-85,1,79,111,-50,-2,2,82,
            -51,-108,-11,-4,27,-125,120,37,92,-55,
            -47,11,-44,22,90,27,48,27,106,34,
            44,-105,-17,125,-102,-97,-123,-52,84,-3,
            59,96,-67,69,-122,55,-6,0,5,88,
            47,50,89,-104,-29,0,-104,5,94,10,
            89,19,60,-27,36,-80,-97,-43,9,-64,
            110,104,25,-64,50,-84,-9,78,-26,-60,
            117,48,-56,10,59,62,-29,50,80,-108,
            72,103,-58,53,48,126,-102,61,5,-116,
            -13,98,125,-36,44,-96,39,-7,126,103,
            -67,-33,2,-77,-62,-118,-125,-39,-123,122,
            -98,110,-77,12,117,44,-104,19,-21,31,
            -127,-3,44,27,-122,-25,83,-26,-101,-105,
            67,-108,75,62,-23,-80,-54,-35,-89,-7,
            121,-56,-68,44,27,102,46,44,125,64,
            -13,7,-77,93,-38,-127,26,-61,-21,-75,
            124,14,-81,-34,127,107,67,57,108,-49,
            -118,-126,-21,59,-121,-61,-87,112,-111,-95,
            -127,26,-24,-27,-60,9,44,10,-80,92,
            1,99,43,-33,-82,-116,90,22,-52,10,
            -78,-98,-55,-91,-37,-50,96,-110,-93,63,
            -85,-117,57,-81,-23,-64,-8,-61,91,-42,
            114,-12,86,48,14,-117,-29,-1,-17,32,
            48,-66,-7,-84,17,24,-89,-59,-6,81,
            99,-64,-8,-124,-17,-41,90,115,1,91,
            -56,-113,123,-50,54,-54,36,57,10,-67,
            14,44,93,45,3,-14,-9,50,-94,28,
            -74,-89,-33,-37,7,22,2,-35,79,54,
            -52,107,112,34,-62,90,14,-4,-46,54,
            90,-69,-17,-65,-43,48,-65,-4,6,-63,
            51,-22,81,23,-55,-27,42,80,99,-77,
            118,25,110,-54,87,-49,111,-6,-127,126,
            -126,23,32,-119,-57,39,87,85,5,-61,
            -19,12,7,86,69,60,36,-11,-69,10,
            -84,116,-51,24,-96,-73,-5,-52,5,106,
            52,-93,39,80,-121,45,47,3,-115,62,
            -44,10,104,73,-46,13,96,-30,-67,15,
            -69,56,12,-56,43,-66,50,-56,-41,12,
            44,-39,23,88,-119,67,15,3,45,-33,
            -14,8,-48,-28,-23,-89,-127,-102,-9,102,
            64,-111,53,46,0,125,-32,-73,6,24,
            -81,64,118,66,-28,115,85,53,48,-60,
            -96,72,76,64,-109,-34,12,4,93,126,
            122,-84,93,-123,-76,-25,-107,80,8,-49,
            -6,-90,34,24,-44,105,-71,124,-37,-15,
            -69,-125,-65,-4,-85,11,104,31,40,19,
            60,20,71,56,68,-111,54,10,-36,124,
            34,72,-45,98,91,-125,126,-112,23,72,
            -52,95,82,118,-43,2,106,-74,48,8,
            -88,101,-4,104,-96,68,54,-40,99,58,
            -69,-27,97,-80,-13,30,34,24,-14,-81,
            -61,30,-69,101,23,-45,28,-116,-22,122,
            119,48,-28,-16,-97,125,-21,-15,-91,-19,
            -116,55,-40,57,15,-58,50,61,12,-42,
            -47,99,78,124,17,-96,-53,11,14,2,
            53,-37,85,27,40,82,-28,91,-108,35,
            -10,105,-48,63,-29,-27,-36,-92,-67,6,
            -6,-1,56,-22,65,-53,-26,-54,68,15,
            -59,122,66,-95,-51,-50,-79,30,-1,-96,
            80,88,-2,-75,29,-100,-91,84,95,48,
            27,-22,16,-76,127,69,-101,6,-18,7,
            -8,-103,53,126,-36,90,112,95,-31,5,
            -100,78,-101,-64,-13,-114,18,-28,-69,73,
            122,29,88,-36,-7,-121,-128,126,-38,58,
            6,104,92,-36,40,-96,39,7,29,4,
            -118,-20,56,5,-88,107,-117,33,64,83,
            35,86,0,-7,71,44,5,-14,109,49,
            16,104,90,-57,47,-127,98,94,-84,10,
            -44,62,-18,6,80,-44,54,13,104,-20,
            -7,-91,64,103,-23,3,96,-79,-78,9,
            103,123,-72,100,62,121,-66,-57,109,6,
            119,2,47,-41,124,45,14,-12,29,-42,
            71,45,-19,106,-88,-2,96,-2,-76,3,
            -100,-127,57,-42,-37,-97,-2,61,-100,11,
            -100,17,90,113,48,-57,-117,-91,-31,-55,
            14,-50,71,-32,-10,57,-47,5,12,-6,
            14,12,-65,-12,105,64,7,63,-34,11,
            -76,120,72,81,-96,39,-62,7,2,45,
            -13,-83,2,-84,44,-33,111,-90,-46,1,
            -40,37,-37,-80,-40,62,-63,-68,-56,-29,
            120,-37,122,67,-18,103,29,71,-4,-2,
            -114,-17,112,96,33,-31,55,-127,-26,13,
            89,7,-44,126,103,113,-96,-41,-45,-25,
            0,125,54,-24,24,24,-66,-14,30,99,
            -107,107,-75,90,14,-94,22,107,37,-63,
            -4,73,7,103,-125,-4,-14,125,-100,-29,
            23,-33,123,121,30,-41,59,47,91,55,
            87,115,-125,-15,16,-75,88,125,14,-52,
            10,-14,124,52,65,25,65,13,-128,54,
            -88,-3,-128,-119,97,-69,-117,-65,103,-111,
            55,109,53,12,-52,10,25,12,-116,-76,
            103,64,-2,-18,42,-74,10,-24,-40,-62,
            15,-127,2,-26,-113,0,-14,11,-24,10,
            36,95,48,106,3,-128,-111,115,0,-104,
            -57,-83,4,119,12,14,-54,-127,92,-42,
            -83,-49,-71,-40,-56,-96,-90,64,31,90,
            79,54,40,67,-108,-61,49,1,-94,38,
            -118,114,-14,114,-57,-25,112,-113,-7,-53,
            -65,56,117,100,31,-49,-85,114,-68,-17,
            108,14,-115,-68,-27,-60,-82,55,24,15,
            22,47,2,116,90,78,-28,110,-125,57,
            -36,-35,10,-26,112,-41,-45,69,-104,-61,
            -35,109,-112,53,-54,-13,57,11,108,104,
            -65,14,64,-47,-15,-66,64,123,120,23,
            -108,38,30,-118,-98,-21,14,-76,-15,-71,
            -34,64,-35,-8,43,-17,25,-14,-54,-78,
            -66,-14,-55,80,118,-127,57,-68,-33,9,
            44,77,77,7,-106,33,-74,43,-18,15,
            116,-70,98,47,48,-54,-56,43,-88,25,
            52,-78,117,97,-22,71,-74,121,77,-66,
            -5,20,-104,103,72,-69,46,63,11,-70,
            8,-3,-33,21,5,-101,-44,22,92,81,
            -41,18,-127,-58,-73,43,4,-12,-68,53,
            95,96,114,98,-89,-60,0,115,59,62,
            7,38,63,82,-87,-8,6,-48,-61,-25,
            82,-128,124,-59,-51,-63,-107,14,-52,-19,
            -70,1,89,-9,-76,-8,-7,64,62,-107,
            -98,-127,-84,-121,-71,-114,35,96,30,-105,
            -89,-109,125,-34,-62,-106,-76,-13,1,-22,
            126,45,1,104,-36,-92,-114,-32,106,32,
            -74,-13,-69,0,-3,-33,-77,61,-78,-71,
            -98,-33,-65,-47,-26,103,-114,-106,32,87,
            -84,-123,119,-60,60,-124,-83,62,12,-82,
            -25,69,5,-34,82,-128,-94,-102,60,4,
            -122,28,-90,107,-95,-64,18,-108,-35,-64,
            82,29,-61,-128,37,-55,111,29,110,0,
            43,55,-42,5,70,-94,107,25,-80,-121,
            -12,-73,-128,61,50,-15,109,48,-110,124,
            51,-127,21,-107,13,51,5,88,-78,56,
            -82,76,-121,-1,-34,-92,50,24,-109,121,
            -66,90,-120,124,-82,62,14,-82,78,50,
            -3,53,-16,-50,30,91,3,37,-4,-34,
            39,3,127,116,-32,-97,-70,-86,-95,-39,
            87,-17,-108,79,115,-113,-98,6,-67,-19,
            -102,-99,64,-21,-84,-41,8,44,88,86,
            96,48,48,93,-66,-47,-76,-65,-39,-116,
            0,70,-114,103,32,107,125,-11,7,-127,
            78,-123,123,1,29,-109,-57,123,20,-52,
            -19,30,-128,-84,-119,-88,28,52,-16,-29,
            -53,116,-59,-21,-115,-2,-64,-126,-42,108,
            7,-38,116,52,30,-12,-90,-39,7,31,
            59,-85,-40,-70,-72,-48,-20,-11,-112,-1,
            -126,-68,-9,-108,40,14,94,125,-83,-91,
            51,-119,-2,51,-128,-79,-122,53,-63,53,
            -114,-49,31,30,117,-52,3,90,-31,29,
            13,-44,-37,-85,21,80,95,-81,126,64,
            -3,-68,-6,-128,25,15,2,51,110,0,
            20,-29,-35,29,-88,-101,-17,5,-96,23,
            -83,10,-90,37,74,34,-48,18,-33,19,
            64,67,-68,-89,-126,121,15,-86,3,20,
            35,-113,35,-114,43,-46,-31,-23,-118,124,
            -120,124,-15,124,-122,53,-84,14,-82,81,
            -94,28,-68,92,41,37,-118,-128,87,-65,
            -100,-22,33,-65,-35,-126,54,-56,-119,115,
            8,-36,-65,-34,90,-54,-24,82,-67,26,
            48,118,121,31,24,-111,41,-43,-128,70,
            -92,68,2,61,-106,28,6,-12,68,-14,
            96,48,-29,24,48,-29,126,112,71,44,
            -42,15,2,51,110,4,-44,52,101,23,
            80,-35,-108,-81,-128,-22,38,71,2,53,
            -107,-37,-27,118,60,-111,46,-49,-121,-56,
            87,74,85,-96,17,-105,15,-128,17,89,
            -67,22,-120,6,-54,-24,18,92,6,-18,
            -33,-112,83,61,-28,-77,-96,38,-53,-81,
            -127,28,-48,88,62,-11,126,106,63,100,
            -66,-100,-7,25,-24,113,-105,-121,-126,-5,
            -119,-60,8,112,-121,37,118,6,119,-11,
            -60,104,-8,11,99,-111,15,-98,-81,-53,
            67,-64,-3,68,-26,23,-96,-57,61,117,
            20,50,95,-111,95,-25,-88,-48,120,114,
            78,-11,-112,-49,-126,51,107,-12,-30,-7,
            40,99,-32,52,121,-45,47,15,-18,96,
            -21,19,88,115,-98,113,19,-52,-11,-98,
            -119,-83,25,7,67,62,-118,121,-66,100,
            62,121,-66,-75,10,-32,-106,-9,76,94,
            -50,-87,57,-42,67,62,27,94,-121,103,
            95,61,-76,14,-97,-67,-14,-121,-122,95,
            -14,123,-49,62,62,-1,-39,-57,95,-96,
            -19,59,-98,-49,99,-111,-49,83,-68,28,
            99,-77,-105,107,-88,-19,-97,-11,-53,122,
            -56,-73,-61,109,-15,47,-40,118,-37,-65,
            -25,82,-2,127,4,123,57,118,-37,-54,
            -99,111,-125,125,-4,-49,-1,-11,-75,-125,
            -113,-26,28,-4,31,13,58,-38,-2,-51,
            -29,-88,-20,-27,18,-27,-52,-79,30,-14,
            91,87,39,-125,120,-63,39,-66,-96,-28,
            -1,-79,-124,-46,-4,111,30,55,-77,-107,
            -85,106,46,-11,80,16,10,66,65,-8,
            75,-62,-1,1,-118,-99,-1,-37,29,-110,
            -41,102,0,0,0,90,122,84,88,116,
            67,111,109,109,101,110,116,0,0,120,
            -38,115,73,44,73,-75,82,8,79,77,
            -47,81,48,50,87,112,44,77,87,48,
            50,48,-80,80,48,-76,-76,50,-78,-80,
            50,49,81,8,112,9,-31,114,44,45,
            -55,-56,47,-78,82,112,45,-54,76,86,
            112,73,85,-16,45,-51,75,81,-80,73,
            77,76,113,-56,-84,-56,76,-52,-45,75,
            -50,-49,-75,-29,2,0,-31,23,21,24,
            123,-69,10,-77,0,0,0,0,73,69,
            78,68,-82,66,96,-126
        };
      }

    private static byte[] blackKnight() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,14,34,
            73,68,65,84,120,-38,-19,-100,121,124,
            21,69,18,-57,-25,8,-121,-31,-120,-56,
            37,-73,-8,65,84,-114,-128,-56,125,9,
            2,-118,2,10,42,34,74,-72,36,10,
            42,-126,11,120,44,-56,-94,-94,18,5,
            5,15,92,20,-15,2,92,84,14,1,
            23,81,17,36,7,-70,40,-62,10,-82,
            32,-62,-54,-115,28,66,32,36,36,111,
            122,-10,-3,50,-65,-98,-57,20,100,-77,
            -8,33,-121,110,-6,-5,71,-91,-5,-51,
            76,-34,116,117,87,87,87,117,98,24,
            69,-91,-88,20,-107,-94,34,75,9,81,
            -1,-36,19,-26,48,-47,94,-70,-88,-85,
            -14,-73,88,-94,-34,-122,-118,89,-62,-6,
            49,-54,78,-71,-36,87,84,-14,-88,-108,
            -92,28,20,108,-114,-71,-39,-109,-75,98,
            -59,-11,-69,-60,125,69,37,-113,74,-87,
            96,53,-54,14,-42,-89,126,-20,-55,-59,
            -57,-59,-60,-7,-108,63,84,-89,52,-117,
            -70,-14,-36,-106,104,-95,-104,-41,-126,-11,
            -82,-75,61,-23,-70,-98,76,79,-16,100,
            93,97,-30,-52,-81,115,88,-69,-118,-54,
            89,-106,40,81,-17,-50,-26,-113,-125,-51,
            -43,56,35,118,-105,15,42,72,-53,103,
            18,-126,-41,-37,111,-15,-121,-30,69,51,
            -23,-20,-118,-18,-88,-13,-124,105,-86,-56,
            -114,125,61,-40,126,97,107,79,38,13,
            12,42,36,-21,-53,96,125,-17,-49,-98,
            44,-33,72,60,119,126,14,107,90,81,
            17,-91,-72,-88,79,-89,66,-90,81,111,
            -107,-125,31,-57,-19,-12,-28,-31,-2,65,
            69,56,127,11,-42,67,-49,6,-21,35,
            110,21,19,-76,27,127,79,63,54,12,
            17,-33,-93,-40,-1,-5,76,17,107,-118,
            -43,-105,29,86,63,-40,94,115,-116,39,
            -105,126,46,20,-78,-111,-14,85,97,-38,
            14,80,62,16,94,101,-62,-24,-10,-76,
            44,115,60,72,-24,85,-91,19,40,119,
            60,102,36,8,-1,-26,-87,-64,-66,-40,
            -88,8,-52,-44,-1,-18,-108,-4,-15,21,
            35,-42,22,75,-69,-65,35,121,-39,8,
            79,14,-25,-52,73,-99,25,-66,51,76,
            -72,-93,75,-126,-48,0,-81,93,13,23,
            -118,73,-93,84,-31,-33,16,-58,111,-33,
            98,-116,7,-50,30,115,38,-48,-19,-65,
            60,93,-94,51,24,92,77,124,-51,58,
            -10,44,96,-91,-27,-80,38,-2,97,-41,
            44,-31,53,89,71,-126,-11,-53,-71,-45,
            95,115,71,120,-49,31,-58,-97,41,-45,
            -52,84,16,-38,20,85,25,-72,105,-26,
            100,-32,102,82,1,-114,81,22,-100,-30,
            28,-60,0,-9,-120,-15,42,-48,-19,42,
            -79,-47,122,-112,-15,70,-21,-69,-128,-45,
            85,95,111,-74,4,-97,-60,95,-40,26,
            84,55,-54,28,0,-31,111,-8,17,-80,
            -65,-73,-10,3,-93,-14,31,109,-61,43,
            -67,-80,107,105,-38,-5,5,-101,123,14,
            -12,102,-56,-119,53,97,-9,32,-116,-65,
            -122,84,-80,-38,3,53,-37,56,8,124,
            5,-84,54,59,3,55,100,-38,-64,-51,
            -16,21,19,-72,63,44,-5,-128,-80,34,
            27,3,-65,-3,-40,-48,-41,64,86,-4,
            -68,63,-127,-109,115,-85,-36,3,-12,-25,
            7,-97,40,-45,19,116,-39,39,94,-89,
            98,-123,97,-64,-88,-110,-125,101,-8,-67,
            -18,87,-52,109,124,-63,39,-125,-19,13,
            55,82,49,59,-126,107,69,-88,-89,-19,
            -126,72,71,95,-103,10,-44,-60,-71,113,
            -64,113,46,-114,1,106,-91,63,3,38,
            -126,83,20,-91,77,-94,-82,95,14,-36,
            12,43,14,-8,-90,-16,-28,-35,19,-128,
            122,48,-21,69,-112,57,-81,-25,-67,64,
            77,-32,-25,-109,-50,-101,9,6,63,45,
            -36,-12,-99,-65,55,-17,79,-50,-108,-50,
            124,-111,-95,84,-48,-93,-63,-113,111,-113,
            -29,72,-3,-38,-17,64,11,56,111,91,
            81,32,-46,-79,45,-42,3,53,-24,64,
            15,-96,-66,113,89,-66,125,10,-88,59,
            -93,71,1,117,-44,127,78,26,112,79,
            -8,117,27,-100,-94,-88,107,-127,123,50,
            -22,8,-16,77,-28,-113,-61,-82,1,-22,
            73,-17,-23,-95,-99,-93,111,6,89,95,
            -23,-5,74,44,2,67,-70,26,-49,102,
            99,88,-65,2,-5,-51,-62,-18,76,-56,
            -103,-78,-97,-118,121,87,-20,87,-66,-15,
            -28,-62,-30,-63,69,93,69,25,-25,3,
            -107,102,101,23,-65,35,51,122,62,15,
            -100,5,-103,-113,1,117,-109,86,76,-42,
            19,-64,-99,-54,-22,95,38,-11,2,-50,
            94,-34,119,-60,94,8,-62,-49,41,14,
            -62,-50,66,78,51,-22,50,-32,102,68,
            -83,4,126,-5,-93,-45,-106,3,117,-117,
            -9,120,21,61,116,34,8,-43,-30,-25,
            43,-93,19,64,-41,80,-40,33,31,-110,
            -19,-108,-65,7,-84,-125,-123,101,109,-110,
            51,-27,122,42,-28,62,42,40,62,-8,
            113,63,-122,88,14,-115,18,-5,-109,46,
            84,-48,0,-93,54,8,-73,95,13,-4,
            -114,-102,-33,97,31,112,102,117,95,14,
            -100,80,-105,113,-64,-99,-48,-75,63,80,
            -73,117,-1,22,56,75,-102,91,64,-91,
            -13,-66,125,70,6,8,63,-89,24,56,
            69,65,57,40,-54,-84,1,-36,99,-70,
            94,-90,57,80,43,-74,-57,0,-75,-63,
            83,-108,83,-91,97,18,-48,-9,-19,25,
            85,-31,101,80,-75,-100,24,-96,125,10,
            -54,-28,-55,-103,-62,23,-78,-33,14,-74,
            87,-95,87,-74,-24,43,-79,95,97,71,
            -123,-12,-102,-15,111,-47,113,-95,51,-121,
            104,114,-107,90,1,89,-71,92,-105,46,
            102,-18,9,-102,-66,55,117,-69,-99,10,
            34,-41,-35,-75,6,56,91,-12,-52,125,
            127,14,-56,-84,-23,95,-65,1,-68,41,
            102,-118,53,-109,63,92,-100,95,-50,-125,
            8,-71,-104,-115,-68,23,51,-29,-126,-19,
            -3,43,25,-109,-63,-31,-5,-116,-107,64,
            -81,41,33,-50,32,53,68,-116,-36,-76,
            -45,70,116,-10,-11,17,-123,89,-73,1,
            -9,23,-69,27,112,15,-39,-51,-128,-101,
            101,55,2,110,-86,61,6,-72,-121,-84,
            122,-32,-108,-3,79,41,-112,-93,-94,28,
            126,31,-83,-120,97,-38,-21,51,107,3,
            -9,-80,-66,-66,-30,-109,64,-67,119,-92,
            1,80,25,-103,101,-128,-6,-82,102,83,
            -32,-48,25,81,-27,74,-10,1,77,-42,
            8,69,117,-52,-81,72,4,71,-128,-39,
            42,50,73,-128,-34,-41,45,126,-58,-72,
            21,-124,59,96,1,112,-46,-116,-47,32,
            -108,97,-18,6,-18,14,-65,-125,-68,-114,
            -5,-83,51,-27,-73,74,-19,4,92,21,
            108,-33,-3,16,-93,-35,101,-4,-10,-6,
            -64,-3,-56,-22,8,-4,-5,71,-3,-3,
            69,-96,-6,113,34,29,25,24,15,50,
            -5,-22,-5,-118,119,3,-45,111,19,94,
            -98,14,25,93,-112,-57,-6,-79,46,13,
            -42,-21,45,-9,70,-4,-81,-19,-116,26,
            -64,95,83,122,68,-59,3,39,-87,-60,
            7,-64,-67,-47,55,101,-47,-64,-97,33,
            42,-97,21,68,83,28,18,38,117,-28,
            -113,-98,-4,-94,-108,-33,-98,29,-95,112,
            86,-46,-44,-23,-5,-73,39,44,4,-86,
            -82,54,117,111,28,0,-95,-9,124,83,
            -9,29,-8,-7,83,-77,15,40,117,-73,
            -24,-64,3,121,-84,32,91,36,-62,58,
            -82,49,122,-125,-16,11,-84,2,-103,-38,
            -76,116,89,60,26,-88,-40,-109,-47,64,
            -99,-20,25,15,84,-116,126,17,107,45,
            8,-113,104,-23,77,-27,-115,-44,107,82,
            -100,-120,114,47,-16,100,85,-50,-128,69,
            -47,65,83,27,-6,-54,62,8,-36,-109,
            -70,61,126,31,80,-51,125,-9,-66,63,
            80,55,112,109,-99,-25,59,27,91,65,
            -121,-114,98,-128,-1,-100,-57,107,-111,53,
            35,88,111,-74,-33,-116,3,126,71,116,
            -85,84,21,-88,-66,59,-26,0,53,56,
            -85,18,80,110,-85,-119,64,-7,-90,-64,
            -22,6,-62,-117,112,-2,40,-120,29,28,
            58,17,108,95,-70,42,-8,62,-21,-102,
            11,103,102,-84,125,29,-120,120,117,-41,
            -73,1,-22,6,-22,-25,-121,67,-81,0,
            -107,94,-95,38,-56,-36,-27,-65,-33,44,
            48,-10,45,-31,-12,-106,-54,99,-73,-37,
            -70,57,88,-113,-99,-54,-3,-123,107,-76,
            6,46,77,-123,91,58,122,28,80,15,
            87,-65,22,-88,-113,125,-73,-73,31,-16,
            55,-116,42,127,76,-102,-114,102,59,-91,
            -126,-97,119,-26,-56,47,-74,-114,-23,-118,
            -113,-123,119,-9,-119,-11,14,112,83,117,
            123,-13,116,-96,30,-89,-126,102,-86,68,
            -32,-10,-67,-68,45,-56,106,-23,-101,-70,
            11,-64,-126,19,-94,-1,-10,-6,43,119,
            30,41,72,-104,-72,-70,83,-67,25,-96,
            62,-28,11,109,53,-86,1,-121,-57,-101,
            -44,12,35,17,-88,116,115,1,56,-59,
            -51,13,4,63,-49,-103,12,-99,57,-102,
            -99,89,55,-40,-66,-92,119,-16,61,122,
            -76,20,51,71,15,-76,-81,-83,88,16,
            81,80,-67,109,64,45,-94,-126,30,-92,
            -100,-43,114,10,8,13,-14,21,-44,14,
            108,44,-98,67,71,-114,-53,43,39,110,
            -79,-56,-53,108,101,-20,-84,77,-48,45,
            -50,104,102,108,5,105,7,24,101,-42,
            29,54,-21,-76,80,75,-64,-35,-11,-21,
            15,80,-63,-52,-112,58,-116,68,56,-33,
            -79,-99,47,-24,-10,-94,124,-118,-19,12,
            21,-123,-104,-90,-56,-6,41,-8,-36,-97,
            -17,-15,100,-115,23,-126,-17,-15,-39,122,
            -95,-96,14,122,-19,50,21,-120,40,-88,
            -18,-123,64,125,20,-38,12,-36,62,84,
            -48,-122,-85,38,1,103,-70,-81,-96,43,
            -64,65,-82,105,49,34,77,97,86,-49,
            43,5,-115,8,-42,43,49,-108,-109,-66,
            -119,-79,-76,-123,-98,-20,84,-42,-109,77,
            62,-16,-28,-47,-99,-62,116,108,22,10,
            -7,-124,29,-53,-114,115,27,-117,-49,79,
            -28,-78,49,-43,-110,38,68,109,9,94,
            -105,50,-42,-109,117,68,-122,116,-30,-69,
            66,49,-70,-125,127,-47,3,-57,58,15,
            -72,126,108,-17,-46,22,64,-51,112,-110,
            -127,-37,-101,10,90,-46,118,0,112,-2,
            -27,-81,65,123,65,38,-99,-127,-117,100,
            -124,-31,-117,-68,82,-112,-56,-33,84,98,
            74,121,38,15,103,84,-103,27,-4,-68,
            85,19,79,-18,123,68,40,104,22,-21,
            60,6,-91,98,-123,119,53,-47,-109,-37,
            -71,-72,-81,-89,-126,55,113,-28,-19,101,
            72,-55,-87,25,-68,-17,40,15,42,46,
            -25,126,-26,-6,27,-59,34,-51,-61,34,
            47,-20,16,-118,-71,-105,-11,-61,50,-46,
            96,-11,6,17,5,-59,38,1,-11,28,
            21,-13,40,-27,-77,77,-70,1,117,-103,
            127,-33,40,-96,-97,-33,-80,-86,88,42,
            58,-25,-107,-97,45,98,74,102,-118,8,
            -34,114,63,-16,74,59,126,81,61,34,
            107,80,-58,7,23,109,-11,114,-80,-93,
            94,-90,-119,-86,67,69,20,27,41,94,
            -116,-103,-45,18,55,121,-14,124,102,80,
            43,115,-93,105,-67,120,-26,69,-71,23,
            103,-8,-58,27,69,12,80,15,-108,79,
            115,50,-71,-42,78,16,49,113,-19,-110,
            -127,26,78,-59,-52,119,-94,-128,123,103,
            -35,91,-127,98,-76,-34,-19,106,-90,1,
            -3,-68,-90,-103,-30,123,77,-53,-21,-120,
            2,71,-88,41,70,-24,69,21,60,-71,
            118,-69,-104,49,107,-39,33,60,67,-112,
            -71,63,-8,-7,20,113,70,-38,-94,98,
            13,-122,40,-94,68,80,-10,2,-18,95,
            -6,49,-24,26,-57,25,58,-122,-89,116,
            -34,-89,-119,-35,-33,80,116,60,19,-124,
            -50,21,65,-45,122,-38,126,-55,55,-91,
            -10,-29,64,-49,116,-41,-19,-11,48,80,
            3,-87,-96,-67,-57,-42,3,-73,126,-43,
            -9,-128,122,-122,-41,69,123,103,29,-12,
            115,90,-36,33,-34,-17,-35,60,14,-11,
            -8,69,79,93,-67,54,77,22,-34,17,
            15,119,124,-97,121,-26,53,100,-41,10,
            118,-8,63,-125,-9,21,27,-52,23,89,
            46,-38,89,95,38,-46,20,-50,115,-94,
            -125,-45,-60,-25,52,-75,-50,-97,-59,-9,
            -56,16,107,-40,105,-34,-95,125,20,68,
            18,127,67,-54,0,-43,93,111,84,-9,
            -9,7,-22,80,-52,6,-96,-72,79,116,
            -17,52,-53,3,-3,-100,43,127,17,10,
            122,58,-65,-94,-38,98,-28,-101,-9,-97,
            121,-124,20,-93,-105,53,-108,57,-1,-12,
            57,12,-79,60,33,-82,91,-54,-25,-76,
            -27,115,120,-116,-86,57,-37,83,38,5,
            59,48,-109,47,26,-94,-73,-108,-11,45,
            -21,60,78,-91,-24,13,-70,7,115,-15,
            26,115,-108,81,-11,64,-60,125,31,125,
            15,80,61,-76,-126,126,-86,8,84,98,
            -15,-20,-94,-26,-5,-9,47,3,-6,57,
            13,-54,11,5,-75,46,-24,-68,16,23,
            107,-101,65,67,83,5,63,-82,-64,122,
            -23,30,108,-40,21,-4,-68,26,-67,-65,
            53,-91,-123,-87,100,58,-61,89,38,58,
            -14,120,110,51,-31,-73,-54,-88,-90,32,
            18,108,-99,48,2,-88,-10,90,65,-21,
            70,3,-11,18,-65,-33,116,-93,57,8,
            95,127,31,56,-55,-17,85,107,-101,24,
            -56,43,11,58,-77,42,-49,56,-45,-108,
            -39,-38,4,-14,0,-95,-35,37,120,89,
            13,-58,-82,-74,-84,16,-117,57,-99,0,
            -25,-26,28,-36,-17,115,31,-111,96,8,
            42,-86,14,-16,-37,15,37,60,6,84,
            -68,86,-48,-118,86,64,113,45,84,-73,
            48,106,-17,26,-51,-64,97,14,-48,114,
            -30,92,-99,-39,-40,40,100,69,-20,-88,
            45,113,120,-67,-20,4,-70,-45,-125,-123,
            -69,77,83,-27,126,121,-114,18,122,103,
            -81,-96,10,-64,111,-33,-7,-100,1,84,
            103,-83,-96,121,-125,-128,-30,-64,113,54,
            -40,85,65,-8,-6,-14,-32,39,-90,-58,
            109,97,65,-116,57,-123,77,65,58,81,
            -59,-3,-114,85,54,-8,-15,92,17,106,
            -55,106,-106,67,76,-51,-51,55,-55,-56,
            72,84,52,-16,-37,83,-97,111,3,-36,
            86,90,65,47,22,3,-22,58,42,-88,
            -68,-99,2,-62,-41,103,31,-108,76,22,
            -90,-33,-44,33,-98,14,-123,116,6,-39,
            -62,-87,-24,-35,65,-104,-78,1,-62,11,
            59,-98,-17,-118,-111,51,-88,28,-16,-37,
            119,76,126,22,-88,88,-83,-96,113,119,
            0,69,-9,61,-108,16,-52,31,45,18,
            65,81,63,-108,89,-79,-80,41,-24,37,
            49,-110,120,70,97,-35,48,-31,14,47,
            -49,55,19,-10,-65,58,9,-105,-128,72,
            -38,-30,-79,54,-64,-9,-30,82,31,120,
            4,40,-102,-26,80,15,123,15,-48,10,
            -98,51,69,40,-24,49,-2,16,93,88,
            20,-61,125,-109,117,40,-40,-36,-23,71,
            -95,24,109,-62,-66,47,108,10,-78,63,
            5,-111,88,-32,-19,-79,64,-43,-41,49,
            -72,-15,-105,0,-99,-80,11,-35,100,111,
            3,-6,20,-47,60,113,98,-42,-42,81,
            -12,-78,-123,-51,-76,-3,16,108,126,110,
            -86,48,109,-115,115,9,-118,22,-104,100,
            -102,-60,95,3,-85,85,2,-22,77,42,
            -24,-30,-39,101,-127,98,44,47,52,-35,
            110,11,-62,-9,87,6,-97,-55,-77,-25,
            -116,108,24,13,11,-117,-126,-72,120,-102,
            45,-126,-51,-55,-77,-59,12,-22,119,-74,
            27,-56,124,-109,-75,-128,27,-78,54,3,
            -65,125,-34,-37,15,1,53,112,103,44,
            -48,-5,32,-57,15,-106,26,79,-128,-51,
            -9,27,59,-128,-75,71,-12,-53,-85,-123,
            69,65,71,69,48,-107,17,-127,61,83,
            -60,6,52,-95,-48,41,70,75,-98,17,
            55,-21,-125,-56,12,47,-65,20,-88,93,
            -69,-70,3,103,91,-89,17,-64,-39,-19,
            7,89,-109,-64,-55,43,-67,-45,75,-75,
            39,8,-53,-65,42,-121,-48,89,-66,47,
            65,-17,-120,-115,40,15,-16,-91,39,-119,
            -114,-72,-94,-48,42,40,120,62,-49,-75,
            122,-128,-56,33,-110,-85,54,1,-11,-41,
            -41,87,-128,-48,6,-102,-70,-123,-58,88,
            16,-66,-17,0,-24,-67,95,-84,69,67,
            -59,-10,-93,-96,-118,37,98,80,-105,-11,
            -28,11,112,102,57,-1,96,-99,27,81,
            -3,-9,57,-50,53,-108,-116,52,56,-36,
            -40,58,-116,52,56,60,-41,-26,-76,-89,
            100,-20,-50,-47,25,88,-98,-37,115,104,
            90,29,-18,-85,-100,-90,34,120,-54,-75,
            -49,97,20,-36,-31,-38,-32,-16,47,-6,
            -100,122,-66,-23,-54,-98,9,14,83,-25,
            78,29,107,38,112,42,-79,-98,-48,-84,
            53,-56,116,47,-101,5,78,60,-32,43,
            52,6,-52,-8,70,40,104,124,48,43,
            80,112,10,-70,58,88,111,-80,94,-116,
            -48,35,-108,-45,10,-19,12,-110,82,-100,
            -91,48,-105,1,-3,7,95,97,-81,-81,
            59,80,-111,4,101,25,-80,123,-74,55,
            19,-53,-118,-45,60,102,65,-1,43,26,
            83,-4,21,118,45,38,-14,82,-104,-71,
            92,67,91,-100,-62,127,-43,-110,-62,-88,
            115,-54,103,-108,-116,-35,-91,-16,60,91,
            10,71,120,50,-99,-119,100,-114,-52,100,
            -2,-98,100,-90,61,-110,57,-45,-110,-103,
            106,78,98,6,53,-119,-65,39,-119,81,
            -12,-92,91,40,47,-15,100,34,55,-56,
            -119,76,68,38,50,47,-107,-56,3,-120,
            -119,-116,120,36,-106,-12,14,96,-82,102,
            -22,126,-11,60,-85,49,88,61,-36,-8,
            16,-84,102,98,114,-43,37,-34,-87,-89,
            -75,13,24,-107,23,-50,-110,53,-56,40,
            42,5,99,58,68,-99,-79,58,115,-116,
            104,47,-16,13,-85,56,-90,-91,-1,-119,
            -111,-2,-77,12,83,127,113,30,104,52,
            -103,-1,49,-71,51,55,57,114,77,-18,
            51,-52,-111,-108,-93,41,105,74,76,-38,
            116,-13,113,74,-26,-97,76,102,56,77,
            38,-12,76,30,74,49,95,-95,-28,127,
            38,49,-33,8,58,53,38,-13,89,38,
            -45,30,-26,34,74,-99,-81,-30,-7,57,
            -109,51,93,31,2,49,-109,41,-71,-90,
            -102,52,-23,-2,-47,-128,-74,-71,-12,79,
            65,69,18,78,27,49,-99,-123,-20,36,
            -92,94,-69,58,10,-39,65,-56,-85,-124,
            108,47,100,59,33,-37,10,-39,70,-56,
            -42,66,-74,18,-78,-91,-112,45,-124,108,
            46,100,83,113,125,-59,92,-6,-89,-88,
            20,-107,-94,114,78,-53,127,0,85,-8,
            105,-125,-46,105,29,-35,0,0,0,90,
            122,84,88,116,67,111,109,109,101,110,
            116,0,0,120,-38,115,73,44,73,-75,
            82,8,79,77,-47,81,48,50,87,112,
            44,77,87,48,50,48,-80,80,48,-76,
            -76,50,-78,-80,50,49,81,8,112,9,
            -31,114,44,45,-55,-56,47,-78,82,112,
            45,-54,76,86,112,73,85,-16,45,-51,
            75,81,-80,73,77,76,113,-56,-84,-56,
            76,-52,-45,75,-50,-49,-75,-29,2,0,
            -31,23,21,24,123,-69,10,-77,0,0,
            0,0,73,69,78,68,-82,66,96,-126
        };
      }
    
    private static byte[] blackPawn() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,6,107,
            73,68,65,84,120,-38,-19,91,107,108,
            84,69,20,-34,71,-79,42,-75,96,-48,
            74,75,21,-88,-94,65,67,69,-76,4,
            -126,-14,-48,18,107,-79,6,-93,36,68,
            73,48,32,42,40,-46,4,-119,-47,106,
            -128,8,40,10,-95,10,81,8,26,-116,
            54,49,20,11,-92,53,81,67,-127,-96,
            -115,4,-37,-88,-75,17,-94,24,44,106,
            83,68,93,37,109,-79,-19,-34,-69,-41,
            31,-9,-101,77,-50,-105,-67,-113,46,4,
            118,-37,57,127,-66,61,-25,-50,-29,-52,
            57,119,102,-50,-100,59,27,8,104,-46,
            -92,73,-109,38,77,-102,52,105,-46,116,
            -95,40,72,-4,80,-32,-61,-64,-121,-128,
            67,60,-22,105,58,79,116,9,-15,-49,
            -38,16,126,3,-8,36,-28,71,-128,-115,
            -112,47,3,-82,-122,124,-111,71,-69,-102,
            -6,72,87,-48,4,-40,11,-20,-96,114,
            67,28,-22,-109,60,20,66,-3,42,42,
            -105,-91,77,-35,55,-70,-100,28,-125,-103,
            18,56,-116,25,81,100,-29,28,24,124,
            -41,20,27,-101,-98,1,62,106,99,117,
            37,86,-68,-103,112,-48,86,-76,-13,55,
            -38,93,-19,-34,-81,38,-113,-67,37,-8,
            -86,-28,-121,-105,-39,120,-88,-50,70,-53,
            -78,49,-122,114,-42,95,-64,-45,-112,-65,
            46,-53,29,28,105,99,78,53,-11,-13,
            -126,-34,-85,-4,-47,-96,-60,-10,25,-110,
            103,99,-13,32,105,-16,-24,109,54,26,
            -41,-37,104,110,0,98,-58,25,112,72,
            -76,72,-42,-5,-74,-64,-58,-20,-63,-44,
            -47,89,-87,-121,38,-90,92,44,69,-106,
            20,111,126,90,26,-72,-25,-76,-28,-29,
            104,18,42,121,15,-22,-3,42,-27,111,
            -27,-54,126,-62,-54,49,-61,-75,43,18,
            -81,112,-113,73,62,-25,35,27,35,21,
            100,112,-124,-43,86,-89,-125,-93,24,85,
            -71,-63,82,30,-71,9,-3,-36,71,122,
            -52,-45,-66,72,72,-31,60,-55,23,-97,
            -107,6,53,-21,-63,31,-13,-23,24,-123,
            49,89,-49,-36,47,-97,23,127,67,122,
            -28,106,95,36,118,-48,46,-55,-49,-67,
            78,26,-46,-88,-105,65,-128,111,7,89,
            -78,94,-68,29,-56,-25,14,119,-41,67,
            -109,50,-52,29,-110,47,-55,-91,25,-76,
            9,-4,79,73,58,-24,71,-76,83,41,
            -27,37,-39,-18,122,104,82,107,-1,22,
            -55,-113,-8,-45,-58,-114,107,-55,-48,56,
            -41,88,29,62,29,-45,65,-11,32,-17,
            -56,65,63,-65,-111,30,-101,-75,47,18,
            -45,13,-120,-30,62,-106,-30,29,-7,-76,
            -44,85,57,68,107,-106,123,116,23,-81,
            7,-7,-114,97,-108,105,-88,-111,122,104,
            114,-56,32,4,-57,80,52,-121,-28,103,
            -13,-125,116,64,-99,1,-34,112,112,-116,
            65,-27,32,111,-98,-123,118,-53,104,-26,
            -116,-47,25,5,127,-87,-99,-27,56,47,
            -26,75,-7,42,10,14,-116,54,-16,103,
            28,28,116,-122,-54,65,-66,-86,-115,-50,
            -57,-7,-78,95,-99,-6,81,68,73,-54,
            -48,-13,48,-44,1,41,-81,-52,-94,-103,
            -77,6,124,-109,-49,37,-82,-119,-22,65,
            94,121,25,-67,24,7,-92,30,78,122,
            -10,-29,40,0,-104,73,81,-45,82,-78,
            -57,45,54,-42,110,-96,40,14,-11,-84,
            -10,36,-93,-72,118,106,7,-14,-38,-11,
            -24,119,-84,-69,94,113,-67,-5,93,-82,
            46,68,60,-106,-108,12,10,103,71,29,
            -60,30,-79,87,26,48,-118,-84,-77,69,
            -101,-68,21,-11,-23,24,46,87,69,-19,
            -86,-67,105,55,-12,-40,47,-11,-118,-21,
            -103,-17,49,-82,-76,39,-107,12,-91,61,
            96,50,54,-19,-10,63,-92,-63,-70,111,
            -58,30,-126,-92,-89,17,75,28,4,88,
            49,-113,12,2,5,17,6,28,102,-116,
            -110,-3,-88,-25,-19,-104,105,-109,57,5,
            -44,38,-57,-47,127,72,69,75,17,41,
            46,63,74,6,-3,29,-72,-42,-61,-32,
            -79,36,-105,56,-81,-6,-86,95,10,42,
            -54,91,104,60,17,26,87,-38,-45,-41,
            -110,-51,-58,-9,-100,37,-9,-37,-72,8,
            111,-28,124,-92,118,22,44,-74,113,-31,
            -35,54,-50,43,-76,-79,-30,11,7,-61,
            126,-30,32,-81,75,44,-81,-40,-120,118,
            39,-95,-97,-23,-78,95,-91,-121,-46,107,
            9,102,120,118,-83,-5,-72,6,94,-22,
            103,-102,-28,75,31,-95,-96,-31,101,-16,
            -57,61,82,59,91,40,-75,-45,-96,83,
            59,-110,-8,114,-58,115,48,-52,-37,64,
            100,-83,-61,-83,64,-100,119,50,-113,-54,
            106,-21,-81,-94,-32,-95,-57,-33,-46,22,
            -19,-106,-4,-70,2,10,-50,-114,-53,126,
            -29,122,-100,-107,122,42,-67,29,-57,53,
            96,104,54,54,-27,-47,82,-36,-80,-113,
            102,-48,44,127,65,-126,57,94,-54,15,
            -67,70,-101,-65,-70,-19,83,-94,115,6,
            -18,-108,1,-125,125,38,-59,5,39,17,
            109,-27,-111,3,38,-6,12,14,70,75,
            -66,-21,23,27,71,-74,-110,-93,-74,75,
            61,52,49,93,-118,37,-91,70,-118,-105,
            78,-94,48,-7,69,-16,-1,-6,116,16,
            -94,46,-125,-10,-94,-59,119,-47,94,-12,
            -66,-44,67,19,-99,-113,-30,97,-85,33,
            31,127,69,119,9,-52,85,-55,29,84,
            -51,39,-92,-4,-53,58,-102,65,-9,-30,
            -57,61,122,38,37,76,-103,-124,-66,-109,
            -4,-116,41,-108,-125,59,1,-66,-70,-113,
            -25,33,85,-18,67,-32,48,-7,124,-22,
            15,-92,71,-67,-69,-98,3,-121,84,-74,
            120,2,12,-77,79,62,-2,-4,29,-102,
            57,120,-77,-83,-18,36,15,-86,-88,103,
            -46,-110,-7,-23,49,114,16,-62,-13,0,
            -50,95,-127,-63,3,-51,49,52,-32,65,
            35,36,-1,-8,28,114,-52,50,-54,52,
            68,-109,116,-112,-86,-121,37,-45,-92,-37,
            66,11,39,-110,94,-41,-72,-21,-35,127,
            40,-108,56,24,-56,-24,-107,-30,34,124,
            -65,-23,-102,74,-122,29,7,-4,47,73,
            -57,48,-86,118,-58,74,121,39,-50,65,
            69,107,40,-72,-20,114,56,-9,-92,125,
            -78,-108,-93,33,44,29,25,-17,74,-15,
            56,44,61,-89,102,-45,-98,51,-99,12,
            107,-100,39,7,81,59,49,-38,-21,78,
            -31,-123,24,-9,51,57,74,-23,-35,72,
            -29,-54,76,55,-57,-16,-121,48,100,-117,
            -61,-27,-108,-70,-39,105,99,-124,-106,44,
            -77,-111,12,-38,117,-98,28,99,-71,-73,
            107,30,-111,-4,63,8,14,74,-23,75,
            107,120,37,-58,-107,-29,62,-18,-108,61,
            104,-58,29,-125,-21,76,65,-36,-32,12,
            -67,98,-29,-90,-123,100,24,-68,-87,49,
            -70,102,-27,-5,-42,-50,-71,34,-11,19,
            -69,26,122,-99,-108,-14,-115,-8,59,76,
            16,47,90,-16,41,12,-76,-117,-20,16,
            78,85,7,-87,-87,-66,-101,82,86,-37,
            108,-84,-93,100,103,116,15,12,-78,-126,
            12,-42,121,-127,28,99,-71,-9,27,-61,
            -52,-119,86,75,-7,30,92,-42,-49,-96,
            -37,71,-127,109,-87,-66,-28,-107,-30,-51,
            90,43,-59,31,-48,-91,-114,-34,59,-63,
            -65,68,6,-22,-67,72,-114,-79,60,-12,
            64,-110,-76,-105,110,7,-67,55,-97,86,
            12,-11,-7,97,90,-86,6,107,52,-43,
            -117,35,-108,-94,81,127,3,-63,-27,-116,
            -8,-67,-74,88,-118,56,-58,-21,3,31,
            -10,76,99,-69,44,55,125,50,-39,-95,
            57,-43,60,-125,53,55,76,95,76,-33,
            28,79,14,10,56,24,-64,74,113,36,
            61,-93,-108,3,-36,120,59,-103,-61,72,
            -75,48,124,40,20,-69,81,-118,-85,-105,
            -53,-127,-12,-32,-118,-83,-119,-91,-48,124,
            32,-51,16,122,-9,-100,-112,-29,-38,-71,
            -110,28,-124,-52,-120,-29,127,103,47,56,
            93,9,-59,110,-107,-30,-102,9,52,-125,
            86,-92,-55,-116,-15,64,30,71,-51,76,
            114,80,-95,-76,-53,-59,-89,44,25,70,
            43,90,-121,-127,-76,96,-45,109,-8,-34,
            -58,-61,91,-45,27,27,-80,-57,-76,-32,
            -4,-74,-114,-18,-49,-59,-19,-96,-1,61,
            -82,-23,-100,14,-86,-127,86,-124,-97,11,
            -128,-8,-92,28,44,75,115,44,-111,-29,
            82,-29,116,-76,67,-54,16,127,55,81,
            107,-79,-70,0,88,12,-100,-103,-26,88,
            76,-29,42,-12,-80,-125,38,77,-102,46,
            10,-3,15,17,89,-99,64,66,121,-119,
            -43,0,0,0,90,122,84,88,116,67,
            111,109,109,101,110,116,0,0,120,-38,
            115,73,44,73,-75,82,8,79,77,-47,
            81,48,50,87,112,44,77,87,48,50,
            48,-80,80,48,-76,-76,50,-78,-80,50,
            49,81,8,112,9,-31,114,44,45,-55,
            -56,47,-78,82,112,45,-54,76,86,112,
            73,85,-16,45,-51,75,81,-80,73,77,
            76,113,-56,-84,-56,76,-52,-45,75,-50,
            -49,-75,-29,2,0,-31,23,21,24,123,
            -69,10,-77,0,0,0,0,73,69,78,
            68,-82,66,96,-126
        };
      }
    
    private static byte[] blackQueen() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,14,28,
            73,68,65,84,120,-38,-19,-101,123,-100,
            78,-43,26,-57,-33,-67,-33,121,-115,23,
            77,82,52,-88,73,-51,16,70,116,104,
            -70,-96,-36,101,-62,-92,-108,58,-110,46,
            10,49,35,-91,82,-93,-122,-54,-19,-112,
            -110,68,46,-57,-91,-48,81,-88,79,124,
            -24,-94,-114,91,-88,-112,114,31,73,20,
            -50,-104,-63,-72,12,99,102,-17,-75,-50,
            -2,-51,-5,-84,61,-42,99,94,-17,-28,
            56,-102,-103,-49,-84,-17,31,-49,-69,-98,
            -67,-9,-70,60,-21,-6,-84,-67,95,-113,
            -89,52,-108,-122,-46,80,26,74,67,105,
            40,13,8,6,-55,-6,36,-93,-126,92,
            -65,80,-7,120,88,62,-11,47,112,62,
            -59,54,-108,101,-15,-71,1,-31,-3,49,
            32,-51,-14,36,-9,-110,-66,41,-35,87,
            -122,61,-25,45,100,126,-4,-66,50,122,
            -70,42,31,-107,-81,119,61,-35,55,43,
            68,-71,75,92,8,99,-15,33,-44,97,
            -25,50,125,121,22,-113,-92,-5,126,-93,
            120,120,33,71,22,-41,-121,-77,116,34,
            -49,-99,-81,-15,17,-3,24,24,-94,30,
            37,38,-88,17,-112,-62,-44,-109,-55,14,
            15,4,-28,-54,38,1,-71,120,70,64,
            38,124,-61,12,-9,24,-3,-72,-124,-92,
            47,72,126,62,-3,62,-9,57,10,9,
            43,-12,124,86,-74,13,-56,23,78,82,
            -71,-38,-80,-12,-110,-125,-116,-28,18,19,
            -18,35,67,-75,-48,-43,-77,43,4,-92,
            -108,1,105,-5,3,82,52,-45,-11,73,
            -33,-23,-49,-103,-69,-125,-116,56,54,34,
            -52,95,117,117,-46,79,122,-70,-30,-114,
            -128,-76,-92,-82,127,127,50,-21,24,-3,
            -23,71,-57,18,-38,62,-26,41,61,-34,
            -70,-79,110,-112,92,-22,-23,-42,-20,-128,
            -52,-55,-46,-81,31,120,37,32,35,86,
            50,-61,93,17,100,-122,99,-6,-120,5,
            -108,78,-78,-98,-82,-54,-57,-102,74,-27,
            -24,-94,95,111,-2,1,-85,-57,-82,-110,
            -70,4,-59,-78,25,99,19,107,-96,12,
            -118,31,39,-71,-112,100,87,26,89,-1,
            8,-56,122,108,-60,-104,-73,4,-23,16,
            -51,-11,120,-67,-93,-108,78,54,-91,75,
            -41,-27,98,-110,39,-88,28,-23,122,-71,
            94,-52,98,-11,-88,83,66,27,-56,59,
            67,-113,63,-76,-121,53,-48,6,54,-11,
            -80,-8,-111,-85,3,-78,-54,-91,108,-92,
            116,11,50,-126,-98,102,123,13,90,59,
            50,15,-77,124,126,99,-27,-40,-81,-57,
            -69,-114,96,-11,-104,80,82,-41,-96,116,
            61,-22,111,31,-112,27,110,96,107,-48,
            -41,36,15,-24,-6,-108,-23,108,-124,-68,
            73,63,-82,12,-110,95,53,-70,-17,37,
            -74,121,124,-97,-27,71,-27,-78,63,-47,
            -11,-21,-54,-47,-18,-102,111,10,126,45,
            -87,13,68,-2,-120,17,-95,-85,47,107,
            16,-112,-101,120,67,49,67,-58,-75,102,
            13,116,43,-37,-51,-15,64,122,-77,-79,
            -82,-114,-21,-51,-14,-103,-86,-57,127,-90,
            93,92,-59,-31,108,68,-6,-1,-92,-1,
            -11,-105,-123,56,-110,-73,-99,-89,-125,-86,
            -42,-122,28,-3,-14,-58,-15,108,-15,110,
            -87,-57,103,77,97,13,-76,-105,-7,51,
            6,-109,-92,55,-9,-24,-49,-51,-102,-59,
            -14,105,-91,-57,55,-78,-111,-22,57,70,
            -78,-59,121,58,-84,-73,49,-69,93,48,
            127,69,-123,-98,-44,113,62,-93,-6,31,
            34,-7,5,-23,-43,-10,119,124,-63,-114,
            -31,89,107,-61,60,118,-46,-110,26,-112,
            -89,-74,-79,-75,97,4,51,36,25,40,
            118,27,75,79,45,-30,126,93,26,108,
            113,-113,-35,65,-23,-76,62,119,62,-89,
            126,-95,114,-79,-35,-102,49,63,-120,-67,
            120,61,-57,-21,118,81,118,82,118,83,
            118,84,118,13,106,-9,-77,-73,61,44,
            126,55,37,-68,42,72,-127,-54,-79,10,
            44,-94,31,3,-126,120,-34,-44,-77,-67,
            -75,-11,-53,-19,6,-80,41,-121,12,33,
            55,-46,-10,119,-88,126,125,18,91,-92,
            -67,106,10,-85,-88,75,47,-101,-38,38,
            77,-46,-45,113,-45,-91,124,-20,121,-6,
            -11,118,-52,-95,118,-53,109,4,57,81,
            24,-64,-20,-32,97,118,98,13,-23,-38,
            -11,-18,16,-19,112,-106,-25,-83,28,-79,
            118,122,15,-24,91,61,32,87,81,15,
            89,77,126,-59,115,-105,81,121,-9,-79,
            -12,-6,-77,116,105,-18,14,99,83,68,
            50,-33,-51,-39,20,87,126,80,83,-3,
            122,-42,67,1,25,-77,-127,85,-8,107,
            93,-86,16,67,-114,105,-42,-61,122,58,
            110,-70,89,44,95,-70,-98,-100,-63,-6,
            89,89,-74,6,-7,88,61,-43,125,-5,
            117,-69,-84,-90,14,-73,-22,93,-35,-114,
            -58,-25,-70,-99,93,-69,7,59,25,81,
            -89,-71,102,13,93,61,-114,45,-94,-10,
            -125,36,-37,-22,-6,-103,-84,39,24,125,
            -23,71,7,-110,84,48,-13,15,-3,-66,
            -7,91,89,-49,126,-121,-30,-126,100,46,
            -23,-97,-46,-17,27,-37,-108,-11,-16,-105,
            117,-87,-62,88,118,34,-31,-90,-109,-85,
            -25,-29,-26,75,-9,-51,103,107,-105,91,
            -18,-22,122,-67,-116,62,-6,125,51,-103,
            -93,-85,-20,-92,-20,-90,-12,-29,-40,-110,
            96,94,-85,-73,-61,-39,51,28,-101,3,
            107,83,-126,-42,96,-54,-128,78,2,44,
            90,-28,-83,105,36,107,-77,-35,-42,90,
            -106,-18,104,-6,49,-121,25,-108,12,-108,
            58,-123,-83,9,21,89,79,87,-114,101,
            -124,-82,-49,-116,-89,-75,-94,94,-63,-11,
            -119,-94,-35,97,102,7,-106,94,4,75,
            55,72,-66,-87,-17,7,57,27,-99,-61,
            -22,-91,-10,0,108,13,-75,106,49,59,
            53,-41,29,101,107,16,-39,-71,-45,-71,
            -37,33,-33,96,-52,35,-113,-97,-50,122,
            -126,58,51,59,-86,79,13,-42,64,-26,
            -40,53,97,-23,-46,105,-75,-63,22,-35,
            -24,-50,-76,104,127,-56,12,-56,13,42,
            -39,72,98,83,-43,-16,3,5,-41,103,
            56,59,17,112,-97,-53,13,-110,62,-53,
            55,-121,-42,-92,-24,-121,-40,-52,64,-11,
            -16,126,-92,-21,-69,-14,-111,58,-112,77,
            -43,71,-103,29,-23,-66,-8,-119,-52,94,
            55,7,27,65,85,-11,120,93,-102,-78,
            -20,-27,36,-23,-16,-48,-70,-65,96,71,
            82,-48,-74,-71,-2,39,44,67,-22,-126,
            94,-74,-51,76,-56,44,120,42,-112,-69,
            -125,24,-112,122,-98,-40,-87,-21,51,-88,
            -89,86,-94,-87,-72,18,-11,-40,12,54,
            -78,-35,-25,-78,-125,-92,79,39,10,118,
            27,93,-97,-64,-36,1,85,15,47,-13,
            -65,26,40,55,-31,-97,-84,94,-121,-104,
            -35,104,-58,-80,-105,-110,-99,-17,100,-19,
            16,25,-62,-65,49,78,-21,-22,-9,-104,
            63,98,-45,-95,-91,-88,-87,-21,79,-110,
            -68,99,20,-85,16,-19,106,-54,48,-65,
            -23,-75,118,108,115,64,14,-94,-76,-126,
            24,80,-55,-100,-126,-41,-92,36,74,47,
            -87,67,-112,53,39,39,68,-70,22,43,
            7,-23,95,-21,-52,118,-61,84,15,-17,
            23,-70,-66,25,-19,-66,78,-79,25,69,
            -60,-22,118,83,-6,-9,-40,76,-29,-38,
            61,46,-44,46,-18,113,102,96,-86,-16,
            115,52,55,-82,-91,-5,-10,92,-59,42,
            -104,72,5,-95,53,40,-123,118,121,6,
            -11,104,-109,-19,-14,22,-115,97,-122,92,
            -55,54,7,-63,12,73,107,-95,108,-88,
            -21,-9,95,79,-110,-115,28,-9,-66,83,
            33,-46,21,-84,28,-92,95,-60,-90,32,
            85,15,-125,70,108,10,-83,73,-10,106,
            122,-114,117,-100,61,-107,-55,110,-31,-70,
            29,-67,-19,-103,-3,31,15,-75,-117,-29,
            71,24,127,-89,-126,44,33,57,72,-65,
            124,53,-23,51,-30,-39,72,-8,78,-113,
            -81,-94,67,-46,26,-76,-104,-101,-12,98,
            110,79,36,-21,105,-83,66,24,80,6,
            105,64,-102,106,-59,-49,-70,-108,7,66,
            60,23,68,-14,114,-20,-95,-35,-107,73,
            107,88,13,-102,57,86,77,102,-11,94,
            -85,-57,-45,-41,-112,-99,-30,-39,72,25,
            -88,-37,85,-39,57,104,59,-124,-12,-116,
            -121,-48,115,-76,111,15,-37,-82,95,-18,
            -25,97,5,-91,99,123,-85,-68,-82,63,
            72,115,-4,16,-22,113,89,-17,50,3,
            -67,84,-56,6,-110,108,-15,85,-122,61,
            -88,-53,96,-9,-123,-108,-84,28,89,-109,
            -11,114,31,100,107,-96,85,-82,96,63,
            42,-15,25,-26,31,109,38,59,46,-47,
            -19,26,-22,-28,-27,-49,-97,-91,-87,109,
            -12,-19,-108,49,-99,42,111,96,103,92,
            54,-51,-23,54,-51,-75,-126,-115,44,57,
            -79,-32,93,90,-95,13,-7,-1,-110,-71,
            -25,46,-89,-86,-121,-86,-105,-99,-63,78,
            -63,-57,82,67,-88,-41,20,47,-110,-4,
            -9,121,-98,-27,-99,-9,-31,52,-37,-106,
            119,-98,-58,26,104,6,-85,24,121,-44,
            -42,-31,32,107,-126,85,68,26,-56,42,
            120,-9,-88,-54,-83,-22,-31,-42,-13,7,
            61,-34,-23,91,102,-89,90,23,-5,52,
            59,-100,-51,-99,-53,-12,-53,107,-40,-30,
            109,-85,51,48,-38,-91,-56,-52,34,54,
            98,10,59,-94,50,-11,122,-40,-81,-78,
            -75,-106,-81,53,101,-39,89,-37,69,-1,
            -40,68,-19,70,-18,97,123,-117,117,-84,
            -127,-18,-94,-8,-82,98,-46,32,-95,36,
            -83,-63,54,59,-28,-19,-62,26,40,-24,
            -9,124,23,45,-88,-19,-32,18,93,-19,
            -89,-77,-92,-99,61,-40,20,119,-53,-1,
            60,114,76,-32,-56,72,32,109,79,51,
            -32,-8,55,-113,0,39,93,37,-107,94,
            -35,-89,-98,-69,32,35,73,84,-41,-11,
            -37,-23,-101,-119,-16,91,-103,125,102,-121,
            -38,54,95,-84,-32,47,-40,113,123,-35,
            100,-69,-99,-42,103,-83,61,6,56,-61,
            16,81,64,90,70,37,32,-77,-52,31,
            -127,60,22,118,57,-112,57,97,38,-112,
            -46,-4,5,72,105,36,3,-25,-71,-63,
            32,95,42,-67,-71,11,72,25,-26,5,
            -114,-68,12,-56,28,-13,123,32,-113,25,
            21,-128,-77,-53,-69,10,-100,-79,-10,-16,
            114,5,36,57,-80,22,59,105,24,92,
            -127,-115,-100,-91,23,121,51,80,-40,-87,
            -50,-84,-55,-114,62,-56,-17,17,-121,89,
            15,126,-34,83,15,-56,92,-29,48,112,
            26,-96,1,112,12,-41,19,-100,97,-104,
            -78,-32,44,-125,-87,-87,114,-74,-18,32,
            -53,20,38,19,-39,125,-69,-35,116,-62,
            65,126,62,-26,11,-64,105,-64,88,32,
            115,-116,-3,64,30,-13,-60,2,119,-60,
            -9,-46,-21,97,-45,46,54,54,-115,57,
            -78,-31,69,101,-28,-16,-48,-9,44,-57,
            -85,34,-8,-63,61,91,51,-70,1,123,
            -76,89,21,-100,-47,99,-5,1,-41,-111,
            -100,18,49,29,-120,-45,77,110,0,98,
            72,-97,91,-128,-24,60,46,1,-120,-8,
            79,-61,-128,24,-74,98,39,16,-93,-41,
            117,3,66,40,-71,98,59,16,111,124,
            -110,5,-60,-48,113,55,2,-47,-3,-87,
            95,-128,-24,-48,120,14,16,111,69,92,
            13,-124,106,-128,105,-84,28,-46,-116,2,
            -50,-120,-18,2,-20,36,-73,-68,121,-11,
            90,-101,-52,-22,-81,94,-44,117,-9,20,
            -47,64,14,-101,119,-77,-82,30,-15,-71,
            -89,10,112,42,-42,21,88,52,37,-54,
            -105,-93,59,1,-15,105,82,12,16,119,
            45,60,10,68,-65,-125,95,2,-23,-109,
            -127,-80,80,126,-104,-121,-108,89,121,8,
            -46,43,-87,-62,14,-110,-87,76,47,-28,
            -119,60,-124,-4,56,15,21,-66,60,88,
            30,72,-49,103,-69,-128,-24,-41,-9,48,
            16,-15,-41,-115,7,98,-123,42,-89,99,
            114,7,-57,33,-83,12,-44,8,26,-54,
            94,115,120,119,-22,83,126,-47,11,116,
            68,-31,-83,-31,-50,125,-99,64,-4,-57,
            -86,-25,-59,100,0,-21,-127,101,115,-127,
            40,-101,125,24,-56,-34,-82,-55,26,-27,
            -95,-62,-18,19,73,64,54,-3,126,62,
            16,83,103,-100,2,-30,-119,-63,-41,3,
            -47,-71,119,61,32,-110,-69,127,11,-20,
            -109,-35,-65,1,118,86,-17,-82,64,12,
            24,60,23,-120,-114,51,55,1,-15,-28,
            15,107,-127,88,-112,85,7,-56,56,-54,
            -25,119,-39,48,15,-73,65,-77,-57,3,
            -103,-76,-52,15,-124,17,-109,8,-84,-73,
            84,61,-54,-68,14,-38,-82,97,-43,47,
            87,-40,-93,-102,-65,56,24,-117,-43,-81,
            43,-26,-127,107,-56,-47,59,-35,35,110,
            40,112,-19,48,77,-42,-52,99,75,106,
            34,16,43,-58,95,9,68,-117,-124,31,
            -127,120,36,114,36,16,-87,-98,106,64,
            118,-13,68,3,-71,-100,122,-14,-119,-112,
            -69,-81,-64,46,-18,4,61,-65,-52,-45,
            30,-56,-18,85,39,2,-79,-82,83,34,
            16,-113,-66,-69,18,-120,54,59,-121,1,
            -15,-107,28,-109,7,-115,72,-15,102,-93,
            24,-96,-46,-51,-18,80,-39,4,81,79,
            -78,122,-49,-15,20,-109,-96,62,-121,-14,
            84,63,2,-54,-111,-93,-74,111,65,-107,
            -97,-128,-108,-93,115,-128,53,-96,-59,-57,
            64,76,-12,-19,0,-14,26,50,108,-86,
            39,9,-72,-122,-74,61,111,3,41,125,
            117,-127,60,-19,51,64,-2,89,91,-51,
            109,64,76,-118,89,14,-124,-6,-98,46,
            -37,-9,29,112,-98,-69,25,56,14,-26,
            32,112,70,-70,-3,-127,27,-33,26,24,
            -15,-78,90,-53,109,64,-68,51,-6,123,
            96,61,92,57,13,-88,-5,-2,-72,55,
            111,105,50,-3,9,-84,-34,-37,-117,75,
            3,-115,81,63,-18,-40,7,12,-6,87,
            -61,-10,92,127,51,32,101,-28,-3,32,
            55,127,-69,-99,9,-92,-16,126,6,-92,
            -19,29,3,100,-106,-15,2,-112,-57,61,
            27,65,-2,-95,-89,-71,20,-56,-26,79,
            -73,6,-74,60,17,13,-92,-9,88,115,
            32,-78,122,-59,1,65,83,-115,-20,-30,
            -71,21,56,-7,44,6,78,-70,-81,2,
            121,-52,59,9,-56,44,-17,-121,-64,105,
            -80,52,-112,63,2,35,-37,3,-53,-12,
            55,4,74,-65,109,-57,115,-53,-127,103,
            49,-85,-9,-120,-30,-46,64,111,120,-36,
            -13,109,4,-125,94,64,109,-1,-49,-27,
            61,-128,-108,-23,-119,32,119,-31,-120,-43,
            64,-12,-88,-104,6,100,125,-73,-63,-26,
            0,41,-116,-81,-127,60,-90,-12,-41,117,
            1,-94,-55,-82,10,64,68,-88,-55,-46,
            74,1,-46,-74,-122,1,119,45,-79,55,
            -25,5,17,30,117,4,8,-9,-93,13,
            99,58,112,26,-98,-14,81,-6,-118,123,
            -127,108,52,50,10,-120,-124,-76,-99,-64,
            -70,-76,82,12,112,27,-24,116,32,121,
            126,-60,-27,25,89,92,26,-56,-3,118,
            -71,118,7,-32,-89,-9,50,-5,98,-85,
            -51,4,82,-98,-84,14,-20,-59,100,-56,
            25,-93,54,3,-101,62,-118,-105,-109,-51,
            -115,-64,49,-28,4,-112,-1,94,-57,63,
            26,-56,7,-98,-115,6,34,46,99,57,
            16,91,41,-99,92,37,-113,-10,6,66,
            -66,-38,23,-120,58,-105,12,1,50,-38,
            109,-96,101,64,-18,83,-7,-112,126,-46,
            -88,47,-127,-83,26,126,-50,-119,1,-64,
            -98,24,-7,44,112,-89,-72,-79,-41,111,
            5,126,-10,58,-63,-77,-93,-104,-76,-113,
            -5,-41,65,79,-27,44,16,69,111,32,
            -77,39,54,93,10,-92,-52,76,0,-10,
            23,-9,85,4,-126,-2,127,35,-73,121,
            -42,3,-57,16,-73,1,105,115,-49,-34,
            -88,6,-14,-11,-107,-17,5,-78,-54,71,
            -3,-128,-104,48,-1,43,32,-34,-87,-102,
            2,-92,-49,109,-112,-38,-64,25,41,-127,
            116,-14,95,-36,-87,124,84,-66,84,-114,
            -5,90,0,49,-5,72,127,96,47,104,
            60,31,-72,-101,-124,123,42,127,0,-94,
            94,43,110,-101,4,-11,113,-120,-21,65,
            -121,-39,-96,-19,18,101,-112,-70,13,65,
            -82,-65,-42,30,96,81,-61,-120,127,-7,
            82,-127,24,-20,-5,28,-120,-18,-66,123,
            -128,-24,-27,-21,3,68,79,95,71,-32,
            -60,-105,1,-47,-35,95,7,-120,-57,-52,
            8,32,22,25,57,-64,-74,2,14,-80,
            125,-38,24,5,-60,34,127,44,16,79,
            -108,-15,0,-47,-51,-41,18,-120,-34,-66,
            68,-32,-92,-85,-14,81,-7,6,-54,-111,
            -94,-54,85,107,61,-80,-90,-41,89,10,
            114,-107,-1,38,-61,78,-127,-74,-20,-112,
            -40,107,-22,118,40,122,65,-99,-55,109,
            113,27,104,55,24,-70,87,85,-20,-67,
            -111,-32,116,-22,-15,39,-127,24,-98,49,
            30,-120,-58,-23,41,64,52,77,-17,2,
            68,-21,-12,-66,64,-76,74,-17,9,-50,
            -112,79,3,71,-42,5,-94,-43,-95,62,
            64,-36,116,-24,121,32,-70,29,122,6,
            -120,-121,15,77,4,-94,81,-6,13,-64,
            -71,-65,31,112,100,47,112,70,122,42,
            31,-107,-17,43,64,-36,-98,49,28,-120,
            -58,-57,103,1,49,108,-62,124,112,-38,
            -83,-121,-81,26,24,-54,-2,5,-18,-42,
            -69,-56,58,-86,-55,5,-85,-41,101,-6,
            107,2,41,55,29,1,-39,19,50,-113,
            0,-21,-79,3,-13,-128,21,-109,-42,13,
            88,13,-46,58,-125,66,-56,-5,-127,35,
            -17,4,-42,-115,105,99,-127,85,47,-19,
            109,-32,72,-91,87,-9,21,54,93,42,
            -57,-127,-87,-64,-118,62,122,59,-80,-98,
            -8,-71,37,-56,126,-71,-20,-75,-64,125,
            115,26,108,91,-3,82,17,109,31,115,
            -65,-5,115,0,104,-27,126,96,104,30,
            4,98,-118,-17,119,32,23,-102,47,1,
            41,-67,13,-128,35,47,7,69,72,-2,
            13,56,-27,126,22,72,89,-26,82,-32,
            -108,-5,40,16,-7,95,-62,6,-22,-7,
            96,80,59,20,-111,-96,-2,77,-67,94,
            87,15,-94,-9,65,91,-24,-17,-17,-85,
            -88,103,-83,-95,93,-34,-102,-19,-59,76,
            82,-71,87,-47,76,-79,-27,81,-86,103,
            47,-42,64,-21,117,-69,20,-127,-95,-61,
            -30,-44,32,38,-1,-1,-53,37,-98,-110,
            17,88,61,-36,122,62,30,-62,46,127,
            121,-32,-17,63,-24,59,58,-125,10,110,
            -88,-65,-77,116,44,-26,-78,-99,94,47,
            -49,-96,16,118,40,-78,-31,38,-110,-22,
            93,-67,-6,-113,105,-101,98,46,91,-77,
            122,-35,-28,41,13,-91,-95,52,20,-63,
            -16,95,110,68,-118,76,27,50,-52,-48,
            0,0,0,90,122,84,88,116,67,111,
            109,109,101,110,116,0,0,120,-38,115,
            73,44,73,-75,82,8,79,77,-47,81,
            48,50,87,112,44,77,87,48,50,48,
            -80,80,48,-76,-76,50,-78,-80,50,49,
            81,8,112,9,-31,114,44,45,-55,-56,
            47,-78,82,112,45,-54,76,86,112,73,
            85,-16,45,-51,75,81,-80,73,77,76,
            113,-56,-84,-56,76,-52,-45,75,-50,-49,
            -75,-29,2,0,-31,23,21,24,123,-69,
            10,-77,0,0,0,0,73,69,78,68,
            -82,66,96,-126
        };
      }
    
    private static byte[] blackRook() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,7,-79,
            73,68,65,84,120,-38,-19,91,107,108,
            20,85,20,-34,-19,-82,-76,69,72,91,
            40,-31,97,4,44,66,2,88,121,-40,
            32,88,-38,16,-34,-126,66,3,4,68,
            106,75,-83,84,-123,-122,32,-113,64,12,
            -47,-112,32,-110,98,108,-125,65,52,42,
            74,-119,65,64,68,20,10,49,16,84,
            20,4,3,-38,52,-118,85,40,22,95,
            -107,62,40,2,-74,101,119,-25,-38,-61,
            -100,115,-11,28,59,83,53,-80,-99,93,
            -9,124,63,-66,-34,-37,-39,123,-65,-17,
            -36,-103,-69,119,-18,-52,-70,92,-111,-120,
            68,36,34,17,-119,72,68,34,18,-111,
            -120,-60,77,-94,-4,-68,73,-18,-39,-56,
            105,-56,-93,5,-113,66,-66,31,57,17,
            63,-17,17,76,-15,57,30,55,-85,-107,
            118,-45,-8,113,-12,57,29,-94,125,-22,
            87,-21,24,-43,74,-69,-77,-71,79,-53,
            60,68,34,18,-1,44,-90,-13,98,-20,
            64,-109,-69,36,-101,-36,57,31,-7,-112,
            -32,-105,77,78,-52,51,57,-66,22,27,
            24,-122,-100,-62,-37,-19,-80,8,-37,-115,
            -61,-49,47,-79,104,119,9,63,-82,-61,
            19,66,111,10,-17,-121,-6,37,29,-92,
            -21,111,-19,-26,115,95,-79,119,-40,-25,
            -63,49,-31,41,-27,-27,121,107,77,86,
            -54,-28,-102,66,-109,-21,103,32,-49,52,
            -71,110,-75,-55,-115,83,76,62,81,97,
            114,84,1,-14,10,-34,-18,-58,100,-47,
            -18,65,-34,30,49,-43,-45,113,27,83,
            121,59,81,-53,121,63,-44,47,-23,32,
            93,-70,-35,25,-36,7,-75,59,111,-67,
            125,30,-100,51,64,-33,-14,114,118,41,
            55,-30,-65,23,-53,-115,-68,-34,-24,-58,
            -53,-89,-35,38,-73,-21,-124,83,-70,48,
            92,-4,-116,104,-9,110,94,-42,-11,-61,
            121,-71,120,-125,-8,-86,-8,-110,-9,67,
            -3,90,-23,34,-35,-2,-119,-68,62,91,
            -8,-10,124,-25,-44,1,42,16,87,122,
            12,55,-30,-37,-117,-58,79,33,-1,-126,
            -122,47,-16,-29,-54,31,-57,51,27,-65,
            108,-93,-66,-26,-19,110,-70,42,-38,-19,
            -56,-37,35,-90,122,58,-18,-11,-10,-30,
            10,-6,-118,-9,67,-3,-22,1,-66,32,
            -38,61,-59,125,-48,113,-45,-29,-20,-13,
            -32,-100,1,26,-54,-53,19,-46,-71,-111,
            -64,83,88,-82,16,103,-22,107,-68,-4,
            19,-82,-74,38,103,32,-113,55,57,29,
            7,-4,64,-86,104,119,76,-53,87,-112,
            -84,63,64,-19,-32,64,77,30,-57,-5,
            -95,126,-83,116,-111,110,-19,3,-21,39,
            -116,-74,-49,-125,99,34,-86,-116,-105,71,
            14,16,-122,31,-60,-14,-89,45,39,-44,
            -15,-116,-70,-115,-39,-68,126,100,-78,125,
            30,-100,51,64,11,121,121,72,-122,48,
            -120,-85,47,85,44,-22,-59,-108,-91,54,
            -31,-103,-102,45,-8,78,-4,-65,71,28,
            127,-59,34,-95,-78,-34,-61,-37,-111,-19,
            83,-65,-106,-70,-118,-123,15,-84,31,50,
            -51,62,15,-50,25,-96,118,-68,-36,-17,
            12,38,96,-69,48,-70,-80,-107,51,-43,
            -121,124,78,-16,-9,-56,-105,-1,-29,21,
            112,89,-76,35,-37,-9,-75,-14,121,-95,
            -101,124,-111,79,-85,60,56,38,-36,59,
            121,-71,103,-106,-55,13,116,38,-113,68,
            30,43,-116,7,116,-39,13,104,67,-74,
            -46,-59,117,-93,-113,6,28,-16,-98,-103,
            -10,121,112,78,40,113,67,-119,83,-54,
            105,49,103,-5,22,-72,-30,0,74,-81,
            -78,-36,89,0,117,-47,-101,11,80,-122,
            -73,59,64,41,111,63,-64,13,-28,110,
            -128,102,126,4,-96,12,-46,-95,-11,50,
            -99,-66,-7,-30,118,0,-73,-112,58,-72,
            -19,-13,-32,-68,43,105,20,47,119,-57,
            -17,-94,-3,98,121,106,-32,-3,-121,42,
            119,-75,3,-24,41,-90,10,-7,-41,32,
            115,-107,-48,81,46,116,-94,-18,-3,37,
            -36,-105,-107,111,7,-82,-77,69,-7,96,
            -53,-101,-108,75,-17,114,87,2,124,41,
            100,124,64,61,32,80,93,-108,6,8,
            -108,-82,57,3,48,-6,-82,93,5,48,
            18,-41,-82,3,24,93,-82,27,99,-69,
            -44,79,81,58,32,80,58,-32,119,64,
            -96,90,95,-39,-90,78,-68,17,94,58,
            -68,-27,77,91,-19,-45,42,15,-114,-117,
            -101,-59,-103,-11,51,-34,-71,79,-91,-102,
            -10,-17,0,50,51,41,17,57,-43,0,
            95,-127,50,99,-85,63,21,-48,-4,87,
            -45,53,24,-22,122,7,-74,-85,-5,49,
            -29,-83,-100,70,-128,-81,-128,116,-75,-33,
            9,-56,-52,19,59,16,83,-71,47,43,
            -33,-50,15,-79,-102,-15,-22,-65,98,-34,
            6,100,-52,-94,68,76,90,13,-16,109,
            41,-71,13,-96,6,109,77,5,-88,-68,
            109,13,0,-107,125,-93,-104,-6,41,73,
            2,-88,65,-109,-98,3,-8,54,-109,-82,
            -104,29,-128,-116,92,43,31,45,-5,12,
            -67,-64,47,81,79,13,-81,-50,-13,-118,
            -83,-107,110,124,-18,87,103,-125,-52,-27,
            66,7,-22,-54,-117,23,51,88,45,-9,
            21,54,-31,-3,-13,-114,-5,22,-64,-118,
            88,61,-41,79,4,-8,74,61,115,1,
            -54,-16,-36,10,80,-54,-45,31,112,3,
            -103,-6,-63,126,73,-121,30,32,83,103,
            -94,-91,-113,48,9,60,-45,-68,125,-2,
            82,-45,-116,117,51,117,34,98,0,-66,
            111,92,-99,0,-54,112,37,0,-126,-56,
            -44,47,-23,96,-9,101,-21,114,-60,0,
            -11,9,-73,43,8,-25,104,-49,33,-79,
            43,-3,-66,-104,-30,122,57,99,-49,77,
            -22,-40,116,64,76,113,-121,-62,-27,-69,
            -121,-94,43,110,-127,-44,-14,-22,-35,31,
            -118,-60,108,116,-56,0,9,29,-69,63,
            19,91,57,-75,-36,87,-24,-57,8,-100,
            17,118,-16,-22,-61,-30,-50,60,-48,-33,
            25,3,36,117,28,94,34,102,-20,-19,
            -36,87,-24,-57,-30,-106,-85,-53,-106,-119,
            -60,124,-28,-112,1,18,58,-54,-98,-2,
            119,-66,66,47,62,-31,-59,-24,-63,38,
            87,126,33,-74,126,-90,57,99,-128,-92,
            -114,74,92,52,68,15,-77,-9,21,-70,
            -117,-72,65,-68,-100,-128,123,95,117,57,
            34,49,101,14,25,32,-95,-93,-18,49,
            -44,93,111,-17,43,116,7,104,55,47,
            -9,-60,-83,-108,-90,68,-111,-96,116,103,
            12,-112,-44,-47,-44,3,117,-81,-73,-9,
            21,-78,17,53,-122,-105,-109,99,69,66,
            -24,65,90,-95,67,6,-120,116,92,-30,
            -11,-55,-99,-124,-81,-79,-31,50,64,21,
            -6,-49,94,-128,-76,-121,-75,-15,75,0,
            99,-72,-69,6,-96,62,112,127,12,80,
            -65,-71,-113,3,-126,-56,-121,1,-51,-116,
            58,72,-105,-42,105,-22,-98,111,-23,43,
            -76,-61,-77,-120,-105,-89,62,-21,-118,2,
            52,27,-17,1,-64,7,-105,46,117,-34,
            33,87,-48,121,83,-105,95,105,-99,-90,
            -18,66,123,95,33,-73,-75,-29,-62,-41,
            -92,60,123,-12,63,30,2,-28,46,-89,
            -124,-60,95,2,-8,-26,36,-3,0,48,
            54,-12,-14,1,-116,125,-67,87,3,-116,
            -9,-126,-59,-44,47,-23,-120,-65,2,-16,
            -51,-47,123,-123,-90,-18,85,98,-128,-10,
            112,-97,-50,-33,-6,-111,111,-11,-29,54,
            125,-12,-19,84,17,115,28,-80,76,63,
            111,41,124,23,-48,84,-125,-49,99,114,
            27,75,0,42,-45,127,20,-96,-78,-126,
            -59,-44,47,-23,40,60,8,104,-46,47,
            84,-58,28,3,44,19,111,-90,70,-9,
            -27,62,45,-13,-48,-26,33,31,-48,-31,
            13,-100,55,65,87,37,1,-122,110,-9,
            -34,3,-88,120,-109,-116,111,126,20,-32,
            -97,-115,-119,121,-14,-86,2,-88,7,-116,
            19,0,-107,29,44,-90,126,73,-57,-26,
            -59,0,-65,126,-105,2,117,111,71,31,
            -69,-60,-26,105,2,-9,109,-107,-105,-74,
            -113,12,20,26,-51,-85,-13,95,48,-25,
            -14,-85,19,92,-117,0,122,-82,-97,-39,
            117,31,-64,56,-41,-9,90,24,69,73,
            43,1,-58,-106,-92,38,-128,-15,70,-48,
            24,-5,37,29,93,-9,3,-116,74,-46,
            -87,117,-101,62,-16,-35,-20,-4,23,-59,
            -119,25,-51,-13,-32,-100,-81,28,-15,-120,
            49,-18,85,-109,119,-84,18,-101,-112,-72,
            115,16,-112,-17,-97,-43,59,100,113,-96,
            -20,117,5,-4,-36,7,-43,-109,-49,-72,
            87,-20,-13,18,-4,64,1,-98,42,94,
            93,36,-18,31,-102,-16,29,104,-93,-63,
            21,13,-48,9,104,112,-89,1,-44,-91,
            40,15,64,-43,-69,-49,0,-44,69,-9,
            -113,-128,32,50,-10,-85,117,-96,46,-46,
            -119,111,-5,40,-19,-125,-7,-94,-6,-94,
            -117,98,17,81,-59,-13,-44,118,-113,17,
            -86,121,-11,46,-15,-13,18,-1,74,113,
            102,-6,29,122,-59,-76,-58,126,123,95,
            -28,91,-33,39,85,-73,-11,99,9,-6,
            45,103,71,94,125,-84,-73,-104,26,-26,
            -123,-24,-128,-76,-74,-21,-99,-57,-53,-57,
            -110,-60,20,-41,-111,-25,41,-8,-79,87,
            -84,46,-73,-103,124,54,75,108,62,-114,
            8,-49,1,-110,-66,-50,-50,-59,60,-120,
            -25,94,-82,-110,-74,90,28,76,-27,-27,
            46,-8,-96,-85,78,-4,42,-64,-113,123,
            86,-127,41,-31,-59,126,-15,29,84,103,
            96,30,-110,-19,-13,20,-68,61,-74,33,
            -68,60,48,-59,-30,-116,91,19,-98,87,
            -112,-107,-81,-127,-87,-10,121,10,-34,0,
            -115,-25,-27,-63,-72,106,57,-119,-17,-105,
            29,-63,-73,95,-114,-30,-98,-43,-47,-105,
            -62,-116,-47,-41,17,-4,-18,57,-119,-65,
            77,29,124,-63,62,79,65,95,102,-21,
            88,-128,-36,-37,98,111,46,-36,66,-6,
            -22,45,-14,-32,106,-21,101,-74,124,73,
            28,47,109,55,-19,40,-48,42,-17,62,
            81,14,23,-106,-66,50,120,30,44,-13,
            -44,102,103,80,103,100,-36,10,113,-47,
            3,-83,113,97,-50,99,-123,-17,-50,-1,
            -109,25,36,18,-111,112,120,-4,1,-13,
            59,-63,114,7,2,17,66,0,0,0,
            90,122,84,88,116,67,111,109,109,101,
            110,116,0,0,120,-38,115,73,44,73,
            -75,82,8,79,77,-47,81,48,50,87,
            112,44,77,87,48,50,48,-80,80,48,
            -76,-76,50,-78,-80,50,49,81,8,112,
            9,-31,114,44,45,-55,-56,47,-78,82,
            112,45,-54,76,86,112,73,85,-16,45,
            -51,75,81,-80,73,77,76,113,-56,-84,
            -56,76,-52,-45,75,-50,-49,-75,-29,2,
            0,-31,23,21,24,123,-69,10,-77,0,
            0,0,0,73,69,78,68,-82,66,96,
            -126
        };
      }
    
    private static byte[] whiteBishop() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,16,-110,
            73,68,65,84,120,-38,-19,-36,121,124,
            22,-59,-63,7,-16,-35,125,-98,-28,-55,
            -59,125,72,0,3,-31,62,2,-106,-5,
            82,44,10,114,-45,96,56,-59,82,64,
            32,20,-124,20,90,43,1,109,41,-14,
            65,80,48,-36,72,44,-120,-128,7,65,
            -126,-124,64,80,65,-114,34,4,-88,72,
            -72,15,19,32,34,71,-28,-54,-99,-20,
            49,-45,-3,-19,51,51,33,75,-97,34,
            36,-81,31,-56,-101,-7,-2,49,-49,110,
            -98,-99,-99,99,119,103,118,118,-97,72,
            82,105,40,13,-1,7,65,-79,45,-41,
            102,-15,53,22,95,97,113,-83,-5,108,
            87,26,-118,57,-108,-79,-43,-73,-117,-59,
            123,61,-76,-29,-9,-18,88,-50,-1,-33,
            -23,-108,-122,-94,6,63,-15,105,13,-56,
            53,88,-59,87,113,-66,7,-110,84,-83,
            23,-8,-99,-83,86,7,-4,-74,-71,118,
            -127,-7,-107,-41,64,-82,45,-51,2,-71,
            -70,-57,116,75,67,-111,-126,44,121,-125,
            -4,122,-31,-43,117,103,41,3,97,-45,
            23,82,103,-72,94,85,122,1,-82,101,
            43,35,96,99,-110,-5,123,-63,103,-59,
            38,94,32,79,18,-23,-106,-122,98,105,
            -97,81,-30,99,19,-88,90,89,106,11,
            23,-65,112,-81,-92,-76,-20,40,32,99,
            -54,62,15,100,24,95,47,117,-128,11,
            67,-91,-58,80,-91,-93,45,-35,-105,74,
            -21,-74,88,-126,35,-121,127,-14,94,12,
            83,-114,-16,6,24,-22,-124,-100,-68,-52,
            51,96,52,-55,108,5,70,-5,-63,-27,
            32,55,-120,127,-49,123,14,76,-103,96,
            75,55,-69,-76,110,-117,122,105,-77,-126,
            -45,33,26,-24,-33,-80,112,58,-81,-8,
            85,3,65,-83,73,109,97,-27,88,80,
            59,-118,6,58,9,11,103,20,78,94,
            -92,91,122,-87,43,-30,25,116,88,52,
            -112,6,-31,-1,-30,21,-33,119,34,-88,
            -73,-46,-121,1,89,121,103,1,-112,53,
            -67,-125,65,-67,45,26,40,3,-62,-53,
            -37,-46,61,80,90,-73,69,11,94,44,
            94,43,-42,-68,7,-82,-97,-92,-31,-80,
            110,43,111,-128,-118,77,-64,72,-86,-112,
            14,-58,69,-47,7,-115,-122,53,53,-91,
            40,112,-75,-73,-91,-65,-46,-74,-97,-46,
            -16,11,-125,-65,-8,-28,3,-14,7,74,
            26,56,-5,-5,-116,6,115,124,-4,36,
            52,-67,-31,-40,9,119,66,69,-125,-80,
            -40,-67,-2,118,-126,95,75,104,-78,-41,
            103,54,-104,119,70,-41,-63,25,38,-71,
            64,-98,-17,113,-65,-91,-31,-65,-11,53,
            -110,75,-84,-39,5,-54,9,-10,-25,20,
            -71,57,-104,95,88,9,-29,-101,-56,117,
            33,111,0,111,-112,73,-25,-63,-8,-29,
            -92,-119,96,-44,-32,-21,-27,54,-112,87,
            -42,-11,41,-116,59,-84,-8,-127,-40,45,
            -111,118,-128,-78,-49,-106,31,-97,-46,-66,
            -119,117,2,119,53,17,52,-112,-105,-128,
            -29,68,-64,95,-63,-84,-87,-81,-96,-30,
            75,94,-25,96,-45,124,94,-15,-43,91,
            -128,126,43,-31,28,24,-29,-40,-40,-32,
            22,-117,-113,38,52,7,99,72,-32,50,
            -48,51,-8,118,94,105,-80,-15,13,-97,
            108,40,-97,95,-58,9,-26,-18,-33,5,
            -57,126,-106,-113,-22,-1,-33,-89,-120,-100,
            -123,111,28,-91,-119,-54,115,-32,-20,-28,
            -77,14,-52,-43,-21,-95,-3,73,103,95,
            -72,-76,-115,87,112,-65,116,80,-99,-73,
            42,-125,49,-121,-113,-38,-76,-85,64,111,
            107,63,1,-67,-51,-41,-33,-100,2,-58,
            -101,125,-33,5,-43,-55,-45,113,-122,-61,
            -123,-3,94,59,-96,-115,-22,-77,12,-52,
            -106,104,1,-50,-50,102,14,97,-104,-57,
            124,-105,-8,51,-58,-35,48,-109,-107,84,
            112,-100,-16,-107,-63,-84,-127,104,24,53,
            92,9,1,45,-127,85,-24,-26,-39,111,
            -128,-6,7,86,-17,67,73,40,80,-86,
            -57,3,-67,-55,-42,27,-108,88,116,125,
            19,-48,-101,100,12,-120,-47,-9,-117,-77,
            102,-126,-38,-101,55,-108,50,0,-44,112,
            -25,26,24,-2,-116,-49,2,48,27,106,
            15,56,-82,-79,124,14,47,-23,103,-108,
            108,-69,-108,117,118,-8,91,-70,-5,-50,
            7,-77,-27,40,-4,109,-81,84,13,40,
            117,109,6,-67,98,108,28,104,59,-59,
            -103,114,8,-88,70,-6,2,-51,16,85,
            31,97,33,34,102,-127,116,3,-102,-95,
            -99,2,-86,-15,-11,-97,-49,2,-19,35,
            -17,29,-96,15,-110,126,7,-26,-103,-27,
            -128,-56,29,62,17,96,-74,-60,119,-32,
            120,-115,-27,59,-60,-42,64,37,-90,-113,
            18,-93,35,-7,11,112,-4,-34,57,1,
            -52,-122,25,15,51,-10,-117,41,-101,-54,
            -96,-82,-3,87,25,48,-68,69,-61,52,
            2,-77,65,-2,108,-55,21,13,-93,90,
            -18,13,-7,-106,-126,48,-63,-110,-85,-43,
            -121,-126,-122,-35,-3,3,-24,23,2,-74,
            -125,42,102,40,28,-81,-62,-12,22,78,
            63,48,-13,29,5,-114,55,74,-38,-88,
            -81,-32,-102,-3,28,56,122,73,83,44,
            -110,-14,25,-116,92,-59,43,-60,-81,46,
            104,-46,-66,-122,96,-52,23,13,-45,22,
            -52,-50,-33,-80,104,-9,52,-60,-49,-106,
            60,-70,-50,114,65,-60,124,-67,61,-28,
            90,52,-83,25,20,-12,85,123,78,-125,
            49,-60,-25,34,104,47,-117,75,-32,-89,
            -16,114,87,105,-92,69,114,79,-54,58,
            58,123,-68,116,63,102,-127,13,-101,-27,
            52,113,6,125,6,77,43,-55,31,67,
            -42,106,86,17,39,99,91,-125,-2,-91,
            104,-104,39,-128,102,-79,69,-99,-81,-41,
            -85,3,-67,-60,-105,-113,126,11,-28,120,
            -67,16,-96,-109,89,-4,-6,-47,29,64,
            -110,109,-37,-91,-38,-102,75,-41,-54,-126,
            -40,15,-35,-16,53,-24,98,-76,40,-97,
            -128,12,73,-2,8,26,-114,-76,93,-71,
            15,-37,-122,-27,-113,93,-97,-45,91,26,
            15,-14,52,113,74,-3,29,-30,111,-16,
            10,-104,-38,6,84,62,26,59,-95,117,
            2,81,-127,26,-51,-79,-120,96,-68,10,
            52,-109,47,31,-70,8,100,3,75,111,
            49,-117,63,56,116,6,-120,104,112,99,
            2,20,108,103,75,87,-45,94,0,-79,
            124,-20,-75,91,-96,78,16,-93,-66,21,
            -80,-71,96,86,-3,101,-112,-29,-40,-46,
            -13,-113,91,-97,-60,78,121,101,-73,56,
            -107,-22,-61,111,63,-26,5,-82,-1,52,
            -24,-17,100,-91,2,-119,23,85,83,-37,
            -110,37,-6,-106,-118,22,81,-107,103,-82,
            -126,113,105,127,50,-24,-87,-47,75,65,
            103,79,84,41,-37,31,-3,54,58,10,
            -12,-61,-5,83,64,-65,-60,-74,43,56,
            -125,120,-70,-102,-123,-46,74,22,113,38,
            101,-26,1,-39,82,-17,73,-48,15,-16,
            124,-69,90,66,-25,-101,-74,65,93,-36,
            -29,118,-87,-85,-64,-14,59,80,-116,-86,
            -29,96,-27,45,94,-48,37,-34,32,-122,
            -51,-71,122,0,-36,-43,-57,-72,59,121,
            -94,-41,7,-79,-10,-78,53,-45,-29,67,
            26,-80,116,-62,29,-73,-128,-50,-70,103,
            -86,-57,-67,-2,45,-74,60,-114,111,-57,
            -45,17,-23,-78,-3,-120,107,-98,11,68,
            62,-14,22,119,5,117,-68,-72,-47,77,
            -128,-107,-97,-37,-114,-57,-127,-123,-53,-3,
            -24,-121,-115,-4,67,-3,110,-32,100,-105,
            -96,-13,81,-50,85,64,105,114,12,-24,
            7,-59,112,-8,69,48,47,65,-18,78,
            28,-67,14,80,-3,37,16,13,-108,62,
            49,20,12,-42,-48,100,87,-39,-109,64,
            -78,-20,13,-60,-42,103,-13,-17,-15,-19,
            120,58,34,93,-74,31,-66,95,118,127,
            37,46,-123,-55,35,64,63,-29,-100,8,
            60,-3,-13,51,-21,25,-32,-4,-44,83,
            -71,31,-11,46,104,17,-1,84,54,30,
            106,76,117,47,-27,-76,-85,49,31,-52,
            -6,88,13,-92,-75,-88,-6,-106,-106,-100,
            123,70,93,49,22,49,96,-66,-45,7,
            -56,-47,76,95,32,125,-65,44,7,-58,
            -37,-84,97,120,95,-76,-7,75,111,48,
            22,101,-6,3,-23,125,-57,66,-114,122,
            74,87,-124,38,22,-111,-113,-100,-67,64,
            -22,85,31,1,-68,-127,114,90,-106,-19,
            6,53,38,123,42,-9,-93,-34,64,91,
            -7,-89,-128,105,16,116,-55,-67,-108,119,
            44,-16,20,-104,5,-81,0,-92,-117,-88,
            -104,-6,-106,28,122,-97,96,76,-126,-126,
            35,-4,112,32,-112,29,-74,65,66,-12,
            -31,114,64,14,-38,-74,-53,-70,95,-6,
            -76,-86,37,-101,47,102,-51,5,-46,-86,
            -102,11,120,3,-27,29,-15,79,-123,-96,
            115,-98,-54,-3,-88,7,-2,126,-102,84,
            -26,125,-16,99,-49,103,46,-113,-112,79,
            -125,-39,-39,-9,5,35,88,92,-30,-22,
            -128,121,3,-55,47,113,30,110,68,-11,
            15,-127,94,-27,-53,7,-109,-128,-16,-25,
            66,-1,100,-15,-102,-125,-119,64,-10,-38,
            -74,-69,-26,-15,6,-105,-115,-22,72,0,
            20,-36,-56,-98,60,3,70,-96,-5,57,
            20,111,-96,-53,-125,2,-26,-126,-33,89,
            79,-27,126,-44,3,123,23,64,57,46,
            6,9,111,-63,-6,65,-68,-113,-104,63,
            29,-44,56,86,15,-73,-75,91,96,-34,
            -118,-118,67,-34,114,-17,25,52,22,104,
            -70,24,102,55,0,50,-121,-11,53,-20,
            61,56,114,-22,80,77,32,75,61,109,
            87,-48,-30,-84,15,-30,-29,-19,-21,32,
            -10,124,-13,-99,63,-128,42,38,107,-67,
            22,-64,-6,62,-74,81,-36,-15,-62,-27,
            126,-12,3,-101,-85,82,118,-15,21,-34,
            -13,-96,-53,71,-68,-96,-63,-29,64,95,
            -111,-47,28,-56,122,113,38,101,-62,93,
            -9,43,60,92,-77,24,-28,54,20,84,
            116,70,11,-96,109,-9,-12,7,99,-45,
            -98,80,48,54,103,60,5,-76,-125,72,
            55,29,-52,-19,88,58,-10,-28,-55,-113,
            80,-80,-33,59,-97,0,89,93,43,28,
            -12,47,120,-66,-67,-69,65,-105,127,-38,
            -118,-69,-21,113,-101,68,-27,55,108,109,
            -91,118,-96,-120,59,109,-25,94,-120,-87,
            -59,11,60,121,31,-88,-49,-15,-119,1,
            -83,2,-104,71,124,109,-96,55,-75,4,
            -96,-122,-47,16,-52,111,36,88,-24,125,
            -25,-30,-20,-21,-29,44,-44,8,2,-13,
            76,-39,14,102,-70,-75,-64,-36,79,93,
            16,-33,-34,31,-47,1,-44,58,-30,70,
            117,31,-60,84,19,37,124,10,20,95,
            81,-50,-57,116,-14,-108,53,-116,60,67,
            -76,-36,10,8,90,-87,-76,-121,-76,-47,
            -68,2,-42,-50,1,61,84,92,-110,-122,
            -125,56,115,120,-89,-67,-114,-114,-75,-12,
            -54,126,13,-88,35,-83,49,-112,-85,63,
            78,0,-14,-2,-113,-81,2,-119,78,107,
            7,-28,74,118,52,80,7,-3,-85,-91,
            59,75,103,13,75,55,-53,24,6,5,
            -19,-72,-58,27,-12,23,-59,92,92,127,
            72,27,37,79,-127,-96,21,-74,-29,112,
            -58,-29,58,-43,115,-17,36,-94,-11,2,
            -95,-93,-87,-5,-51,79,-13,90,48,20,
            66,-81,72,-45,-128,63,64,-45,-11,-40,
            -103,96,124,127,-92,5,-112,-41,-1,-111,
            6,-28,-39,126,71,-127,-84,104,20,2,
            -28,70,-71,88,-96,13,-107,113,64,121,
            -33,-42,-115,-57,74,36,-48,65,-27,82,
            -127,54,104,52,15,-56,-43,-33,-123,2,
            -7,-121,-11,88,104,22,-23,121,100,17,
            -112,-39,-79,-75,-63,56,-21,-84,14,-122,
            83,122,27,-52,6,-22,6,-95,23,-92,
            22,22,73,106,5,102,57,74,-56,100,
            -23,-67,-113,27,-6,-127,99,-85,-93,29,
            -104,-53,-42,-111,57,-75,-105,-5,-99,3,
            -77,66,8,24,97,-14,-25,64,-5,-38,
            110,64,-9,-56,127,6,-6,74,-27,117,
            64,-85,55,-104,13,36,-67,-23,37,32,
            7,-102,94,4,-110,-40,96,44,-112,-76,
            -54,-61,-127,-42,-108,7,2,29,-61,-89,
            -126,-60,100,104,44,-48,126,10,5,-29,
            69,-98,15,57,28,-90,118,113,-44,0,
            51,-97,93,-64,-79,-83,-92,61,110,-80,
            7,111,22,-121,40,125,-64,-79,-54,-69,
            51,-104,-121,-32,113,88,118,-116,87,92,
            -83,38,-96,-19,123,43,2,72,-56,55,
            -61,-127,-52,-72,-68,16,72,98,-34,87,
            96,-34,-79,-72,67,15,115,-32,13,61,
            -40,-91,-85,39,27,-99,61,-97,-73,16,
            -24,19,-105,-37,2,-39,-73,-13,24,-112,
            -103,51,13,32,-99,-126,-82,-125,-74,79,
            76,17,-3,27,-106,37,122,55,3,-13,
            76,-17,10,-114,85,60,-33,-74,114,-108,
            -72,96,27,-27,56,43,-13,79,126,115,
            96,-44,-33,121,69,-51,-48,64,109,-60,
            26,-32,-100,118,26,48,-36,98,-20,-77,
            -46,-79,22,74,55,89,-20,-77,-43,98,
            59,-111,14,-101,123,-3,91,10,-88,-83,
            -60,115,-87,56,24,105,-5,-43,-125,-77,
            76,73,127,-28,109,15,1,-20,-46,125,
            84,-100,90,-67,33,42,-119,87,-44,-22,
            48,80,-97,-30,53,-103,125,12,-24,113,
            -83,34,-48,20,-19,34,-48,84,-67,61,
            -48,-53,122,95,-72,43,102,-21,-75,20,
            -96,-105,-40,115,-97,-108,-20,-99,64,79,
            -120,87,-124,107,-126,26,34,-122,-47,43,
            96,30,-79,117,49,-25,74,-22,37,-51,
            -45,-16,123,48,-117,15,22,-2,-13,-95,
            26,-18,-31,43,-91,-89,-5,-127,-34,76,
            -116,-22,78,-61,127,-71,63,122,-64,96,
            -20,-123,-69,102,10,-86,-127,-106,91,-72,
            -81,59,48,-55,-106,109,118,64,73,-3,
            30,-41,-31,-12,-125,94,-23,-70,-119,51,
            103,16,52,-65,-20,94,-46,-97,-87,18,
            1,-22,-119,45,49,-112,123,113,-5,114,
            -56,79,-113,127,2,-116,-26,91,-53,-125,
            -47,125,123,46,24,-83,-73,89,-63,120,
            58,126,60,24,61,120,-68,-19,75,48,
            -98,-26,-33,-29,-37,-59,87,2,-93,-39,
            -10,-115,-112,-97,-71,101,8,-28,126,87,
            -7,103,-48,42,-71,-13,-95,53,-16,-98,
            13,33,-69,109,-7,110,81,-46,-49,-96,
            38,-84,-96,-47,98,-115,-11,90,-45,71,
            123,-60,17,-36,31,-12,69,-18,97,-83,
            24,-67,25,-26,85,-33,68,-61,-7,115,
            32,22,-65,-49,-30,72,22,-9,-80,-59,
            -111,-74,-17,-123,-37,-46,49,-92,54,96,
            -18,103,16,-24,-17,-119,124,88,-7,90,
            109,-5,-115,-85,-14,9,-5,-48,-8,81,
            -81,104,-5,-53,-26,-20,1,22,-1,77,
            -88,99,57,-117,-25,-79,56,-118,117,-74,
            -49,-80,-17,-77,107,124,-103,92,119,69,
            -52,103,-99,-12,-14,1,-46,111,97,89,
            71,-25,14,-120,58,-27,-75,21,-106,-100,
            -108,23,-125,-90,-6,87,7,74,39,95,
            7,-19,66,-57,10,96,-92,-78,-118,-67,
            -54,-29,-114,86,48,110,79,78,3,45,
            -59,63,16,-52,-31,-13,66,-48,110,120,
            109,-125,37,-39,-50,45,-80,-96,-126,-44,
            9,-106,-79,3,104,-7,104,-106,47,54,
            90,43,-61,-50,36,-103,-11,65,-50,78,
            -123,-53,37,-54,-55,-54,-83,28,98,-27,
            -4,-8,62,-11,86,-20,-93,47,-37,-75,
            87,97,63,-84,-110,-21,21,-11,-2,-64,
            -89,49,72,53,-35,47,18,42,75,-36,
            -61,110,51,-95,16,-104,30,-58,-113,-20,
            -80,-89,65,-117,101,93,-120,-41,-60,70,
            64,-70,-77,-65,-77,87,118,105,-115,73,
            49,64,66,-8,-9,-62,-22,-128,-58,-97,
            23,81,-1,62,16,-7,-102,-29,43,48,
            75,-46,-35,18,-61,94,-96,-84,86,-44,
            -5,61,86,47,-83,89,61,-35,-12,80,
            -97,69,30,13,-34,-45,25,42,-99,45,
            -77,-53,-113,6,-13,-40,-22,14,-82,-72,
            -78,79,66,-89,69,1,95,-61,-128,-10,
            1,-13,32,-84,126,-64,50,8,123,41,
            96,9,-124,77,12,88,10,97,99,2,
            -34,-123,-80,15,2,-126,96,-48,33,-1,
            -75,48,44,-51,-9,18,76,88,-20,108,
            3,-15,21,-59,19,-46,64,32,65,-89,
            -89,3,-103,-62,-58,-50,-37,91,5,1,
            -65,20,94,99,-105,-50,107,-85,-38,-20,
            3,49,-58,78,60,-107,1,100,66,-103,
            100,32,-99,-59,28,-37,64,-40,-68,-44,
            55,15,-58,-1,-32,-1,9,12,-5,-114,
            -27,-21,32,-53,-25,10,-111,111,94,14,
            119,-71,-122,-78,-65,-41,11,-40,9,3,
            58,-69,-13,-39,-55,17,-112,6,-82,102,
            -27,87,-128,89,111,-115,45,51,61,-44,
            -17,67,15,58,124,69,-61,108,-76,-52,
            116,-99,7,115,121,21,-12,-82,-94,-76,
            -127,-28,96,113,-19,14,0,-54,-33,25,
            -72,99,-101,9,-16,28,91,63,51,-95,
            68,-70,14,5,-21,3,83,64,-33,-70,
            -3,93,48,-60,40,-18,-85,83,-96,-58,
            -117,57,-77,-6,-80,-58,75,-2,0,-42,
            -120,51,-18,-101,-23,-96,-118,59,-97,-124,
            19,96,-92,7,86,7,-3,-102,-40,-1,
            -49,96,-18,-33,-6,89,10,37,-65,56,
            -33,-18,-8,54,43,119,125,-111,-97,30,
            112,126,-66,-14,33,-12,88,-27,-118,6,
            -77,-34,22,90,-26,122,-86,-25,7,13,
            126,-18,107,-79,-14,25,123,71,89,-110,
            126,-126,81,-111,-30,-50,-69,10,-48,-127,
            -67,123,-126,-66,110,-54,96,-56,95,48,
            41,7,-14,3,35,-26,67,126,-85,-120,
            85,-112,31,18,-15,57,-28,-73,-117,88,
            14,121,-58,-97,-26,-128,-6,78,100,23,
            -48,26,45,14,4,-75,-31,55,85,65,
            107,-111,-34,28,104,37,94,-63,41,-81,
            -125,-79,-66,-42,-113,-96,-7,-118,-5,-105,
            60,-24,80,-50,-43,20,58,-28,-13,-11,
            117,-58,-126,122,-26,98,2,24,-79,-30,
            113,66,18,-48,-54,59,-69,-128,22,-68,
            104,46,-88,-83,34,-85,-127,-42,-20,79,
            111,-125,58,55,98,25,-28,25,17,27,
            -64,-52,55,47,7,43,-41,-92,44,-56,
            15,-100,18,14,-7,81,-67,-38,-127,-66,
            91,-7,11,-36,-11,50,-53,57,24,-15,
            -84,-28,-80,-80,-63,-111,-14,113,17,27,
            72,78,20,-9,-46,87,-95,14,-69,-93,
            -50,-100,-85,76,1,74,-73,-106,3,-51,
            -59,-33,-79,48,14,-127,-72,-109,-41,30,
            -8,70,101,-89,69,-91,-111,22,30,-66,
            -34,16,9,-38,-79,-86,-2,-112,31,37,
            -114,-44,-53,16,-43,-39,49,13,-52,110,
            -6,60,-96,119,-124,5,-30,12,121,98,
            36,-28,-65,-75,-47,31,52,-2,-58,-23,
            65,-10,42,49,-91,-69,45,-22,3,-25,
            55,-53,-94,-119,114,-77,-103,-113,-72,-55,
            -96,-22,-14,1,-32,103,90,-58,118,-25,
            113,8,-2,-115,-83,-98,119,63,100,-33,
            -29,16,-1,-95,-61,123,53,76,-109,121,
            -127,-57,12,-121,-20,125,-7,111,-126,-74,
            -29,-121,-37,-96,101,-90,56,65,75,76,
            25,12,90,82,74,119,-48,-114,122,-120,
            -109,82,20,48,-65,63,9,-76,35,103,
            -81,-128,118,62,-26,101,80,-21,62,107,
            5,-83,54,27,-4,45,-105,-9,-126,121,
            -26,-6,-62,-94,-76,46,63,-128,36,-71,
            -86,-126,124,-58,-75,3,-28,-8,55,-38,
            -125,89,-126,-118,-80,-12,21,54,-7,-54,
            42,-118,76,123,-106,-128,-42,39,-26,77,
            80,127,115,54,21,-76,115,41,17,-96,
            125,47,-14,-59,-13,-23,-87,28,67,64,
            75,74,-42,65,75,76,14,6,45,35,
            -1,47,-96,125,61,122,42,100,47,23,
            103,-6,74,-120,124,-45,54,83,-111,-9,
            -96,125,81,77,-5,19,81,-9,-1,25,
            -40,-110,-52,119,-28,-37,20,-12,13,-114,
            87,-64,-20,108,-5,-125,25,111,5,51,
            94,15,102,-68,1,-2,71,108,-67,-79,
            105,-58,58,-104,103,-124,-11,-5,28,-86,
            -78,-118,20,-41,122,-27,6,-92,94,119,
            -90,-62,-32,86,21,-34,3,-13,-52,118,
            -127,-110,-30,-2,-65,9,-46,100,-9,112,
            89,-118,116,54,3,-27,96,-59,-21,96,
            126,-17,39,24,26,-89,92,-126,-53,-29,
            108,125,-120,-84,-76,4,-86,-14,124,-120,
            124,-35,47,-1,-68,-100,-37,-64,-116,123,
            -126,121,0,-123,-125,89,79,93,65,-113,
            17,-5,-77,-2,-49,-61,-106,111,108,67,
            -81,61,-123,-21,-3,-2,-127,-35,49,-53,
            -63,-123,87,71,-77,83,51,-119,-3,-60,
            -16,-37,-49,-40,-44,-56,-103,-121,-116,79,
            -70,127,40,117,-128,-91,-109,88,-63,-20,
            -11,76,123,-86,121,117,-124,15,-53,-6,
            28,-125,-63,83,-27,-109,-32,-89,42,71,
            0,-1,4,6,28,124,82,115,-109,-121,
            114,36,-56,53,-64,-31,-89,36,-125,-71,
            93,14,-8,31,-10,-39,11,67,78,121,
            -75,-126,15,-41,-78,-3,-78,114,39,-106,
            99,-7,90,47,-14,89,-92,114,126,-53,
            -46,73,98,-73,1,-47,-83,109,23,-84,
            -32,-62,-11,-2,-16,-113,9,126,-19,127,
            66,100,-1,95,58,-20,126,2,79,-110,
            -84,80,-57,-10,119,-89,45,-74,-51,96,
            56,-40,13,-90,116,-95,-72,70,81,15,
            25,-54,20,-41,99,11,-5,-109,66,-10,
            127,7,-28,17,44,-18,-55,-30,-66,69,
            -116,67,89,-4,28,-117,89,-25,-18,-8,
            -114,-59,124,-118,-27,-76,-121,3,-25,-105,
            30,96,60,-80,-9,-12,28,-79,44,102,
            -109,-74,-14,21,91,62,66,-117,-87,124,
            61,11,-41,27,-81,-57,-30,127,34,-53,
            95,-114,96,115,93,82,87,22,119,43,
            -90,-8,5,-37,-87,-18,-6,-107,-114,104,
            -105,109,-65,47,20,115,-71,-70,-38,-22,
            -83,-83,84,26,74,67,105,-8,21,-62,
            127,0,-6,12,1,63,-93,-37,-107,104,
            0,0,0,90,122,84,88,116,67,111,
            109,109,101,110,116,0,0,120,-38,115,
            73,44,73,-75,82,8,79,77,-47,81,
            48,50,87,112,44,77,87,48,50,48,
            -80,80,48,-76,-76,50,-78,-80,50,49,
            81,8,112,9,-31,114,44,45,-55,-56,
            47,-78,82,112,45,-54,76,86,112,73,
            85,-16,45,-51,75,81,-80,73,77,76,
            113,-56,-84,-56,76,-52,-45,75,-50,-49,
            -75,-29,2,0,-31,23,21,24,123,-69,
            10,-77,0,0,0,0,73,69,78,68,
            -82,66,96,-126
        };
      }
    
    private static byte[] whiteKing() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,16,-28,
            73,68,65,84,120,-38,-19,-36,121,124,
            14,87,-93,7,-16,89,-98,-56,-86,72,
            72,80,-54,71,107,125,-5,41,-75,85,
            45,17,-102,-110,-22,-126,-118,-68,65,107,
            -85,-91,-118,90,-14,-70,-19,123,-75,111,
            -75,-91,82,91,81,109,-44,78,42,117,
            27,-73,-75,68,93,69,-72,47,-105,114,
            -47,-38,106,75,-59,-106,87,98,75,4,
            -55,-109,103,102,-50,-71,-13,-101,-25,-100,
            -13,120,-122,-89,-126,104,-45,-37,-100,-17,
            31,-25,-103,-55,-52,-103,-77,-52,-103,57,
            115,102,-112,-92,-78,80,22,-54,-62,-97,
            47,-56,-74,-27,8,22,-113,-78,-59,17,
            119,-40,-81,44,-4,70,-63,-97,-59,117,
            108,-79,127,89,-43,-108,-123,63,117,80,
            109,-53,85,-36,-111,-33,127,120,-81,126,
            -30,-122,59,-2,57,-102,-59,93,-40,122,
            -22,-67,-99,-40,-81,-54,29,-114,83,22,
            -18,47,40,-97,122,47,55,-1,95,119,
            76,-87,119,-36,-4,-16,-81,-17,87,22,
            -18,55,60,111,-85,-32,-26,-18,-72,-1,
            46,119,-4,70,125,119,60,101,-63,-19,
            27,104,74,50,-37,-82,-98,-9,126,60,
            29,95,-57,41,11,-59,13,-1,101,-69,
            68,-71,-36,-15,-87,105,-34,13,65,-94,
            111,-33,64,-10,-11,124,63,-98,-114,-81,
            -29,-108,-123,-30,-122,-59,-34,-117,-114,20,
            119,-68,107,-110,59,-66,-76,-45,29,95,
            -8,-8,-10,13,116,97,10,-37,110,-121,
            -9,126,60,29,95,-57,41,11,-59,13,
            65,-73,-65,20,85,96,55,121,7,27,
            70,71,125,113,-5,6,-118,90,-50,-74,
            -13,99,-5,-123,-5,-72,-92,5,-3,-39,
            31,36,-85,-78,-8,51,22,-25,-38,-50,
            -36,-16,-69,123,-96,84,-26,-40,6,9,
            -38,-19,27,-88,-123,-1,-81,-17,119,-57,
            124,-121,-37,-14,-55,-13,61,-25,-113,-10,
            32,108,127,32,-52,98,-93,-41,51,-84,
            98,126,-7,-11,7,71,-123,-35,19,-44,
            121,108,-59,4,-37,6,-27,-68,-121,-61,
            -22,59,-34,127,110,49,-33,71,3,109,
            -76,-115,-90,-33,-30,-41,74,91,-70,60,
            -80,75,-96,-70,-120,-27,-85,-16,-41,123,
            -74,-110,-51,-74,63,-58,-42,31,-67,67,
            -67,-4,102,-63,97,91,-82,-53,50,-38,
            -46,44,50,-28,-16,63,-76,12,7,73,
            -22,-72,6,-44,10,13,-98,-124,-32,14,
            -51,18,64,-67,-46,96,12,-36,-108,82,
            0,-56,108,-12,-91,-20,-72,-23,-36,-124,
            48,41,31,28,89,114,13,48,-41,-25,
            65,-69,58,-22,48,-96,84,-19,15,102,
            3,-99,-128,-74,27,101,127,48,-73,-53,
            6,-57,121,-106,-50,67,-74,19,-28,-128,
            20,8,50,-17,-31,82,-125,68,-112,-92,
            102,123,64,-51,107,-16,60,4,71,118,
            92,6,-86,-14,-44,51,112,-13,-8,-59,
            -94,-77,122,120,-100,-83,-83,-10,91,63,
            111,5,-40,-106,51,-27,119,64,-83,95,
            -18,11,48,91,-18,34,-124,84,9,108,
            6,-3,15,-85,-99,96,77,39,-9,-26,
            39,-86,-102,23,5,-45,-7,36,-9,114,
            70,-114,-93,6,124,123,42,-80,2,-12,
            40,96,-19,63,91,-23,15,120,-26,4,
            -11,97,-7,16,-88,73,-27,-106,-126,89,
            -91,79,66,-36,55,-54,48,-72,-104,104,
            -17,65,-54,104,-72,-16,-79,82,15,98,
            -89,-108,-117,4,-77,-99,-41,-125,-6,-75,
            116,17,-44,-86,-114,23,-64,-36,-27,101,
            112,20,5,54,-121,-40,-75,14,25,86,
            -79,-122,-52,-72,-56,-14,-51,26,-16,-8,
            101,-75,31,-84,-2,48,-80,3,-12,-85,
            -24,-8,6,-126,43,-107,75,0,-13,56,
            67,64,125,-126,-43,-45,-18,7,-35,-77,
            60,9,86,3,57,-48,17,6,74,118,
            104,39,48,79,-96,-118,-16,87,89,-7,
            28,-50,102,-37,-122,-69,-41,42,84,7,
            -86,-43,60,6,-108,-122,-84,0,-54,-82,
            -23,36,67,-86,12,102,15,-24,11,63,
            -83,115,-84,-126,-89,-121,85,92,0,102,
            -127,-33,7,51,-2,10,-122,70,-54,75,
            65,-92,-97,17,-9,20,-112,-42,61,-37,
            1,97,103,46,-11,-109,55,3,-91,-14,
            34,24,-36,64,-18,7,-110,20,-38,0,
            -52,83,-31,12,-76,89,-82,-58,-64,-127,
            -90,-26,-59,-53,36,-46,-3,37,36,27,
            -24,-43,-102,89,64,-23,67,89,64,-39,
            37,-103,-28,-118,19,34,5,78,-25,-5,
            -43,-122,30,125,42,54,1,-77,-53,28,
            -80,84,48,-17,-62,112,-75,-92,27,74,
            -71,-23,54,15,103,-126,98,65,102,61,
            66,105,-94,46,-122,-71,51,-28,87,64,
            20,108,122,-9,105,-32,58,-69,53,4,
            92,103,-13,66,-63,-120,112,-99,3,99,
            -46,-27,-90,96,-44,74,123,18,92,95,
            -74,-53,1,-19,-81,-94,-64,-87,96,54,
            -40,124,120,-3,5,-9,37,-86,77,47,
            41,1,-24,1,-9,118,-58,-100,-59,91,
            64,27,67,-35,-31,52,-117,-49,47,-116,
            2,45,-118,109,-105,38,125,1,-76,-83,
            -76,13,90,-97,86,-41,-61,-16,127,40,
            -119,-32,-23,121,81,127,7,-83,-26,-70,
            75,-32,74,-71,-4,62,24,-11,88,-66,
            -89,-27,-98,2,35,52,-3,10,-72,126,
            -20,-6,9,104,108,112,64,-42,-54,-1,
            0,51,-33,43,-31,-45,13,-84,-10,-42,
            6,60,15,-14,92,86,-113,59,125,-42,
            -13,93,-122,64,22,-113,14,74,7,-55,
            -55,110,8,-83,-44,68,72,93,-49,11,
            22,-47,17,-118,14,109,120,6,116,127,
            86,81,123,-56,22,-80,-86,14,52,-54,
            67,-102,-91,-120,-58,89,120,-104,60,111,
            2,104,-113,-107,-37,13,100,-114,52,14,
            92,-103,-86,37,-113,13,-101,-23,-31,-103,
            -13,65,-33,-53,-10,-53,119,117,5,74,
            93,67,65,-92,-105,51,99,13,-24,123,
            -39,126,107,28,18,92,125,83,-86,2,
            -38,-13,1,31,0,121,117,113,0,104,
            79,-80,-3,38,-46,-106,22,74,55,89,
            92,34,-59,-109,22,23,-39,13,98,-19,
            -82,-17,-66,1,-19,122,-107,-107,80,-108,
            -49,-21,69,-99,9,41,-109,83,-58,-125,
            -39,101,40,-56,108,-82,80,-22,107,-85,
            -25,-69,-18,57,-31,-46,24,80,-2,83,
            -114,0,-77,-53,78,-122,-23,34,3,-11,
            -73,-126,51,-25,116,109,32,79,-14,28,
            107,71,-128,58,-11,20,-96,-41,-55,25,
            -96,-41,105,23,-53,13,-110,14,-12,-70,
            -34,11,-24,53,-35,1,84,-25,-5,111,
            -37,0,70,-81,74,-7,-96,15,103,103,
            40,-115,94,8,122,95,-74,-39,0,109,
            15,-104,-65,-26,91,110,-48,89,-106,-21,
            -38,57,16,21,-104,-16,76,6,-24,-43,
            89,58,105,-107,-57,-126,62,107,-25,81,
            48,70,-118,124,23,0,-43,-11,-127,96,
            -26,123,27,-120,124,23,-112,76,48,-13,
            -67,8,-52,-29,-100,4,-22,-28,-5,103,
            14,7,82,-85,-34,-93,80,20,46,26,
            42,9,18,-33,-106,-83,96,-82,26,6,
            10,-69,18,72,21,-17,-74,39,-79,81,
            -102,50,87,-84,120,15,34,-81,-13,3,
            -122,126,5,-6,-128,19,21,-63,-72,38,
            10,-40,5,-24,69,-102,107,-15,-12,24,
            -35,-30,9,46,-117,39,-52,-76,56,93,
            51,-127,-26,-13,-43,-23,-35,-64,-40,36,
            95,0,50,60,96,40,-48,-41,118,71,
            3,121,-114,111,103,108,6,90,96,-92,
            2,45,-32,-21,119,-98,5,-14,-108,-65,
            3,-24,-101,106,50,-112,-63,-1,109,-128,
            113,76,100,103,1,-48,43,116,-87,-59,
            -23,51,-97,-10,114,100,91,52,-83,53,
            -48,75,124,-11,-79,-93,96,28,-81,-76,
            29,-12,125,-68,-34,28,107,-96,-11,18,
            91,127,-8,-69,-113,-47,-79,-49,-48,-108,
            -19,-40,94,52,80,15,88,-107,-59,15,
            -108,-12,6,-72,-10,-117,114,108,0,81,
            49,-124,94,-77,-36,57,16,11,53,-85,
            -60,121,83,-75,104,-82,-71,32,-106,-113,
            37,-66,6,-58,69,126,-4,-10,-21,-127,
            68,-77,-65,-73,35,58,-104,-65,6,90,
            40,-21,73,-11,-37,70,3,121,-111,-19,
            103,76,-39,4,70,54,63,-31,93,73,
            64,13,81,-3,-123,22,79,-66,-18,20,
            -68,-53,73,92,-85,-64,-20,-55,44,124,
            62,16,92,-109,68,3,-67,13,43,-65,
            -74,53,-48,-69,-20,71,-93,-30,78,4,
            -120,7,-63,-121,-66,-123,106,108,106,36,
            -1,-79,-86,115,-128,-46,-85,-125,-64,-56,
            21,89,107,104,-71,94,-20,-122,-15,21,
            120,5,-59,91,10,-24,62,11,117,30,
            5,42,55,62,7,-28,109,86,-32,31,
            87,78,5,18,101,79,102,-59,38,48,
            -2,-59,-74,-37,-46,36,19,-56,-108,-94,
            122,64,-125,-23,14,11,-91,-3,44,5,
            -52,-67,7,94,-18,122,22,-47,64,87,
            27,-125,-111,93,53,22,-8,96,36,55,
            -65,-30,-61,16,-98,111,-85,-9,65,-59,
            125,79,38,30,-88,-54,119,-127,14,19,
            -7,25,-48,45,13,-12,88,118,-4,62,
            70,5,-64,-72,-55,98,-48,-5,13,-74,
            51,87,-33,8,86,-109,89,97,-43,33,
            32,-20,65,-112,-82,-17,-48,29,72,111,
            -10,-25,119,88,-4,70,-28,56,32,61,
            -39,118,-87,107,-69,2,9,17,-23,-18,
            7,79,69,22,-69,-57,-36,41,-80,122,
            48,42,-126,88,-5,106,-73,-55,-96,127,
            -64,-21,-79,-4,50,104,-1,-93,-83,39,
            93,47,110,3,-91,-15,95,-63,-57,32,
            -2,7,-98,112,66,38,104,65,-20,-64,
            -121,53,2,-73,92,-94,-18,63,-16,-12,
            -36,55,127,39,-3,-38,66,-55,65,-96,
            53,-102,100,1,97,-77,-43,-76,-33,-15,
            46,64,-26,28,-35,10,-28,19,-74,-2,
            -43,-26,-37,-128,36,-118,126,30,101,-95,
            52,-43,82,-12,-96,-14,45,-22,-59,29,
            14,37,28,7,45,-124,-41,99,112,14,
            -60,119,-79,-43,-5,-22,-30,54,-48,50,
            -2,43,36,6,-30,-86,-14,-124,-57,-19,
            5,-83,35,59,-16,81,45,31,110,115,
            51,45,-31,2,-21,-19,-63,-77,118,70,
            43,32,-95,44,95,71,102,42,64,-126,
            102,80,48,-8,-28,-23,-66,79,-9,2,
            -87,35,122,-50,115,96,-10,-12,-110,110,
            24,-37,-96,66,-44,11,-85,-89,113,-5,
            64,123,-119,-41,99,72,111,-24,-71,-64,
            86,-17,-59,125,-19,-95,28,23,-9,-96,
            12,104,123,-128,39,-36,51,16,-12,77,
            -20,-64,-3,-11,44,40,-63,75,-100,-113,
            2,-109,32,-16,-116,-18,-114,-42,0,50,
            -124,-27,-21,-125,-106,-39,96,100,-74,-72,
            1,-30,-34,-13,-42,47,-107,-128,-68,47,
            -82,100,-83,-64,-68,91,104,-106,-110,15,
            -84,30,-12,-13,-32,-71,-60,-59,30,3,
            93,12,-78,42,-44,-121,54,-74,87,-16,
            -54,-66,-30,14,18,-6,-13,95,97,-13,
            32,124,-90,123,41,-81,83,-83,-23,96,
            -98,-62,-27,-63,-56,18,89,120,-55,82,
            -16,-64,10,-18,-18,26,69,-12,-96,-123,
            -121,-26,-115,47,2,-31,-17,-125,-40,-32,
            -122,-114,111,-47,14,-56,36,-74,93,20,
            125,-42,66,-23,50,75,-47,-125,58,-111,
            -24,-117,22,113,-49,44,-36,15,-58,-91,
            90,-47,32,6,9,-63,-107,63,-125,42,
            -3,108,-11,30,91,-36,97,-10,83,-84,
            69,-59,-5,13,-27,50,-84,79,-29,103,
            64,-38,53,-48,26,-119,75,71,14,-104,
            55,93,-5,115,66,73,5,-9,40,-119,
            -24,93,64,-84,-35,59,-84,39,-112,-102,
            -10,-55,-46,81,127,3,-14,40,-17,112,
            -6,-29,96,-2,-70,97,33,37,-98,63,
            86,110,-3,20,120,6,31,105,50,104,
            -43,-60,3,107,121,88,103,-69,-9,40,
            13,-67,31,111,-118,-3,-96,-86,14,20,
            -77,122,111,65,-17,75,-4,64,29,-89,
            -128,43,-109,-27,-29,105,35,7,-52,95,
            7,44,69,-26,-128,27,74,-66,2,-38,
            -128,-25,12,93,-8,18,16,-10,-95,34,
            -3,-119,-59,-1,92,-74,24,72,67,-79,
            123,28,-104,61,-36,-80,-108,92,-32,-27,
            -36,99,41,34,1,32,-2,-38,-54,-102,
            -20,14,-44,34,120,-67,-7,39,65,124,
            -104,-19,-34,-45,-13,110,31,84,-7,-108,
            -61,26,-15,-58,-84,-95,37,74,57,9,
            7,-73,-15,3,46,63,7,-102,120,-14,
            -47,-61,-63,83,-127,37,125,51,54,-38,
            -128,-25,94,-76,-67,51,-112,127,99,-7,
            89,-54,-30,73,63,116,5,50,66,-20,
            -41,31,-24,-43,18,31,101,-14,114,-5,
            -127,-89,-36,-55,31,-125,-26,-103,-109,-13,
            -125,-3,61,-4,91,-127,-29,-72,-83,-66,
            83,-18,117,-46,-44,-49,-3,-126,75,21,
            -17,-127,-44,-97,32,86,124,-92,81,-43,
            15,-76,-60,115,21,-127,124,40,110,-58,
            -27,-128,-26,121,-26,6,74,-26,-34,68,
            26,-127,-89,-127,78,-74,5,-14,-107,18,
            6,116,-128,-93,61,-48,-105,78,17,32,
            43,-60,126,-99,-63,-77,-33,61,7,91,
            57,-116,11,-32,41,-25,-23,-13,64,-58,
            -122,87,2,77,-100,-56,-22,-9,-48,125,
            -99,-9,11,72,-91,72,-44,-13,61,6,
            -2,74,-72,-82,-36,18,-108,78,-54,117,
            48,-5,-30,39,-112,34,-34,-5,-60,84,
            2,-105,-54,39,39,-115,76,48,127,53,
            -80,20,-8,-100,-53,-78,63,32,-2,96,
            -47,-56,1,-96,55,-116,-63,64,-13,-11,
            72,-96,5,-82,-49,-63,51,87,113,97,
            63,-112,-20,-32,-121,-128,-74,99,-17,111,
            58,94,106,7,68,12,6,92,-53,-127,
            94,-29,-23,-16,116,-7,113,-8,113,125,
            -26,-53,-98,-17,42,-106,27,-92,33,-104,
            -101,-17,0,58,48,58,25,60,-77,-39,
            -114,-119,-16,101,77,101,57,-104,-19,18,
            14,-54,80,86,-81,-75,125,-68,122,-65,
            -41,-122,-110,109,115,69,97,79,59,122,
            -61,-119,-90,60,67,99,-86,-128,86,-97,
            21,-29,-97,90,58,-32,-39,-34,-30,105,
            -88,49,22,-89,-47,30,104,62,-37,-50,
            32,113,112,-101,107,60,15,53,45,-94,
            -127,10,27,3,89,20,58,25,72,126,
            -40,18,32,-25,10,-13,-128,44,20,-5,
            -43,-75,92,-13,-107,46,63,46,-49,7,
            -49,23,-49,-89,-40,-48,61,53,116,67,
            59,3,98,-19,-26,-111,61,64,11,20,
            13,-13,6,28,99,115,-103,-107,-74,-37,
            70,107,-43,75,-86,97,110,-3,6,33,
            20,-108,69,98,121,0,52,77,83,87,
            -64,53,-15,65,-31,-28,89,-96,7,-117,
            43,67,36,80,77,-37,14,102,-127,-7,
            -80,-105,-121,117,22,126,-113,-8,91,94,
            34,-112,-62,61,59,-128,124,-70,98,4,
            -112,33,-109,-73,0,25,60,-22,32,-112,
            41,-67,-77,-127,68,4,116,6,58,62,
            32,18,-24,107,-67,-49,3,9,24,117,
            0,-56,-12,-55,-23,64,6,-83,24,9,
            100,-56,-98,-99,64,102,-15,-29,-16,-98,
            111,-53,7,-27,-7,-44,-74,0,117,106,
            -35,-64,-45,-105,38,-66,2,-102,83,92,
            -54,118,64,-2,32,-87,21,52,-15,124,
            83,81,1,-108,-23,119,-8,-74,-29,-66,
            67,-80,-8,53,4,-44,37,-46,110,-117,
            36,-113,-128,24,-86,118,0,-99,-99,25,
            -28,-85,-60,38,-96,-15,2,29,33,69,
            64,11,-76,-55,64,91,-18,-102,11,-28,
            -61,9,111,2,-119,109,59,6,-56,-46,
            -96,120,-96,-20,-77,12,-102,34,-115,0,
            122,-120,45,-97,96,49,-85,0,-70,-121,
            -59,-20,57,-126,-58,-79,120,-81,109,-69,
            19,-74,116,82,-126,122,1,-115,102,-57,
            93,50,97,12,-112,-40,93,-13,-128,124,
            -96,125,4,-76,37,-107,45,-68,39,-19,
            -7,40,20,116,-2,38,-11,-112,58,16,
            -76,88,57,26,58,119,-110,-26,89,36,
            -87,15,-88,-97,-7,-84,-57,7,20,66,
            -60,47,-21,-93,14,71,-124,50,4,-52,
            -122,-38,15,-35,-58,-87,-49,-126,-2,48,
            -85,-120,-39,-93,67,-63,8,-97,58,12,
            72,116,-19,-115,64,50,-40,-33,55,-55,
            -43,-128,94,97,-53,-55,-31,125,-128,-106,
            -17,124,28,-56,-78,-124,-53,64,70,37,
            -75,1,-46,125,85,18,-112,-111,-23,95,
            2,-103,-72,-11,36,24,100,-85,1,68,
            78,-1,8,-56,-121,-85,22,0,121,51,
            -87,53,-112,-105,121,58,-99,51,-128,-16,
            -29,-16,57,-78,100,91,62,54,-43,-2,
            30,72,-58,-44,119,-127,-60,-116,74,0,
            -125,95,-54,82,-44,30,-96,-83,-109,45,
            93,-101,42,-51,-64,-4,-45,14,112,84,
            -14,89,111,-65,-11,-9,112,114,121,112,
            68,-86,-83,-63,92,78,-122,-25,-30,-108,
            -73,-32,-58,-49,82,107,16,-33,44,-28,
            42,23,-128,-114,-115,44,4,-14,-2,-84,
            120,32,113,-5,-33,1,-78,-48,121,10,
            104,4,59,99,103,-45,-43,-106,-20,91,
            -26,-66,-19,-113,-77,-18,112,-59,-25,118,
            -34,-23,-52,-26,-57,-39,-1,46,-112,69,
            -77,122,1,-119,-117,116,2,-103,-64,-13,
            -55,-14,-99,47,-75,3,74,89,-71,-110,
            -27,-87,16,-109,-94,6,1,-38,12,28,
            -83,75,-53,-9,113,-14,45,-33,123,-83,
            7,-57,-65,36,-89,69,-14,-33,12,49,
            115,-7,53,-6,47,-57,-63,56,122,118,
            11,-112,58,108,-16,48,-111,126,103,113,
            -78,111,23,68,-59,-78,55,-91,84,-101,
            2,-108,-24,-37,-63,51,-74,-70,-86,2,
            -55,24,-6,34,-112,-86,125,-61,-128,-124,
            94,-119,0,-62,95,40,18,-3,19,-96,
            68,-101,6,102,-70,-101,-32,-90,6,-28,
            -57,117,-25,-93,-112,-25,-21,-20,54,32,
            53,-1,-110,1,-122,-104,-109,-12,95,7,
            49,-29,-91,28,-117,36,-83,6,-57,57,
            31,-9,-104,-33,-3,-117,83,-39,123,60,
            -81,-120,-41,21,65,57,-48,86,-25,5,
            107,-75,13,-76,35,100,-77,101,-86,120,
            -98,120,12,104,-95,-47,17,-52,-47,83,
            -116,-91,-128,-59,-98,-71,-83,-27,64,-57,
            -52,75,5,-46,-93,102,2,-112,-17,89,
            -6,-105,89,92,-8,-16,5,32,-23,-117,
            123,1,-119,115,-43,3,58,77,52,73,
            95,-117,-109,31,-121,31,-41,120,20,60,
            -57,35,-85,44,-117,90,45,4,-99,-65,
            -34,-96,65,-25,-95,-99,-19,-101,110,101,
            -83,-19,-71,-90,-44,125,10,-52,50,-92,
            -26,-34,-44,-71,77,-61,-110,-28,-122,-64,
            47,113,69,-121,-21,-108,3,-110,-104,26,
            5,-28,21,-83,18,-48,101,-94,2,63,
            -77,-24,-92,49,120,30,48,-113,79,4,
            82,-73,-58,119,64,-60,-5,21,-65,33,
            64,53,-57,6,-96,-30,91,-19,-22,-61,
            -127,-124,28,115,-128,-104,-101,-93,-92,14,
            -104,-23,-78,-29,-120,-47,102,40,-48,-28,
            -44,-50,64,94,-83,19,12,100,-86,59,
            61,45,94,110,4,102,57,-36,-27,90,
            104,-101,-78,-55,45,-19,-33,104,-77,51,
            71,61,-17,-67,122,-10,-1,72,-33,-126,
            121,-19,-2,25,-118,78,-118,75,-59,74,
            32,61,-45,95,3,99,-98,-24,81,-77,
            -64,-20,65,-4,-127,-15,101,-117,120,-29,
            -28,-4,17,104,-83,127,-17,3,70,-127,
            123,10,-118,62,83,110,37,-48,-90,-17,
            -98,2,-61,40,76,6,90,-53,54,-21,
            -82,-15,116,-59,113,-8,71,42,-81,-125,
            -79,-60,127,53,-112,1,-30,59,-67,-125,
            -96,-7,-13,114,-80,114,-91,-37,26,-24,
            -4,-3,-50,12,60,-24,-64,62,117,85,
            26,-120,53,61,96,99,51,41,21,120,
            -63,92,-17,-59,47,3,50,-30,-20,123,
            64,-66,21,103,118,14,-120,39,119,-62,
            102,-97,61,55,11,-9,-25,90,-41,-116,
            83,32,86,-1,-68,-37,1,100,-44,79,
            77,65,-68,65,61,77,2,-63,-36,-17,
            2,-36,-108,18,-1,6,-127,29,71,28,
            -105,-123,-77,-29,-127,124,31,-65,14,72,
            12,-21,65,-35,-92,-81,-63,44,-121,-69,
            92,-75,109,-105,-72,6,-34,-11,80,-6,
            -126,-72,-9,-44,93,10,126,-20,-67,-46,
            73,-121,127,35,-96,116,-61,19,-96,-11,
            -25,-81,-52,-55,43,96,-98,-55,51,-128,
            94,36,-35,-128,94,33,81,64,115,73,
            71,48,-29,-42,96,-58,-93,-128,-26,-79,
            123,69,-106,86,29,-52,7,75,-9,7,
            -123,-108,-82,-75,80,87,46,-104,-37,61,
            2,52,-117,-116,4,-102,39,-46,-31,-23,
            -14,-29,-80,-29,-118,124,-12,6,-49,-121,
            -119,27,84,-48,-26,-7,-41,6,126,-94,
            -3,-94,-41,125,27,-4,6,-7,-86,-121,
            -46,118,11,90,-59,127,85,-34,1,-113,
            -80,119,-20,-50,-18,-51,30,5,-47,15,
            -66,-96,45,44,-108,-82,-80,60,-120,-105,
            -27,-9,23,120,-66,-102,89,120,-66,-25,
            55,107,2,-68,-127,-100,-35,43,-17,-126,
            71,-42,-8,-86,-121,82,22,-108,-117,-30,
            -119,108,9,-76,23,-1,-22,-32,-123,-7,
            80,52,-10,114,1,104,-113,103,118,6,
            -19,-48,-103,4,-48,-74,-99,25,11,102,
            60,14,126,-57,-104,-25,-125,-59,60,-97,
            60,-33,47,44,-128,-94,-41,-59,-73,5,
            94,-27,-68,-75,30,74,89,112,120,70,
            47,81,48,124,-70,-104,-93,-118,7,61,
            -52,-81,30,-104,-93,45,107,54,-100,82,
            -57,74,40,-59,49,-53,39,-49,55,47,
            -121,120,-125,-21,46,-25,84,-97,-11,80,
            -54,-122,-41,77,-68,87,-113,110,-23,-114,
            15,-77,-71,-80,-19,19,-36,-15,78,54,
            -59,-77,-13,8,-117,-113,-106,-14,-8,-120,
            119,-66,121,57,120,-71,70,-37,-2,89,
            -65,-88,-121,82,59,-36,-26,-1,-126,109,
            -85,-3,123,46,-23,-1,71,-80,-105,99,
            -85,-83,-36,-91,54,-40,-57,-1,-20,107,
            27,121,48,-117,-39,112,85,126,-15,15,
            30,-57,120,-105,-117,-105,-45,103,61,-108,
            -74,75,-99,8,-4,5,31,-1,-118,-123,
            -3,-57,18,-46,-77,127,-16,56,-38,86,
            -82,70,119,-88,-121,-78,80,22,-54,-62,
            -17,18,-2,15,-80,-87,55,9,-124,112,
            60,59,0,0,0,90,122,84,88,116,
            67,111,109,109,101,110,116,0,0,120,
            -38,115,73,44,73,-75,82,8,79,77,
            -47,81,48,50,87,112,44,77,87,48,
            50,48,-80,80,48,-76,-76,50,-78,-80,
            50,49,81,8,112,9,-31,114,44,45,
            -55,-56,47,-78,82,112,45,-54,76,86,
            112,73,85,-16,45,-51,75,81,-80,73,
            77,76,113,-56,-84,-56,76,-52,-45,75,
            -50,-49,-75,-29,2,0,-31,23,21,24,
            123,-69,10,-77,0,0,0,0,73,69,
            78,68,-82,66,96,-126
        };
      }
    
    private static byte[] whiteKnight() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,15,117,
            73,68,65,84,120,-38,-19,-36,121,116,
            20,69,30,7,-16,-18,-23,-103,-112,-125,
            43,-128,-124,67,-82,-128,64,60,32,-117,
            32,-120,-78,-122,0,-118,18,20,-106,75,
            -111,99,57,-28,80,20,-39,21,112,5,
            68,20,69,-120,-69,28,10,-56,-109,-61,
            11,36,-128,-128,1,87,22,4,1,-123,
            -128,40,-41,66,56,-27,-122,64,18,-82,
            36,36,-52,116,87,-43,-10,-73,-90,-86,
            38,-45,110,-28,-19,-109,-112,-24,-90,62,
            127,84,119,-49,100,-90,-69,126,93,71,
            87,-9,68,-45,74,82,73,42,73,37,
            -55,-103,74,57,-42,55,-8,51,125,-88,
            99,123,-23,-110,-94,-70,-75,-55,-27,88,
            127,64,4,102,-107,88,-49,22,121,-101,
            27,-4,93,73,42,-92,20,42,-14,126,
            -63,-101,-53,117,-15,-25,-75,26,57,-34,
            127,-58,-15,119,37,-87,-112,82,-124,90,
            10,7,-73,-95,-107,-31,80,-12,-74,105,
            95,-39,117,-60,-106,-100,99,55,126,-91,
            120,3,-24,1,-41,58,-15,87,-73,-117,
            92,47,41,-54,-101,-101,-62,-43,82,12,
            -72,63,8,126,-71,125,29,87,79,96,
            76,91,7,121,19,-3,-37,-21,-73,13,
            126,-97,-2,67,1,125,87,73,-6,31,
            -109,-37,-79,-98,-96,85,4,-9,-102,-32,
            -51,-43,-85,-22,107,-32,108,-108,-74,26,
            24,-43,50,-63,14,20,79,83,-90,-28,
            -5,68,-101,-15,-111,88,11,41,-87,73,
            -1,91,-110,5,21,-106,111,-117,-51,117,
            -101,-2,2,24,-13,60,19,-63,46,-25,
            -91,80,-91,73,72,43,-8,-82,-101,-1,
            -51,-116,117,-119,1,111,-21,42,-23,32,
            3,116,62,50,108,23,84,-84,-17,24,
            43,124,94,64,-97,86,-110,28,41,-60,
            -79,62,75,107,15,-58,-12,-112,111,64,
            -113,-86,-44,0,-20,55,78,-123,-34,7,
            -36,-17,-62,-91,-63,50,48,-115,-86,-127,
            -103,-53,120,-94,-1,-18,127,4,124,41,
            -14,-11,-80,68,24,-34,65,-97,12,118,
            -36,91,-126,-69,-125,-88,81,-67,-60,-9,
            14,116,-20,-121,-25,-1,-67,-90,-124,7,
            55,61,-82,-89,92,-61,-63,-72,43,-84,
            33,-40,-99,-60,116,-88,-7,-84,-89,60,
            -84,-34,-88,-115,4,89,51,-56,95,6,
            -24,96,38,101,-60,1,-117,102,34,-91,
            124,6,100,-77,12,-112,-66,23,114,27,
            -107,42,11,-109,27,-69,-102,64,5,-105,
            -47,19,-20,29,-118,7,35,90,-20,94,
            86,-127,-125,-110,-1,-109,-64,4,-6,-106,
            -15,-32,58,19,49,4,-76,17,-30,109,
            -61,66,-58,-63,-13,-27,12,13,-78,62,
            -105,5,29,19,9,-90,123,-45,67,96,
            45,17,-15,-56,34,125,-128,121,-55,94,
            -112,97,98,51,-98,109,13,100,-93,8,
            -24,32,-83,41,48,102,-44,-124,-12,110,
            -18,100,-24,111,12,-66,10,118,-37,122,
            28,-12,-70,90,121,112,93,43,-96,79,
            -4,-35,-10,89,106,-44,-92,-9,3,-41,
            5,119,60,-40,-67,65,109,-120,41,-27,
            -18,1,41,79,-54,-128,-124,36,0,9,
            123,-75,58,-8,-10,-25,-75,6,86,75,
            70,-64,26,0,-20,26,29,9,-20,-102,
            -40,76,105,5,96,94,-79,126,112,-33,
            -25,64,-110,30,127,29,124,-29,-4,-97,
            79,117,125,38,-40,1,-101,14,107,-101,
            -118,65,7,83,123,-36,16,-116,-3,98,
            45,-22,-9,118,-63,-21,28,-123,-75,-41,
            58,-125,-25,105,-67,23,-40,91,-42,66,
            -89,126,-58,12,-56,-3,70,6,-90,101,
            87,-16,-67,-71,111,7,-112,58,-86,78,
            84,-31,-82,-48,-18,-64,-68,-12,21,80,
            -127,9,-92,79,57,-109,124,5,-10,-21,
            21,57,-103,-74,-81,122,12,-84,-124,42,
            -119,-32,91,33,-65,-41,-104,0,-103,29,
            -3,-21,-19,82,-43,-98,87,0,119,37,
            -79,86,-75,-128,-106,-31,55,120,-67,-30,
            -33,-1,99,122,4,-72,95,117,117,2,
            77,51,62,-127,123,-54,-72,-101,64,110,
            115,89,64,-93,22,-128,-17,65,-46,14,
            88,119,89,-94,-33,-99,5,58,-30,-55,
            101,64,-14,14,87,2,-6,-82,124,-99,
            14,6,-26,83,33,-96,-100,-86,104,-28,
            25,96,89,-42,61,-96,106,22,59,127,
            16,104,-46,3,99,-64,-9,-91,-40,-113,
            -86,70,7,96,61,-4,-21,-3,-57,6,
            31,-113,113,-22,-73,54,-6,115,-42,-108,
            -74,90,3,48,-122,-124,-116,6,125,92,
            -123,-99,96,-113,-62,106,67,-49,-127,-98,
            63,66,-26,123,-94,-87,89,49,126,58,
            -104,-57,68,-71,-55,-36,-69,-84,58,-48,
            86,-31,19,-128,69,-55,64,-42,-103,10,
            52,-15,66,93,-96,-37,84,92,-26,66,
            -66,26,101,113,-127,116,-116,-13,89,101,
            -128,93,-108,-101,-13,-26,3,-117,106,-3,
            58,-104,114,16,-46,-57,117,9,-28,-96,
            100,-32,99,-114,64,45,40,-18,-125,9,
            103,77,-71,-96,15,7,-29,51,-49,-33,
            -63,-114,92,95,-88,-78,-59,83,27,86,
            -72,-11,63,-126,26,-123,93,26,-76,4,
            -84,60,-21,57,96,111,-118,-14,-70,58,
            119,7,-112,-57,93,-125,-127,14,-16,28,
            0,-10,84,-7,62,-64,-22,-118,64,53,
            31,62,9,-120,12,65,-102,53,8,80,
            -30,28,-3,89,-45,-105,-61,5,-30,119,
            21,88,-90,92,-49,-40,6,-12,80,-12,
            120,32,-43,-60,9,-44,76,-33,4,108,
            -95,127,-3,81,-61,-47,21,101,22,-105,
            -66,-55,89,83,30,-45,98,-63,24,22,
            50,27,-12,103,42,126,11,118,77,-119,
            -126,94,45,-116,-17,-32,-30,-33,-28,-103,
            95,-13,99,48,-61,-41,76,3,-21,85,
            81,46,23,-84,84,-80,-105,-6,113,95,
            31,-37,12,52,-3,-52,4,96,-6,-71,
            63,3,-35,120,-22,73,-96,105,39,42,
            1,-99,-97,54,5,-40,84,-15,57,-71,
            108,33,-9,-13,-102,-29,76,-41,-72,64,
            -96,26,3,-69,34,-41,-65,78,3,-38,
            65,-100,72,49,122,20,-40,39,86,95,
            56,-41,84,-21,9,-43,110,119,-100,-96,
            61,-118,-86,-55,11,15,-66,-78,-41,-77,
            -11,57,96,124,28,-14,10,-40,87,116,
            119,67,-43,-13,-98,-122,-80,114,-85,-74,
            17,84,77,-103,51,-28,93,-16,109,-49,
            -38,8,116,-65,42,-104,-41,-63,62,-89,
            19,56,-43,55,-80,-21,28,81,61,-54,
            47,39,82,-32,43,-2,-102,20,72,58,
            -105,-61,22,115,-90,-13,115,-52,107,-96,
            -42,23,14,91,11,-76,-95,-1,56,-52,
            84,-93,21,-56,-29,90,-112,-31,-88,56,
            115,-59,66,-12,-83,26,60,4,-90,92,
            -106,-125,-34,56,-92,45,-24,-67,43,53,
            1,-69,-90,116,-121,62,-91,-115,36,-72,
            52,74,-42,-108,-24,100,-16,-27,109,-88,
            14,-42,108,121,-34,-110,-71,96,7,-26,
            52,-80,-85,34,16,-127,-126,-14,113,-124,
            -18,4,-106,67,95,4,-106,77,-121,-125,
            -99,-1,25,-20,-19,49,-64,-82,-46,-93,
            -112,111,112,-32,-27,10,74,-108,-107,-30,
            -14,-40,52,46,-16,-67,89,28,-109,-33,
            43,55,95,-8,2,-24,-110,42,21,-128,
            -118,-47,29,-101,-84,-73,1,-78,69,-69,
            3,-102,-4,-37,17,-88,-42,-73,106,38,
            66,-100,1,-6,-3,-63,-101,-85,81,-3,
            16,36,-65,-83,-51,2,85,83,-34,26,
            -66,1,76,87,-10,7,64,-49,-85,-102,
            -14,16,-40,-41,43,-97,66,-66,-58,37,
            -105,-69,-7,-55,-28,24,107,-49,-87,-90,
            -21,92,60,-112,11,121,-121,-64,-82,-77,
            50,109,-32,84,104,-83,59,65,-67,-102,
            57,-23,0,-112,19,-2,-29,-76,106,27,
            15,-126,125,-36,-107,97,86,31,-57,-32,
            97,-96,26,-100,23,110,114,53,80,-117,
            81,112,-25,26,125,20,92,-66,95,-42,
            -108,-6,35,-64,-105,-66,-29,67,-80,102,
            -118,3,106,72,56,-106,77,-62,65,-123,
            -64,-52,-41,40,-111,95,104,-104,126,125,
            -14,15,6,40,9,-123,-64,55,-115,25,
            15,-92,-50,-113,71,-128,-86,57,7,-6,
            16,-80,108,121,-62,-48,-77,16,56,-111,
            -46,121,-94,95,-35,118,16,72,11,53,
            -123,52,26,78,-50,-41,-54,66,-60,16,
            71,1,102,20,114,-128,12,53,90,9,
            111,6,-83,-41,-55,29,107,17,11,-34,
            22,-30,-16,-26,127,49,22,-24,-6,-112,
            -65,1,125,119,-23,-9,64,-53,-87,120,
            -44,7,52,94,-123,28,24,-39,-60,37,
            114,-41,-43,-9,-97,5,22,87,-49,0,
            114,102,125,75,-96,107,-44,-21,-19,-63,
            110,114,-125,7,17,-44,122,30,-44,-42,
            -81,6,30,2,114,80,-116,-18,-34,50,
            -26,-127,93,-109,-62,32,-18,17,-57,9,
            126,-78,-112,-5,34,-41,28,-71,84,126,
            51,52,59,38,3,-44,-23,20,-112,-119,
            34,64,23,-5,-67,6,84,-76,-71,-12,
            -36,-42,-15,64,-5,-85,2,88,6,124,
            -32,75,111,-40,-27,-1,-102,38,77,54,
            81,-29,-128,93,-106,-21,-85,71,3,-115,
            -41,94,2,58,-2,76,3,-96,-22,58,
            -120,62,13,118,13,10,30,-92,48,43,
            17,2,-115,-16,-94,80,-96,-30,-74,-123,
            -7,-75,-31,1,59,64,124,74,104,-52,
            82,-57,-96,55,-94,-112,-121,-35,-82,46,
            114,41,50,13,26,-67,33,3,-44,-79,
            45,-48,71,-52,-35,64,-74,108,-97,11,
            36,58,-25,60,-48,93,108,34,-25,101,
            71,-72,-32,-21,-5,-4,1,-14,15,-121,
            -87,-24,3,76,-42,-108,-53,-91,-119,96,
            15,6,42,2,-69,66,63,0,-69,-47,
            90,-53,5,66,49,-108,-69,78,103,1,
            -53,-74,-30,-127,101,-112,-65,67,96,-126,
            40,-74,46,-48,97,-115,-106,3,-99,44,
            -74,119,99,81,-100,-100,26,-6,-39,76,
            4,-43,0,-61,7,127,58,-80,27,-24,
            123,-6,46,48,127,-108,-27,-95,-91,-62,
            114,-30,40,-65,-76,2,-90,-122,110,90,
            -128,84,19,23,54,7,-22,-117,91,-56,
            -12,74,-125,75,64,-122,52,91,13,-106,
            120,-100,-119,-100,91,59,20,72,-116,58,
            -79,23,0,-69,-92,14,123,13,-25,19,
            23,-120,-39,-26,21,-80,95,88,9,-7,
            2,39,7,15,87,-71,64,-71,-75,5,
            -5,115,79,3,75,99,-19,56,-109,-99,
            -25,76,-6,40,-88,-73,63,55,-90,11,
            -112,60,81,-112,125,23,-98,-125,64,-45,
            107,-83,5,59,48,-114,26,-88,82,44,
            -89,-6,-94,-68,17,64,89,13,13,-84,
            113,-86,47,-54,-126,125,5,-115,-38,-58,
            22,-42,32,46,57,-33,-75,-79,-83,-26,
            81,127,-98,-5,-96,-38,-79,-5,-32,122,
            -100,127,26,63,-73,-98,39,3,24,91,
            -5,1,-112,117,42,80,-57,-128,-27,-111,
            -51,16,-24,27,-40,22,-18,9,-47,16,
            38,-5,38,-127,-75,-29,66,75,-80,82,
            -114,46,7,18,-111,30,3,52,-127,-51,
            -32,-58,-87,-49,-19,1,-84,67,-6,-9,
            64,-49,126,-81,1,29,-43,-19,14,-96,
            79,-119,-3,60,-46,-7,-97,64,75,-119,
            63,-21,-62,12,-114,-119,11,91,111,-127,
            23,-74,-21,57,31,-99,2,106,-21,-128,
            -106,49,64,98,100,57,-72,-82,65,-90,
            -72,-15,88,-50,113,-101,66,-65,-67,-80,
            2,52,92,45,78,-127,-54,23,-4,43,
            121,-87,-6,115,-112,-71,56,52,22,-38,
            -72,66,71,66,-109,-7,-2,89,-33,44,
            119,-40,35,-64,-40,-105,-87,64,-86,-86,
            26,80,5,88,-20,-122,36,-16,125,63,
            -24,105,-96,9,-9,77,7,-58,-86,109,
            5,118,57,-68,47,48,-53,53,12,24,
            11,-27,-40,-70,-72,-87,64,-73,-1,-61,
            0,-13,-45,-122,29,-128,-92,-117,-126,26,
            -22,-118,4,38,-102,102,54,-73,-37,30,
            -96,-107,115,118,67,-96,-49,97,49,92,
            110,-127,53,71,13,-80,57,98,-99,-128,
            -64,5,108,-25,-43,64,-34,83,-77,-32,
            -45,-64,39,-6,-102,-38,-111,-114,114,-36,
            84,88,1,114,60,-11,82,121,-79,63,
            -97,91,81,52,-83,-117,-44,75,-115,-32,
            -2,70,70,34,-100,127,83,-18,-8,4,
            19,-84,-105,-77,-10,2,125,57,-66,37,
            88,-117,84,13,44,15,-26,76,-1,83,
            58,39,-60,-3,-104,61,53,-3,-7,-2,
            71,-3,121,-102,120,72,36,-57,-46,118,
            2,19,109,61,-3,-79,108,28,-112,-2,
            -93,-25,3,125,110,102,89,-96,-125,126,
            -40,9,116,-108,40,-48,68,-102,0,-10,
            -46,3,92,-82,-29,-62,-72,-32,-28,31,
            44,80,-21,79,-96,-74,-90,-10,121,13,
            104,47,21,-96,109,32,-81,7,-17,-87,
            -26,-24,42,-38,22,-42,56,-37,49,-89,
            -92,111,117,76,-34,46,-47,18,97,118,
            115,-1,-107,53,-5,-89,40,-72,105,31,
            29,1,-53,-68,22,9,116,108,116,57,
            32,-115,-43,-115,-72,-113,97,102,70,-51,
            23,-96,94,-123,-95,113,-32,25,-48,126,
            20,-40,-115,68,26,120,98,-44,33,-114,
            -125,87,-10,27,25,96,23,68,85,-80,
            122,-83,-86,10,52,92,20,-36,101,-46,
            21,2,93,22,-7,8,-20,-66,105,7,
            -28,11,-116,99,46,-18,6,-61,118,106,
            61,10,106,-21,-127,-66,-9,0,125,85,
            28,-49,29,-58,6,-112,1,106,-22,115,
            4,104,122,97,-49,40,-108,17,1,122,
            34,120,115,-19,74,90,115,-40,113,84,
            -101,1,114,7,105,-46,-109,119,-126,25,
            -6,64,11,-16,-10,-108,-127,9,-99,10,
            -17,16,109,11,39,15,96,118,84,37,
            -112,119,44,-35,-93,-125,-65,-89,106,43,
            -73,1,105,101,-28,-25,-116,126,22,-84,
            -65,-118,2,-5,-36,-37,6,-20,62,105,
            37,-80,92,-14,34,-40,-93,-81,-35,-100,
            89,-48,112,-4,-122,-55,63,88,-95,-42,
            75,-96,-74,-82,-19,25,2,68,-12,53,
            -12,-70,-70,30,-30,-87,-7,-45,-114,0,
            125,86,-40,-49,16,-88,57,30,-111,-53,
            -66,-23,-99,-32,-105,59,-66,-92,-107,-122,
            3,-70,106,-62,60,96,-25,-89,-31,-52,
            122,49,3,-78,39,-33,-13,60,54,79,
            127,-19,118,112,-67,-84,26,-45,100,64,
            -88,96,-27,100,-7,121,119,-44,3,115,
            73,-50,28,-96,-127,-21,-106,104,46,71,
            53,89,55,-21,122,-21,34,71,-84,83,
            -96,-74,46,-17,-12,47,-112,-41,67,-84,
            -101,-15,12,-56,0,-35,-101,-18,8,-48,
            -92,91,53,-85,-19,-8,117,-128,-2,-126,
            54,19,-100,103,-120,-25,-115,82,15,-61,
            -112,86,-82,-18,-112,39,110,108,-67,-8,
            70,-16,45,100,-49,-118,-80,-79,-96,-33,
            43,14,100,90,88,61,-120,29,27,82,
            13,-42,12,112,61,14,-14,-64,-83,-19,
            -117,70,2,-83,-95,-122,-55,73,-112,111,
            -66,-6,102,-49,84,28,-28,44,-78,18,
            -44,-42,-73,-37,-68,15,-124,-88,62,40,
            2,-28,126,-34,93,-47,17,-96,-106,69,
            125,95,72,-4,-86,-64,16,15,123,-24,
            52,-8,-27,74,98,-67,116,71,45,-98,
            59,19,-4,122,-11,-83,-82,-91,-80,-11,
            117,125,21,-88,-90,-14,122,-87,-85,64,
            -116,89,125,-128,-54,-89,10,-26,-111,21,
            -112,-17,114,-41,121,91,-31,102,-91,36,
            -50,-53,90,112,50,-11,104,-110,8,100,
            -97,26,102,79,4,-81,-72,113,87,-21,
            -104,-29,68,-2,-90,-40,60,-83,35,-110,
            104,-54,12,-39,4,38,-14,-89,-89,12,
            -93,93,-16,-37,106,-60,25,15,-61,-31,
            100,121,-96,-75,-106,-126,-71,-26,-115,72,
            48,-53,-98,60,14,116,-113,40,-104,-45,
            100,2,-88,94,-60,-70,-31,40,-20,-41,
            -90,63,113,42,-12,-26,-99,-64,-22,-42,
            -34,8,100,-122,106,-54,-49,-62,37,-47,
            71,71,58,-98,-85,-45,99,-117,-37,-83,
            112,-57,-109,-94,46,-57,-17,115,-54,-114,
            55,62,-124,-35,-67,-27,1,-34,-5,33,
            120,83,50,62,3,58,80,13,-61,-122,
            1,51,73,12,-40,-29,-82,-3,-100,89,
            -32,51,7,55,57,-47,-45,16,-72,63,
            116,-7,56,-48,-100,10,-117,-127,68,-86,
            0,61,3,63,85,22,39,-90,-93,5,
            -47,22,22,-73,0,121,-44,-43,-47,122,
            112,-107,-43,51,-63,-34,-11,70,-80,-24,
            46,121,96,13,-29,-64,-69,58,-3,29,
            -96,106,-50,-53,-105,8,44,-101,-74,7,
            53,23,70,-43,108,117,97,77,-70,58,
            3,36,110,24,-54,-11,51,-117,-127,30,
            9,93,0,-42,83,42,64,-27,-32,59,
            -57,92,-100,46,-89,120,-30,-118,105,13,
            50,-44,-96,-62,-43,27,-70,-35,43,15,
            40,-4,44,-104,35,118,123,-127,-58,-85,
            10,-47,25,2,79,-37,56,31,-14,-72,
            -43,-119,-70,33,48,89,122,56,2,-24,
            58,-67,59,-104,-37,-43,100,-23,15,-80,
            -14,54,-57,-19,26,-71,112,91,113,11,
            -48,123,-22,82,-74,49,-24,41,-58,114,
            -40,57,82,30,-48,-76,28,48,87,-53,
            -57,-45,-60,-100,-102,-87,106,74,54,87,
            -28,-119,36,67,32,64,123,54,3,93,
            46,110,55,-52,-41,-98,0,59,64,115,
            97,-31,-37,-114,0,77,-8,-17,79,63,
            21,93,18,-41,63,-82,-117,106,42,34,
            2,-38,-20,-110,-127,105,92,1,-84,-82,
            102,42,-48,-109,-20,53,-114,-79,73,92,
            94,81,-41,-104,-97,5,104,10,4,78,
            -107,31,-1,5,84,-116,-54,-52,109,-2,
            31,-118,-39,1,-102,0,73,-114,27,118,
            70,119,-39,-23,22,-73,-90,-19,-96,-38,
            -48,14,-2,49,79,6,104,-10,11,-32,
            107,46,107,-118,-43,12,10,124,34,-76,
            -24,3,116,18,2,55,60,82,-33,2,
            58,83,92,-105,-43,80,77,-36,18,88,
            -73,-41,49,56,-6,-125,88,-72,-69,-72,
            4,72,76,110,-22,-51,-125,55,111,57,
            -20,-98,9,-116,-3,116,28,-52,12,21,
            -113,-121,-63,110,66,126,-7,105,-100,-94,
            -21,-125,-26,64,-96,6,-99,95,6,116,
            95,-23,-25,-63,-54,86,-125,-124,58,112,
            72,76,69,-71,22,59,-54,-27,-3,-30,
            18,-96,-85,114,33,-90,58,68,-120,89,
            -21,115,123,-93,19,-63,-66,-114,-16,-33,
            -127,-99,-50,6,114,-116,44,4,-106,70,
            -29,-127,93,46,110,57,89,12,44,77,
            60,-73,-57,-52,-87,-64,-86,-41,91,5,
            100,-113,10,80,46,92,23,119,88,107,
            59,30,22,-47,-41,-35,-86,-25,-28,110,
            -44,5,125,34,-105,-86,12,-126,26,98,
            -12,-110,-57,90,78,0,117,94,46,-93,
            -67,65,-35,-80,51,89,113,77,-25,56,
            66,-54,65,-32,-73,19,-99,-34,4,42,
            90,12,18,23,-4,64,99,-41,93,-114,
            -66,-88,-1,-83,122,78,-18,70,-73,-56,
            -43,28,84,-7,-2,-48,48,94,28,64,
            108,-117,-9,-63,122,34,-29,20,-8,14,
            -99,-8,8,-52,-93,-89,18,-64,-36,116,
            106,36,-104,-33,22,-69,124,20,-104,-33,
            30,91,15,-26,-74,-84,-29,96,46,31,
            -103,7,62,49,-43,-29,93,96,12,5,
            59,64,-4,-114,-13,-5,-99,28,1,-102,
            -14,-33,-25,50,111,125,-128,-30,-43,98,
            107,-72,123,-73,-22,68,-49,2,-69,-30,
            -34,4,108,-70,59,9,24,115,47,1,
            59,95,6,-59,48,-25,63,86,-74,-13,
            79,-64,-50,15,2,99,-98,-69,64,-51,
            25,14,-10,-1,-65,6,-71,126,-6,-80,
            -120,-57,24,71,1,-111,-94,110,-30,-26,
            5,-81,-41,106,-27,-49,83,60,-2,51,
            107,91,-76,-1,-39,-18,-108,-65,-118,-19,
            7,68,126,-16,55,-110,-53,-3,61,34,
            114,-15,27,-40,20,113,-67,-77,69,12,
            14,118,-117,-71,-56,-5,126,114,-100,-64,
            -49,106,37,-87,72,-110,-29,-25,40,-102,
            -72,-81,-91,59,127,-29,90,-74,-72,-19,
            -88,-104,36,-43,-69,-118,60,65,-28,29,
            127,103,-71,60,-82,-50,34,127,72,28,
            127,29,103,31,80,76,102,18,84,-110,
            83,28,-14,-95,-119,118,-65,-13,92,30,
            -89,-4,37,94,-11,27,-108,79,73,42,
            73,37,-23,-90,-90,-1,0,-71,73,109,
            13,-32,91,51,-13,0,0,0,90,122,
            84,88,116,67,111,109,109,101,110,116,
            0,0,120,-38,115,73,44,73,-75,82,
            8,79,77,-47,81,48,50,87,112,44,
            77,87,48,50,48,-80,80,48,-76,-76,
            50,-78,-80,50,49,81,8,112,9,-31,
            114,44,45,-55,-56,47,-78,82,112,45,
            -54,76,86,112,73,85,-16,45,-51,75,
            81,-80,73,77,76,113,-56,-84,-56,76,
            -52,-45,75,-50,-49,-75,-29,2,0,-31,
            23,21,24,123,-69,10,-77,0,0,0,
            0,73,69,78,68,-82,66,96,-126
        };
      }
    
    private static byte[] whitePawn() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,10,-116,
            73,68,65,84,120,-38,-19,-37,127,124,
            78,-41,29,7,-16,123,-97,-25,-55,-109,
            52,-117,-60,106,-126,-55,-108,-119,88,-3,
            -120,-51,-40,-106,76,-59,111,-111,-32,-91,
            47,-26,-41,75,-37,-88,6,-11,51,104,
            -104,-95,-93,45,11,70,-21,-57,-90,-90,
            65,45,94,58,26,86,74,117,126,-43,
            -4,8,98,74,13,43,125,105,-108,16,
            69,50,67,36,-49,-3,113,118,63,-9,
            -98,-17,-119,123,73,51,91,-73,61,-72,
            -33,-9,31,-25,-71,-25,-71,-9,-36,115,
            -49,-71,63,-49,125,30,73,114,-61,13,
            55,-36,112,-61,13,55,-36,112,-61,-115,
            -1,85,-56,-114,-23,-22,60,-19,-61,-45,
            -34,60,-115,-86,98,57,55,-66,-90,-16,
            59,-90,-57,88,-119,119,46,79,-121,-15,
            -4,-125,60,-51,-25,-7,99,121,58,-125,
            -25,-89,87,81,-82,27,-9,25,-43,28,
            7,-64,31,121,122,-61,49,95,84,37,
            -53,59,-14,61,30,-66,124,-114,99,-66,
            8,-73,-87,-17,47,-62,-59,-89,35,32,
            -49,13,-21,14,82,30,63,34,90,-123,
            117,-127,62,-91,-66,122,-80,-114,-97,-38,
            14,-1,-106,-89,89,-66,-42,-80,-10,-99,
            -80,-50,-48,59,-127,119,-48,-110,-48,28,
            -112,-82,73,127,6,121,70,-91,-21,117,
            -93,-118,107,-53,0,-112,127,101,-1,-70,
            118,-118,-25,48,-20,-34,36,-115,3,-58,
            -84,124,125,-106,-1,25,96,-41,-4,29,
            -127,125,-55,-13,103,75,83,-128,49,-49,
            105,-40,85,-33,-54,-113,94,35,-118,-20,
            11,-14,100,-9,90,-11,-81,69,-56,-67,
            -37,39,-86,-114,103,53,28,-29,-33,51,
            -106,-104,4,-27,-65,-33,-102,9,-127,-97,
            21,62,9,-22,-14,-62,-57,65,-99,-1,
            65,12,4,6,-4,-28,123,80,-66,-123,
            -106,-13,-68,7,31,55,-80,-90,35,-99,
            71,76,-87,-67,30,110,56,-93,14,63,
            21,49,-54,-16,77,-122,69,35,-88,-127,
            7,-42,-122,50,63,-77,34,-114,-107,-104,
            24,27,107,42,99,67,77,-73,25,69,
            91,83,100,-65,-3,112,-5,28,-107,-29,
            -101,8,11,-93,-19,-85,-9,82,-57,-44,
            118,-69,-30,-34,103,-72,52,113,-33,60,
            6,-94,-33,-78,-90,-118,55,-41,-10,3,
            99,-59,113,-96,-99,-90,-10,87,90,2,
            -5,82,-49,0,118,75,31,13,-20,-90,
            -46,8,-40,101,-102,-17,90,58,104,123,
            107,-11,3,58,53,22,-65,80,125,24,
            68,-73,117,-44,99,-112,-37,23,-9,12,
            -17,-73,-59,-19,91,9,116,-70,74,123,
            124,-38,24,80,84,-34,-34,115,-107,44,
            16,-57,-119,110,28,59,-64,-116,99,-25,
            -10,29,-57,-113,-82,-52,0,49,61,55,
            -83,25,-88,126,42,-73,-38,117,-24,-76,
            -49,81,-113,58,110,95,-36,-69,-125,-42,
            -119,-5,-34,112,-24,31,67,13,57,-75,
            33,40,-75,121,67,-97,82,-93,-63,-8,
            116,-35,116,119,-16,124,49,31,95,110,
            106,83,80,-66,67,-27,70,-104,-47,63,
            -70,-78,122,-72,97,111,-104,86,-12,41,
            50,28,-110,31,-89,-122,28,17,14,-54,
            1,-34,-48,111,43,-7,112,71,-121,-88,
            -90,-118,-108,78,-127,-10,-7,86,-115,-88,
            1,-54,17,42,55,50,2,-110,67,43,
            -85,-121,27,-10,115,-1,98,-6,84,67,
            -127,-70,103,-83,-87,27,105,77,15,1,
            43,-41,15,-102,122,106,37,96,-12,-57,
            39,-64,46,-15,14,-48,-24,-44,70,-7,
            90,49,24,25,-7,-90,-89,-101,-2,5,
            -104,-62,-53,29,-8,45,15,-44,61,-23,
            -88,-57,34,-73,47,-18,29,-79,-4,46,
            -18,93,-79,47,-81,-128,21,-15,-76,-57,
            103,61,9,-127,116,-34,17,-13,-75,-29,
            96,124,58,97,82,40,21,-7,124,-66,
            -84,-106,16,24,73,-27,120,-105,-61,-118,
            88,-57,72,67,-82,-67,30,110,84,50,
            -126,32,55,18,-5,-14,10,-120,-18,41,
            123,-32,88,-102,-100,2,-116,-11,106,11,
            106,-50,-34,68,-48,27,-14,-114,56,79,
            -23,-34,-89,64,111,-40,-85,29,-88,57,
            114,55,96,-116,-105,51,80,94,9,-47,
            -35,28,71,78,35,119,68,-95,-86,-95,
            -99,-23,32,79,-112,-113,64,72,-116,53,
            -108,35,73,-2,91,48,-3,52,29,1,
            -2,66,80,-14,55,78,2,-19,-96,-13,
            30,97,-29,52,-48,14,-8,47,-125,-110,
            47,-106,43,-123,-23,39,-88,92,-7,99,
            8,-119,-95,-11,-70,67,63,-107,12,82,
            -54,-75,-64,51,-119,79,-19,-94,-4,-48,
            77,-16,70,4,53,112,-36,80,80,111,
            -100,-119,0,61,65,-36,79,15,4,-90,
            82,74,-7,103,-86,-125,-98,16,-9,34,
            -88,-1,-96,114,66,-33,-121,55,30,115,
            28,65,-69,120,61,38,62,-86,-125,-87,
            52,-122,83,113,-41,20,6,-34,-47,-46,
            41,-109,36,21,64,68,99,111,34,108,
            92,76,13,-102,-108,12,-54,-96,-85,-5,
            64,95,71,29,80,-34,12,-40,21,45,
            4,88,-128,82,-54,-89,-7,-82,-26,-127,
            -66,54,-87,27,40,3,-59,-75,-56,90,
            -49,2,-66,-34,56,81,15,-86,-105,61,
            66,31,-42,-79,58,-113,99,-102,63,-33,
            -8,-60,-19,-84,-89,5,-44,-33,33,-89,
            -62,-79,-51,-44,-128,19,-50,64,121,14,
            63,78,78,-78,85,38,-58,22,-104,110,
            -78,-86,-126,-26,-93,-27,-84,114,78,80,
            -71,-76,30,-66,-34,77,-68,30,-37,-20,
            -43,21,-11,-116,-87,98,-69,30,-102,-63,
            -48,66,-111,83,3,18,82,60,9,112,
            -87,-120,26,-84,115,36,-108,-74,-53,-97,
            12,74,-105,-19,123,65,45,-38,50,6,
            -40,-86,109,27,-128,-27,-20,-56,0,-74,
            101,71,44,-36,-111,-14,-4,109,-21,-127,
            -27,108,25,11,-20,-19,-19,-5,65,45,
            -28,-27,118,-19,28,1,-91,73,98,48,
            -43,-86,-57,69,94,-81,100,-57,9,-96,
            -16,97,29,76,-19,-63,-45,98,123,118,
            -58,73,-29,100,97,-96,-79,49,118,65,
            -118,2,54,-45,51,7,-116,124,107,-102,
            -66,87,-8,-4,-4,117,2,-29,111,80,
            89,54,79,87,57,-46,108,49,-97,125,
            -71,0,-107,-21,88,-49,76,-2,125,-95,
            -67,94,25,-57,29,-37,83,-20,-40,-82,
            7,62,14,-39,39,35,55,89,-23,-120,
            -18,86,-102,30,34,-7,-31,-71,122,-42,
            -12,-13,35,-91,-114,48,-92,83,-56,77,
            24,-44,34,-4,85,-104,-78,-109,-10,-12,
            -44,54,-96,-41,91,55,14,-76,-60,-20,
            14,-96,125,68,-23,-70,-15,-96,37,-92,
            -2,8,-12,88,90,46,60,11,-90,-52,
            -26,-27,38,-16,-11,116,16,-21,-75,-22,
            -63,-121,-126,-46,-7,-104,-35,-120,84,94,
            -17,-115,95,-67,93,-113,82,52,5,111,
            -110,-24,-46,48,72,25,76,13,-67,-96,
            55,40,-53,-8,69,-91,68,-71,2,44,
            64,41,13,98,47,104,15,-54,123,98,
            104,-89,47,36,-25,59,-42,-13,-56,15,
            -19,56,127,-100,-15,18,31,39,-32,-81,
            -88,-67,-4,69,-103,-73,-128,-89,-4,84,
            20,42,-122,96,124,-25,97,-10,-9,-87,
            -95,55,63,11,-127,118,116,-39,47,-5,
            28,-40,69,74,41,-97,-49,-41,-106,-106,
            -13,-103,102,57,94,47,-124,-98,-79,-81,
            87,-44,-93,-44,94,79,-86,119,-91,-37,
            -11,-56,68,47,-29,-122,-42,32,55,-80,
            103,-17,-67,-18,-53,6,-58,-50,30,5,
            -27,-116,-72,47,-85,15,-20,58,-91,-108,
            127,-74,8,-108,-113,124,-53,-128,-82,41,
            -69,119,-34,-47,-60,6,-103,126,-19,-109,
            -20,-114,25,124,117,-8,-8,-35,-46,86,
            -54,-120,10,-121,-17,-98,-80,-90,-54,-6,
            55,121,25,-116,-106,-33,0,-6,72,113,
            59,61,-34,84,38,82,-118,125,-96,-9,
            111,50,10,-88,-125,74,99,-86,79,-126,
            39,28,23,127,121,-103,-67,30,110,56,
            35,-116,-97,82,114,69,-58,59,48,-70,
            21,-99,-94,70,-83,-124,-64,10,-34,-4,
            59,-43,-63,96,124,-70,97,-46,41,21,
            -7,86,108,27,-11,27,8,108,-91,114,
            -62,84,120,49,-47,-66,122,-17,74,123,
            61,-36,112,60,31,73,61,-28,116,-112,
            84,-15,36,56,20,-14,-68,-44,-80,59,
            -21,-126,114,-120,90,94,-85,15,70,-105,
            56,-34,7,-119,124,-22,-55,120,-88,-72,
            89,-16,100,-64,-98,-65,-118,99,-89,-122,
            -87,43,-97,-20,-24,30,73,-50,33,19,
            -13,-71,-61,115,-76,-30,-78,13,-19,-59,
            107,-122,-26,-105,64,75,-43,22,-127,126,
            68,28,27,-61,77,101,119,-115,32,56,
            -14,-75,-91,-96,31,104,-2,25,104,-81,
            -120,35,-87,49,-76,61,-32,-88,-57,-10,
            74,-21,-7,-120,-114,98,-73,-12,-12,51,
            109,-13,13,1,-29,-112,-22,14,31,-82,
            -95,-122,124,115,55,40,105,-12,-2,84,
            -99,7,-30,-73,8,-6,93,29,-60,-13,
            -59,124,-68,-97,-34,-36,3,74,95,42,
            55,100,0,124,80,-28,-67,8,-58,17,
            -37,-58,-12,41,-81,23,-33,65,-92,111,
            60,106,29,83,-79,-63,-77,33,-92,-82,
            60,22,-116,-101,-86,-50,-16,66,71,106,
            -64,-8,-27,-96,-100,43,63,8,-70,24,
            -4,100,-3,76,-73,-115,39,-98,-128,120,
            -22,-71,51,40,-97,-26,-93,-63,-43,-3,
            -96,95,-114,95,12,-54,53,90,-113,-65,
            47,12,-119,-107,-57,-127,-88,87,-83,74,
            -21,-3,-112,15,-106,-122,73,-45,-64,23,
            -96,6,-15,46,-123,-42,23,60,-87,112,
            107,-72,-72,-26,-52,7,109,-120,56,85,
            13,3,-93,-63,53,83,-43,-63,-25,19,
            -53,-47,53,105,14,104,-49,-118,107,82,
            15,-72,85,-45,122,-125,-37,122,-94,-4,
            18,24,95,-67,12,-66,91,-107,60,-9,
            60,-16,-125,-91,-50,-69,-95,79,-83,55,
            -102,-66,-73,-68,-65,0,-4,-17,0,-102,
            23,-121,-12,-127,-94,103,-88,-63,-78,94,
            3,-75,53,111,-49,11,106,14,-36,-47,
            -20,-12,51,-85,-86,-62,62,-97,-90,-82,
            6,49,125,62,107,26,-88,45,-60,41,
            -81,47,20,37,122,95,-127,-26,-97,120,
            95,6,-29,34,-60,-21,-51,-73,35,-33,
            -79,93,-95,15,90,-57,84,-68,8,-117,
            5,-71,-119,-9,-57,-90,-116,-16,-43,96,
            -20,-118,-49,65,-54,-78,-112,106,80,82,
            -99,26,40,-93,24,-108,76,-34,-128,121,
            90,19,48,62,125,102,10,-80,82,-45,
            -3,7,45,-57,-53,17,-27,90,-79,63,
            -93,8,-108,9,-94,-93,34,-96,-92,-90,
            -65,23,-92,-92,-121,79,2,99,-121,106,
            98,-6,-91,-44,16,-28,-24,74,-73,59,
            -72,31,52,-115,120,10,-28,-56,-16,110,
            32,55,-26,103,-124,25,-66,-95,-16,-6,
            0,79,19,-96,7,71,45,125,-22,-21,
            -96,-48,-43,100,-67,-42,0,-116,79,51,
            77,-91,-50,83,-42,125,-121,115,57,94,
            -82,88,-113,21,-71,83,-25,-128,-62,-121,
            120,-76,81,-98,-106,-64,-104,111,24,-52,
            -93,-65,-81,-116,9,-53,6,121,-72,-44,
            -58,116,-53,-47,14,-34,96,-19,32,58,
            -44,-41,59,-122,-84,-106,122,39,-62,-90,
            30,-76,-121,70,-41,-124,-64,-14,13,63,
            0,117,54,93,-61,-43,-7,96,52,-27,
            96,83,69,-57,40,-90,-1,60,-100,-27,
            -16,-11,-88,-65,6,-15,-86,92,93,95,
            2,-22,-12,-102,-81,65,96,33,-43,-37,
            59,13,54,-44,-28,-5,99,-82,99,-5,
            -105,6,-5,41,47,-59,34,-49,-108,-42,
            -104,36,111,123,88,85,66,27,-40,58,
            4,-54,22,94,-102,8,122,61,-15,-69,
            -61,56,96,-41,-39,116,83,-39,93,119,
            99,95,119,56,-53,-27,-21,85,-97,-128,
            -118,-79,-68,-117,-109,64,15,-5,97,93,
            40,27,47,58,-22,105,-56,30,36,-51,
            49,73,82,2,-56,-12,-6,33,41,88,
            111,-42,-60,-95,-18,-113,-126,78,-30,-33,
            4,-11,-22,64,-96,-32,114,51,-48,-69,
            -118,-37,-34,15,-127,-99,-43,-37,2,-69,
            -94,119,1,-10,119,61,9,88,-119,-34,
            1,-2,-117,41,-83,-121,-42,-53,-21,81,
            -98,11,-20,115,-86,39,-33,-95,26,-59,
            36,-126,-46,79,-36,-98,-101,-1,71,106,
            -41,-36,-47,14,-57,-126,-83,103,-8,57,
            -41,43,-34,-104,-6,-29,97,65,2,109,
            -56,-17,126,14,-127,-47,-30,-120,-23,2,
            -58,41,69,55,5,95,-16,122,-87,-19,
            -95,-30,87,66,75,14,67,-96,-89,-24,
            -96,76,-104,-41,-58,-47,28,106,-80,-35,
            -122,-13,-69,48,111,-100,24,27,57,5,
            107,-105,-120,-9,55,-113,-63,-19,-97,94,
            57,13,-54,-107,-126,-18,-96,108,-3,98,
            4,40,123,-66,-56,-124,32,74,121,-67,
            10,58,-126,-14,-89,-85,18,40,-25,-34,
            -17,9,-73,-59,-1,-107,-28,27,-16,-121,
            108,71,7,-75,-92,-63,-8,96,-23,-96,
            111,-14,-118,-75,-80,103,-25,-74,-108,66,
            -64,56,103,39,-127,-102,-23,59,10,-58,
            93,81,14,24,-23,-69,16,-60,-23,106,
            48,-46,67,96,108,71,127,80,51,-115,
            93,-46,64,119,-95,-71,-99,29,29,20,
            111,111,-105,-1,127,-16,31,-10,121,94,
            -75,103,-49,-54,-76,-46,-29,1,43,-35,
            -57,-97,-60,-13,-8,47,68,-13,78,-15,
            -12,111,65,-98,82,61,-7,-101,-41,125,
            62,-5,118,-51,114,-4,126,78,-76,-125,
            -5,-17,113,55,-2,-35,7,85,43,10,
            -8,21,-23,121,-98,-14,87,-54,114,-113,
            7,60,77,-74,111,23,109,103,-91,-19,
            16,-124,-17,119,-20,-61,-11,-12,47,-126,
            78,60,-19,-4,-128,-89,-99,28,-37,21,
            -17,-66,63,114,-61,-115,96,-116,127,2,
            -25,-118,-91,-9,-87,-49,75,-83,0,0,
            0,90,122,84,88,116,67,111,109,109,
            101,110,116,0,0,120,-38,115,73,44,
            73,-75,82,8,79,77,-47,81,48,50,
            87,112,44,77,87,48,50,48,-80,80,
            48,-76,-76,50,-78,-80,50,49,81,8,
            112,9,-31,114,44,45,-55,-56,47,-78,
            82,112,45,-54,76,86,112,73,85,-16,
            45,-51,75,81,-80,73,77,76,113,-56,
            -84,-56,76,-52,-45,75,-50,-49,-75,-29,
            2,0,-31,23,21,24,123,-69,10,-77,
            0,0,0,0,73,69,78,68,-82,66,
            96,-126
        };
      }
    
    private static byte[] whiteQueen() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,17,-56,
            73,68,65,84,120,-38,-19,-36,119,120,
            21,-59,-34,7,-16,-35,115,66,26,9,
            -122,14,42,-99,32,17,-60,87,-118,40,
            8,-120,72,19,105,-105,34,93,12,69,
            90,2,-126,16,9,87,-70,-128,47,-59,
            11,-14,74,9,120,13,82,-92,42,29,
            68,-102,116,-92,73,-109,114,-31,74,104,
            -71,16,-70,-112,100,-9,-20,-52,-35,-17,
            -98,-7,77,56,-61,61,111,-48,-57,-121,
            -101,-16,48,-97,63,-10,-100,101,-49,-108,
            -33,-74,-103,-39,13,-102,-10,36,61,73,
            79,-46,-109,-12,36,61,73,79,18,-110,
            46,-106,21,-60,-78,-88,-97,127,-1,-77,
            -54,-47,-108,114,42,-4,-55,-27,100,-37,
            20,-84,124,95,-24,93,-72,15,122,-105,
            -82,-100,98,-103,36,-42,-65,38,-74,11,
            84,126,-25,126,-56,-14,-44,-19,2,125,
            -13,-91,114,-88,92,-9,126,-79,-35,-36,
            76,-22,-3,-40,-91,0,-27,-5,112,113,
            -64,46,84,-42,-25,84,-66,23,18,-37,
            -3,42,-66,7,61,-28,-103,-91,-82,15,
            82,-14,41,-12,-1,-105,-85,47,18,31,
            -30,50,105,-57,99,-109,2,-75,14,-114,
            -95,-54,-22,-1,115,127,8,113,-47,-34,
            -17,-37,6,-22,46,88,115,-48,125,11,
            -102,108,-120,40,2,-10,33,92,0,-12,
            -9,-60,15,-61,-59,50,-121,-97,-14,104,
            125,56,-3,46,-94,24,-40,-25,-120,55,
            -33,77,84,-114,40,119,-104,59,14,-30,
            -60,-114,13,-84,45,115,122,-41,17,-17,
            -25,76,126,108,82,75,113,100,-42,-66,
            -17,-104,-75,-51,11,-13,126,-31,60,95,
            119,-80,66,-62,54,2,-85,69,-21,-75,
            61,16,-77,-57,55,59,-41,63,-3,-100,
            113,-54,25,-31,58,43,-41,120,-13,-7,
            -103,-14,21,-27,-44,-12,-106,-21,-31,-78,
            -68,48,-104,51,83,57,-93,-6,-119,15,
            -115,31,-45,-3,-29,74,-107,-111,-21,14,
            111,-66,66,1,-23,81,9,-46,-21,115,
            39,-103,-85,46,15,-121,-12,79,-98,-85,
            11,92,4,-18,-14,-14,-110,113,-112,107,
            -99,18,-72,124,126,-82,112,114,125,-55,
            -123,-112,-21,107,-111,-49,-78,-25,26,0,
            -25,-105,71,66,-6,40,81,-18,-30,30,
            17,-112,62,89,-18,-64,16,120,61,81,
            105,-57,-103,-57,-11,22,84,78,94,123,
            -50,66,-4,53,10,-60,-38,3,96,-108,
            -26,-66,-87,111,-41,123,96,-83,-10,110,
            103,45,43,112,6,-54,-69,-108,-128,85,
            -11,115,64,-68,78,-97,10,-12,-124,-14,
            -55,-34,111,-20,-39,110,33,96,125,41,
            -54,25,64,5,-82,-3,9,-116,40,-86,
            87,-114,123,48,56,-105,-46,-114,-88,-57,
            116,7,-71,-65,-110,-35,-95,4,104,-1,
            43,5,-94,119,34,-92,-33,20,113,42,
            -111,-4,119,-80,-102,69,53,5,58,-125,
            110,-76,40,-56,-96,-128,-46,43,-45,59,
            -8,57,-125,-6,-54,29,-28,-36,-124,10,
            -102,-34,111,-73,34,-93,-70,2,-25,-55,
            83,-64,-118,22,-27,22,-19,61,5,-116,
            80,-86,87,-16,6,104,55,70,105,-57,
            23,-113,-21,61,-24,-86,12,-35,113,8,
            -87,-17,-3,118,-96,58,5,-92,112,67,
            -16,124,-18,92,105,122,88,37,-27,61,
            97,57,12,85,-70,-67,-82,73,-30,67,
            65,63,-27,61,45,-74,27,44,-41,44,
            -125,-31,11,40,-33,-120,-98,96,-107,41,
            -4,22,120,-58,-54,-14,-100,-76,-81,-112,
            -85,4,4,43,103,-84,118,-10,113,-35,
            65,110,123,40,104,-45,51,46,25,-59,
            33,119,57,-3,99,56,82,85,-49,5,
            -10,-91,-91,35,120,54,-22,-3,-128,2,
            86,-91,-106,-78,-125,94,81,122,115,106,
            18,-21,93,-43,124,87,87,-119,-42,63,
            0,-69,-100,118,-32,89,77,-27,-22,67,
            -31,-120,-90,-107,-128,-120,81,-14,39,47,
            -126,30,-14,59,-57,95,-1,-75,84,69,
            44,95,-3,-125,3,-44,-41,93,-111,-96,
            -91,-5,-2,-13,-95,-99,-123,42,3,-25,
            -125,18,33,53,-104,-114,104,-41,-73,48,
            119,-106,-36,-76,38,-48,0,83,-114,103,
            116,101,89,72,108,119,78,-18,82,111,
            62,-13,40,-33,65,-77,33,53,-72,-48,
            43,64,7,-62,-95,-61,25,-121,0,104,
            -73,-60,-41,-38,127,112,-64,-6,-86,18,
            -73,63,97,-68,-30,-101,-70,-117,3,103,
            -123,104,-65,-72,-71,-21,-21,-59,122,-22,
            -2,78,-3,-49,3,-61,7,-18,13,75,
            -24,83,-18,14,80,116,-97,-9,91,90,
            68,-7,-61,-64,-7,-99,-6,-32,-103,86,
            44,12,-72,40,-49,24,24,60,23,-54,
            -19,83,-14,-69,43,62,-124,-8,46,-27,
            122,45,120,14,-108,59,32,-14,-119,43,
            22,8,-4,-6,-99,-70,-32,-103,86,-2,
            52,-48,14,74,-53,47,-22,117,80,41,
            103,-87,-97,120,-87,-19,-100,-22,27,23,
            -118,19,-59,-115,-30,72,113,-11,27,-9,
            7,-69,61,-54,-9,-90,34,-29,-19,126,
            42,20,-86,52,96,-107,-8,48,-64,-49,
            -56,91,28,-39,-18,-78,-12,15,97,-15,
            -48,-96,7,29,-47,45,-97,6,-13,-84,
            -72,105,-9,26,-5,18,24,-77,101,-81,
            -22,54,-52,-104,45,115,-114,4,55,93,
            -62,34,-28,-46,119,-67,38,126,-105,72,
            -7,-116,45,11,70,-126,40,-89,119,-53,
            82,96,-2,67,118,-81,7,67,3,-91,
            29,-78,-34,-70,-97,25,-123,1,74,28,
            52,37,78,-54,-114,-108,113,109,-102,-55,
            126,-72,111,-28,-83,59,26,-105,-38,11,
            90,67,-111,-47,90,-9,48,-24,93,68,
            27,8,-37,-9,107,109,97,-25,37,119,
            103,-8,48,-81,-88,-17,-7,-5,114,-126,
            126,-54,-120,94,92,-69,3,-28,37,34,
            -72,33,-60,39,83,96,-58,47,4,35,
            47,117,123,-109,-81,-128,-59,10,69,3,
            29,-31,119,103,6,111,-127,-46,63,42,
            13,-34,-24,-69,-76,-13,-33,4,-91,119,
            -120,-33,-51,42,-44,17,-20,124,47,-126,
            101,81,57,-29,-105,-125,-111,87,-10,-34,
            -68,-11,-70,-83,28,103,-63,-54,61,40,
            -121,-46,78,-38,-18,-126,-116,-117,-120,-109,
            -120,-37,1,17,-57,103,69,61,-41,-108,
            90,10,118,-100,69,-36,51,-103,25,-95,
            -39,92,87,113,-71,102,8,76,-103,37,
            123,61,53,-64,106,19,-111,12,86,61,
            -39,-21,-23,2,-119,-54,-111,-96,-9,22,
            31,-34,22,-53,103,68,-2,23,-28,49,
            -87,-61,-46,52,-54,103,115,8,24,-1,
            80,-58,67,7,70,-28,1,35,-122,-74,
            11,-116,-123,-65,-55,113,-114,86,0,-36,
            127,-107,75,-70,102,120,-73,-85,67,-65,
            27,17,6,70,44,-27,75,5,108,126,
            10,12,121,6,-71,53,88,-90,-114,-73,
            46,-8,-74,-125,-38,-91,-9,-110,-101,120,
            -29,-48,90,-58,75,-60,41,-30,53,-80,
            -38,-56,120,121,-29,-86,-36,18,92,37,
            -108,89,-11,7,-82,112,-14,26,-104,-5,
            37,40,-37,-60,-5,-51,-77,-92,-50,8,
            -16,116,-66,59,16,-116,123,-87,-33,-126,
            25,-37,-88,51,24,-79,84,112,-8,16,
            -88,-78,69,-55,119,-68,-8,48,-97,-42,
            84,-81,0,110,113,-13,61,117,60,116,
            53,-40,35,-5,102,-32,-39,-91,-20,32,
            126,-71,35,88,-21,-13,44,4,58,-109,
            110,-114,9,-66,6,69,75,60,-48,27,
            -71,10,69,75,-117,-19,-58,-27,-103,15,
            118,62,109,-63,90,-1,64,-2,45,-64,
            -77,35,116,13,80,-2,-89,-50,86,59,
            15,-18,123,74,-10,-13,-107,118,105,-31,
            113,80,-27,32,-59,-95,81,59,48,122,
            -90,46,3,51,-10,-18,0,48,82,-21,
            -60,-125,-25,93,17,-41,5,-71,43,67,
            -39,-6,-2,-10,-125,-38,11,-106,35,-14,
            92,-67,-96,-31,92,42,112,-62,81,48,
            46,-120,-10,-92,82,-61,-90,45,7,-29,
            117,-38,46,103,91,104,87,87,-55,87,
            -52,86,-21,114,-118,36,-94,23,-108,18,
            -37,25,87,42,4,2,-27,-54,-58,-14,
            43,14,-18,-47,65,-58,-15,-89,-72,111,
            -64,120,95,-98,73,-89,96,-52,-27,7,
            122,57,-34,-11,87,105,-69,-72,-81,-63,
            -24,78,-7,-56,124,69,57,84,110,-123,
            -100,64,59,-56,-72,-31,29,-113,-107,82,
            -26,-34,-88,29,-18,69,114,-22,-54,-37,
            -18,-73,-88,-68,105,-117,-64,-88,-83,28,
            7,-87,19,-10,-125,113,-119,-74,-53,-43,
            7,26,-50,82,-30,-11,-78,-65,51,-88,
            -80,60,-125,-14,-64,-13,-30,18,102,93,
            -81,-65,26,-52,-85,-58,0,48,35,-51,
            -74,-32,73,110,50,19,120,41,81,-32,
            -45,17,-111,80,-31,27,-91,64,113,51,
            117,-53,110,102,-50,-77,-48,68,-34,123,
            58,-18,0,-109,46,61,95,120,62,3,
            59,100,125,-128,-33,-90,22,-98,107,9,
            86,108,-72,-125,2,-103,-46,57,-84,14,
            -28,-103,18,86,27,-14,-120,75,71,74,
            116,120,31,-80,127,-41,28,-84,88,-71,
            43,68,-66,84,14,-107,-37,113,23,-104,
            125,-28,1,119,6,-102,-36,82,-38,35,
            -38,-31,-106,-29,-81,-89,126,-127,23,39,
            -118,-33,109,106,-14,119,96,-23,102,75,
            -16,-92,24,31,-128,25,89,127,5,-104,
            55,69,92,-1,-107,59,63,60,95,83,
            -39,15,-123,50,25,-33,-24,-23,-9,-11,
            75,108,-45,-33,-112,-77,-51,75,-64,-6,
            52,-1,12,-80,46,-53,107,106,85,72,
            -51,-87,-3,5,106,126,-94,52,104,-107,
            115,73,119,7,-54,113,83,-48,55,48,
            -78,57,-3,126,-54,10,48,10,82,-92,
            60,-95,-64,77,126,-57,97,121,18,64,
            6,114,109,108,3,48,6,-54,-101,-6,
            59,16,83,87,44,27,-48,-6,-40,122,
            96,12,-92,-33,-55,124,40,95,42,71,
            -92,41,-85,-64,-56,71,-65,15,90,0,
            35,59,-36,55,-44,-66,-81,29,-18,-11,
            114,125,51,-88,105,-119,56,-116,-94,-33,
            -25,-97,14,86,74,-66,-59,96,125,42,
            -29,-27,-115,107,117,-27,-52,76,-49,108,
            -68,68,-67,-72,104,37,-64,111,-71,35,
            -32,-61,80,-17,115,-111,-35,-75,-76,-22,
            112,-18,13,-67,63,-48,-111,-52,123,107,
            -109,-63,58,22,-16,21,-48,-44,-116,-98,
            -96,21,3,-41,37,121,-116,52,-121,85,
            -53,-87,-62,-37,27,-128,89,-104,2,101,
            -67,0,-4,22,-65,-25,-32,-84,22,-40,
            33,21,-23,84,123,-80,-66,11,94,11,
            84,-2,37,-15,56,-29,82,-85,-32,-43,
            96,111,-41,22,-84,101,-14,-52,-95,124,
            68,-66,-78,28,-111,-74,55,4,-77,-80,
            28,32,123,-21,-71,65,-122,34,23,80,
            59,-12,25,1,95,-64,-48,4,109,2,
            88,71,-59,-17,122,-46,76,-123,-120,83,
            117,17,-73,-38,50,-114,-34,-72,54,-72,
            -17,-47,-30,-3,113,-49,-31,127,42,-58,
            55,-75,-91,110,-74,-13,92,75,-41,-121,
            -72,-86,-127,-67,-31,-57,80,100,-91,-10,
            50,-92,-44,-89,6,-23,123,-64,-4,-36,
            30,-126,-39,-20,-34,-48,126,-40,62,55,
            -80,53,20,47,-43,-59,-31,18,-41,-12,
            115,-83,-61,-45,-127,-13,-85,-59,-63,-54,
            33,3,57,15,-20,64,50,7,-25,55,
            28,-106,-25,56,-56,51,105,89,-105,-107,
            96,-119,73,80,118,-111,-106,-76,94,108,
            -73,84,-2,78,-28,67,-7,-54,114,68,
            18,-11,8,-92,122,-119,122,-66,27,29,
            6,-82,-106,-127,109,-96,-8,-77,-94,93,
            95,106,-35,-64,110,-9,86,48,-89,-54,
            51,-60,-119,-53,-43,-19,-18,-15,80,-92,
            -82,-21,13,-80,15,-52,32,-48,-29,40,
            -82,-66,113,-10,-69,31,50,29,25,-117,
            71,-45,110,-15,28,38,-32,-124,-52,-87,
            24,-60,92,-89,-118,-107,73,-125,-12,-103,
            -11,-54,65,-6,97,-39,109,-99,0,87,
            46,-121,116,-121,-31,-30,90,125,-9,120,
            -59,46,32,119,-53,14,30,-29,-32,-4,
            75,71,-70,-36,21,116,38,105,-112,17,
            -48,35,-71,-127,77,115,-1,0,-84,-90,
            123,35,-80,90,71,-14,0,-101,38,115,
            -90,-33,-119,124,100,-94,114,-88,92,81,
            15,-86,-105,-88,-25,41,81,-17,73,-18,
            -47,112,37,-119,-38,85,119,23,-92,-17,
            -116,76,-127,-12,4,-39,-34,-4,-48,-89,
            -109,50,62,18,103,-104,123,-83,111,92,
            51,-101,121,-7,-3,115,105,-69,-59,-78,
            70,-2,15,32,-32,83,-19,36,28,-40,
            68,21,-36,48,14,-52,97,-109,127,5,
            83,-116,95,88,57,119,18,-56,75,-30,
            -73,-47,-85,-127,85,18,-111,-39,-30,-23,
            9,-10,-89,-33,28,76,6,-110,-50,-92,
            75,14,-113,85,6,-28,-65,78,108,20,
            11,-84,82,-93,126,-64,42,-46,122,43,
            18,50,126,39,-13,-95,36,-54,-111,-27,
            -118,122,68,-81,4,86,-119,-22,-23,-66,
            8,84,111,86,110,74,43,48,-5,-83,
            -5,9,-52,-65,-54,43,72,110,-40,-9,
            -51,-45,5,-63,61,82,-60,-23,35,-79,
            -36,-4,7,-25,-14,-2,-16,-28,-76,-20,
            -106,7,-83,-127,22,83,-88,-94,85,-5,
            -126,-25,25,-47,-32,-54,123,106,-127,85,
            60,106,18,-104,98,74,-124,111,-101,-10,
            17,-80,40,-39,57,120,6,-20,35,-38,
            114,60,-104,-24,76,122,9,50,-18,29,
            -69,-1,23,88,-98,-35,19,-128,-55,-103,
            8,-79,-35,-19,7,-50,28,121,-77,-13,
            -106,35,-53,-91,-31,-61,-121,-64,-54,80,
            61,-93,70,-128,-39,127,-49,60,-80,-98,
            19,-101,-123,87,58,13,-98,87,101,-89,
            98,54,52,83,-98,-8,-70,-53,60,-22,
            -39,108,58,21,-37,6,-108,4,77,14,
            76,-35,-35,96,87,83,-86,-16,-78,95,
            -64,92,67,13,-65,-77,12,120,96,-89,
            -35,96,125,-66,-83,18,-80,110,50,94,
            51,-128,-33,-28,-2,-110,-17,-114,99,124,
            -123,-61,30,16,57,-114,-54,37,-83,-89,
            -19,-4,-19,112,63,-27,110,-5,31,96,
            -35,58,-19,1,-21,-13,59,83,-127,7,
            -47,-65,47,-116,5,51,-29,-110,-10,35,
            108,-17,42,59,65,39,65,15,86,-26,
            -38,30,-7,-53,38,-12,126,89,115,-71,
            -25,-10,66,-37,-83,84,-15,55,-49,-128,
            57,76,-76,-85,-94,-11,14,-40,17,91,
            5,124,2,11,7,-2,-91,-116,-108,-9,
            38,-18,-31,-103,37,111,55,57,99,15,
            -116,3,-98,74,75,127,-37,-7,77,74,
            -71,-94,94,-77,69,39,98,-126,-75,27,
            -28,-42,-91,107,53,5,83,-74,51,104,
            22,-76,86,-58,51,126,-33,-25,123,100,
            -119,-70,-125,107,-27,0,116,0,-124,-108,
            -11,-10,-30,78,127,70,13,-40,-65,10,
            60,19,-88,-123,-58,22,-32,41,-4,123,
            -121,33,-101,110,58,126,127,74,119,48,
            -71,-4,-67,73,45,87,-44,-53,-104,12,
            -4,-102,-68,-108,-66,7,-98,-98,-14,-98,
            -29,60,-24,59,49,50,-25,49,8,-86,
            -92,-60,103,94,102,-35,-26,71,-107,66,
            -44,-127,91,80,77,24,-27,-94,-122,12,
            88,4,-58,59,-94,-99,-121,61,47,1,
            90,-20,48,31,-72,-124,-91,57,24,31,
            -28,72,99,53,-127,-33,-15,-36,2,123,
            89,20,120,-86,121,4,-72,-57,-4,9,
            -20,56,-45,82,-84,-89,-19,-24,119,-108,
            15,-27,43,-53,81,47,-127,59,28,-122,
            -89,49,-56,-75,91,99,-90,-128,-47,76,
            -98,57,109,97,88,-120,114,-26,-4,-16,
            -120,59,3,15,123,-87,115,69,-54,61,
            118,18,94,20,-109,-104,-4,-7,-56,4,
            -32,57,-51,-78,-64,38,-54,38,-105,116,
            -36,97,-19,-128,-33,-10,-108,0,59,-96,
            -125,-64,-2,-9,-81,28,-22,-103,-30,-1,
            60,-16,93,-6,-1,-99,-56,-105,-54,-95,
            114,-87,30,60,-40,33,-89,-104,-46,98,
            -128,69,-105,104,1,-68,-117,-104,-86,25,
            26,-78,4,-54,41,-81,97,-71,-126,-78,
            -54,-103,-93,-90,-34,90,125,-121,28,120,
            -23,78,-38,39,31,93,-17,-99,0,-26,
            111,50,-94,125,65,6,-126,-119,35,-105,
            -46,78,113,-49,-86,-106,-76,0,-40,-114,
            45,125,-127,125,50,103,21,-80,-104,-119,
            -63,-64,-30,71,28,6,86,-122,-106,19,
            67,-128,-59,-49,89,9,44,102,75,47,
            96,-93,-109,102,3,-37,105,-3,5,120,
            53,42,71,41,-105,-103,77,32,99,7,
            -19,-68,1,-26,1,121,105,27,8,-69,
            51,-34,-105,43,-24,-96,7,117,-99,-78,
            -22,-69,10,52,-123,113,84,94,-22,26,
            -64,-40,45,-44,-80,-49,110,-126,81,82,
            118,-85,45,-32,-125,47,22,3,-74,101,
            -34,71,-64,-38,117,-98,9,-84,95,-55,
            -99,-64,110,107,91,-127,-9,-41,92,-64,
            -65,-41,74,1,-89,-87,21,122,83,-12,
            23,101,-7,-77,86,26,-28,118,27,-75,
            -93,-64,-5,-106,-76,-128,-3,-42,121,57,
            -80,-66,-13,6,0,107,119,49,63,-80,
            -51,-98,-69,-64,7,-53,-127,86,21,48,
            -98,-109,-105,-74,-49,96,-76,114,-49,113,
            -97,86,30,-67,103,-63,-73,118,-68,11,
            -7,-64,47,-28,28,52,92,76,13,107,
            124,11,-52,77,-117,14,-125,117,-90,-58,
            33,96,99,3,62,5,-34,89,108,71,
            -45,-15,71,2,-122,3,111,-5,-62,112,
            96,-69,-37,-33,4,54,110,-40,58,96,
            125,-90,29,0,-42,60,-79,9,-80,-124,
            -60,22,-64,-90,79,75,0,-42,116,88,
            101,96,-79,-19,-61,-128,-59,-107,31,12,
            -20,76,64,16,-16,-26,-54,14,77,14,
            -104,15,-4,-67,26,-73,-127,-115,93,-76,
            30,-84,-61,-115,3,-63,60,73,-19,8,
            -83,14,-11,118,42,-51,15,-51,38,111,
            -3,-24,107,124,-65,23,-49,-31,125,-41,
            57,61,74,-101,4,-10,-120,124,54,112,
            49,-83,-50,-9,22,73,3,-106,-38,107,
            15,-80,-24,21,-51,-128,117,73,-82,12,
            -20,-96,24,104,54,-25,39,28,27,-7,
            85,-57,-19,76,123,103,-1,114,-36,-26,
            73,-114,-11,124,-101,-93,-11,-27,-68,-64,
            54,127,-73,24,88,-25,-98,121,-127,117,
            121,-10,2,-80,59,-94,94,-5,-76,-81,
            -128,-121,107,127,3,-69,-34,-95,-112,38,
            30,-115,23,-3,64,105,-9,-4,-20,-14,
            -2,91,-110,-20,118,79,-121,-48,119,-68,
            -33,46,-114,11,72,2,-50,99,-118,-125,
            -25,-6,-42,-17,-127,-115,76,-17,10,-68,
            -72,8,-21,113,62,-43,-111,-47,-73,27,
            -19,-32,-58,25,-32,-23,102,110,-112,-1,
            -98,114,-30,26,-80,-123,-89,66,-127,45,
            -94,-119,28,-93,33,-40,-65,59,4,118,
            55,97,-122,35,35,-33,-55,14,74,71,
            -45,-50,3,47,-80,101,41,-80,-95,125,
            -26,-128,-25,76,-64,1,-96,-87,-98,11,
            21,-62,54,64,72,43,-91,-35,39,-78,
            -53,14,-102,72,31,-106,-106,1,93,92,
            74,78,-122,5,-33,2,-50,79,88,96,
            126,43,35,-27,36,-69,91,-36,14,-72,
            -27,-23,14,-4,46,27,10,118,-73,56,
            -46,113,87,-34,-69,118,2,-81,57,105,
            63,88,44,103,36,-16,28,-31,58,48,
            -13,-117,36,96,58,95,-29,104,-59,79,
            59,56,-81,-27,-72,-53,6,-128,-35,107,
            -12,-106,119,87,-108,103,49,55,100,-12,
            -9,78,36,-125,-7,-98,-9,-75,45,-38,
            65,-65,-120,113,-106,-10,-99,-46,-18,-79,
            -39,101,7,77,-16,125,-50,-95,-105,-43,
            -42,-61,-55,23,-28,-93,-22,49,96,124,
            29,-1,54,-80,-60,107,45,65,-98,65,
            -100,-73,113,48,-10,50,100,-52,-71,-99,
            -66,3,-84,98,-119,-73,-127,5,-54,-87,
            -106,49,-64,45,-9,40,-112,-109,-79,60,
            106,48,-80,-16,127,14,1,-69,3,77,
            -35,-76,86,96,-17,120,81,14,-83,-65,
            -42,20,120,-47,-8,-22,-64,22,5,126,
            2,70,119,-7,24,-63,105,-57,-119,14,
            -34,-50,-118,-74,76,105,-9,-72,-20,-78,
            -125,-28,-69,-53,-6,121,8,17,47,83,
            92,-100,-21,26,4,-100,7,68,-127,39,
            -100,26,-34,63,17,44,-70,104,-51,97,
            -63,-128,73,21,-121,60,115,-18,45,0,
            -34,126,-30,83,-64,26,-27,-31,-64,-60,
            11,-118,-36,-112,-49,-6,-5,2,-49,-13,
            -15,58,96,85,110,-113,2,94,73,-18,
            -96,52,-32,-65,-55,114,68,-71,-3,29,
            -106,124,121,62,-96,44,-104,-57,92,61,
            -127,118,-4,-59,48,61,21,66,-44,-25,
            56,39,-77,-55,-2,-111,127,58,40,82,
            -47,-29,122,29,72,-93,-34,82,-115,124,
            -57,-63,115,124,126,53,96,61,68,-128,
            -26,-14,-50,14,-50,55,56,76,-27,113,
            3,99,-89,32,-29,37,-116,-28,124,-64,
            22,55,-17,14,-84,74,-109,-110,-64,-86,
            -97,-97,11,44,-111,94,-81,99,63,-125,
            -100,-87,-56,-104,51,-8,-63,97,-54,114,
            69,61,-26,-41,6,-42,46,-33,97,-16,
            -48,107,97,39,-11,55,32,-19,99,-47,
            -82,-63,-39,-83,-109,64,47,-121,-56,17,
            116,-16,34,-88,-73,-114,-114,-56,-41,76,
            48,38,-89,-12,5,-10,-94,-68,7,-43,
            0,-5,-34,-112,11,-8,85,-42,4,-8,
            77,86,29,-8,13,49,-62,-65,105,29,
            2,126,-39,-40,11,60,-103,119,114,112,
            54,30,-8,8,22,7,60,-98,47,116,
            112,99,7,-16,43,-42,100,-32,-105,88,
            55,-80,-13,-83,6,118,-66,84,-114,40,
            87,-44,67,62,-104,72,105,13,-20,-75,
            -22,63,-126,41,-1,18,48,120,9,-44,
            83,58,9,110,-105,111,28,-78,94,-94,
            57,-71,99,114,54,-11,6,-116,-106,111,
            -17,44,110,15,70,127,121,-77,-113,-125,
            -5,102,-97,-77,72,-14,-60,64,70,-67,
            22,-74,1,-93,-85,-68,-121,22,-126,-47,
            -54,31,63,-53,118,103,-39,-127,106,-4,
            127,94,-67,-17,70,-24,116,-32,124,-1,
            58,72,-117,-71,-104,14,102,-54,-71,-82,
            96,126,-97,-44,27,-52,109,73,3,-31,
            33,-106,3,-64,94,-114,0,115,123,82,
            85,48,55,39,-67,12,-10,-110,-42,-45,
            118,15,-101,-81,-88,-57,-71,-114,96,-82,
            -65,84,24,-52,-21,-5,107,64,-38,-5,
            -95,85,64,-2,-3,-48,49,63,113,24,
            -100,69,-9,79,-58,91,59,90,127,-88,
            -45,81,-66,29,-77,7,88,66,-32,108,
            -32,43,3,42,-126,125,19,-34,10,-10,
            114,9,100,-95,-27,-113,96,47,43,-128,
            125,-58,44,6,-66,-46,117,8,88,-126,
            -17,-21,83,117,-38,-8,-115,67,22,73,
            -12,-41,-44,-5,125,87,15,17,-77,-66,
            -57,-60,20,-50,118,113,-122,-19,18,-67,
            -68,93,39,-78,-39,82,-44,123,-5,96,
            -33,118,13,121,95,-39,65,-5,125,-29,
            -110,5,78,29,-27,-69,-8,127,12,92,
            -22,-33,-65,-124,107,-113,71,82,-38,33,
            -37,25,-99,73,92,-2,-21,73,125,-2,
            49,68,116,63,69,-59,117,-15,-94,-98,
            -34,56,-101,47,27,-8,-74,-117,-38,-23,
            55,14,89,54,85,22,75,-15,119,70,
            -38,-101,98,89,55,-101,47,-33,84,-38,
            85,89,123,-110,-98,-92,39,41,11,-90,
            127,3,86,-86,-122,13,-121,19,41,0,
            0,0,0,90,122,84,88,116,67,111,
            109,109,101,110,116,0,0,120,-38,115,
            73,44,73,-75,82,8,79,77,-47,81,
            48,50,87,112,44,77,87,48,50,48,
            -80,80,48,-76,-76,50,-78,-80,50,49,
            81,8,112,9,-31,114,44,45,-55,-56,
            47,-78,82,112,45,-54,76,86,112,73,
            85,-16,45,-51,75,81,-80,73,77,76,
            113,-56,-84,-56,76,-52,-45,75,-50,-49,
            -75,-29,2,0,-31,23,21,24,123,-69,
            10,-77,0,0,0,0,73,69,78,68,
            -82,66,96,-126
        };
      }
    
    private static byte[] whiteRook() {
        return new byte[] {
            -119,80,78,71,13,10,26,10,0,0,
            0,13,73,72,68,82,0,0,0,52,
            0,0,0,52,16,6,0,0,0,-107,
            -24,-57,-88,0,0,0,4,103,65,77,
            65,0,0,-79,-113,11,-4,97,5,0,
            0,0,1,115,82,71,66,0,-82,-50,
            28,-23,0,0,0,32,99,72,82,77,
            0,0,122,38,0,0,-128,-124,0,0,
            -6,0,0,0,-128,-24,0,0,117,48,
            0,0,-22,96,0,0,58,-104,0,0,
            23,112,-100,-70,81,60,0,0,0,9,
            118,112,65,103,0,0,0,52,0,0,
            0,52,0,49,19,11,69,0,0,0,
            6,98,75,71,68,-1,-1,-1,-1,-1,
            -1,9,88,-9,-36,0,0,0,9,112,
            72,89,115,0,0,0,72,0,0,0,
            72,0,70,-55,107,62,0,0,11,10,
            73,68,65,84,120,-38,-19,-100,123,116,
            77,87,30,-57,-49,-67,55,47,68,-72,
            121,80,-11,-118,105,-56,-112,4,-115,71,
            31,65,53,82,-17,71,-122,-110,-102,-78,
            -44,51,-86,-34,-102,-116,118,49,-45,122,
            76,-123,89,-86,107,-86,-76,53,12,-42,
            -116,25,42,109,-46,34,69,106,52,-104,
            90,30,97,-116,89,-88,68,75,89,65,
            68,24,34,57,-81,61,-9,123,-49,-2,
            109,-18,-31,50,-77,-106,-101,-36,-101,-71,
            -65,-49,31,-65,125,-50,58,103,-1,126,
            -33,-33,-66,103,-33,115,-50,-66,-119,36,
            -7,-51,111,126,-13,-101,-33,-4,-26,55,
            -65,-7,-51,111,-127,-90,-19,-27,-122,-77,
            -116,-28,-66,59,-9,-55,38,-33,-109,-5,
            65,-36,71,-14,-13,109,38,79,118,-120,
            31,-105,-10,-120,126,-69,-69,30,71,-25,
            9,51,-11,79,113,69,30,61,31,-47,
            -17,72,87,-99,110,-21,-32,55,-65,-3,
            119,54,-116,26,-106,58,-96,78,-100,-79,
            21,-43,62,-84,3,-120,24,-41,50,29,
            68,-28,52,-97,6,34,-66,108,-70,17,
            68,100,89,24,-120,28,107,28,-33,-80,
            44,116,7,-112,-70,-14,-18,58,-117,8,
            19,65,-24,76,-34,111,-125,70,-111,32,
            98,114,-117,117,32,34,-105,60,-33,-97,
            78,-57,-15,-13,102,-103,-14,-19,28,-70,
            29,80,-100,-122,101,-108,7,-27,69,121,
            82,-34,-92,-125,116,-71,-22,-68,-65,14,
            94,102,-74,-29,-82,-37,19,-77,12,-49,
            -104,-31,75,-1,102,-8,-14,85,-36,-81,
            48,124,89,-95,-31,43,39,24,-2,-56,
            105,-61,91,-105,113,63,-41,-75,-33,85,
            -15,-90,126,111,-15,-2,86,-70,-6,-46,
            -37,-82,-57,-83,74,114,-19,-57,-6,43,
            -41,56,20,-105,-14,-96,-68,40,79,-54,
            -101,116,80,-65,19,127,-1,-16,58,120,
            -49,0,-99,17,87,-48,109,48,-26,56,
            9,105,100,7,106,-65,14,89,-128,85,
            -58,-65,7,24,107,123,28,-24,13,108,
            -95,-128,4,-97,-75,-123,-20,4,65,118,
            62,-91,-33,21,-68,26,108,88,76,-3,
            -74,-102,10,-44,103,-38,-89,1,-58,-56,
            -13,-3,-49,-118,66,26,-25,-83,52,125,
            85,28,-29,113,-62,41,46,-27,65,121,
            81,-98,-108,55,-41,-47,-105,-6,-27,58,
            -49,-104,-22,-16,-67,-73,14,-48,82,106,
            -123,-26,-126,97,86,18,-110,-79,0,84,
            -27,49,-89,-23,54,-10,61,-48,-1,124,
            115,36,-112,-41,-37,63,7,52,64,-89,
            11,-41,-44,1,-106,-101,-4,19,-2,47,
            17,97,14,88,23,74,-3,-18,77,6,
            85,127,-32,-3,-114,-32,126,-28,-34,-127,
            -96,106,5,29,-57,-49,107,110,-70,-126,
            78,-82,9,2,20,-25,116,33,-27,65,
            121,81,-98,-108,119,-58,59,-96,106,7,
            -11,27,-102,3,-122,-123,-72,-85,-125,-73,
            13,80,34,-75,-62,-30,64,-97,-82,36,
            100,-46,-109,64,57,97,8,101,27,-40,
            5,39,-20,-58,7,64,-67,19,53,4,
            -48,0,93,-118,-75,103,-126,1,73,-11,
            -25,-127,1,-55,-10,96,-48,-61,26,50,
            24,-20,-18,39,6,40,8,-88,105,-52,
            100,123,27,2,-75,63,29,23,50,8,
            -20,30,-50,-5,9,-32,-3,-66,72,113,
            40,46,-27,65,121,81,-98,-108,-9,-92,
            -90,64,17,51,67,88,59,-48,39,-55,
            93,29,-68,-52,-84,39,-88,-43,-32,109,
            -48,45,-102,-124,-92,-19,7,-22,119,92,
            -24,-21,-92,-8,122,60,-48,19,-20,125,
            -127,-2,51,49,117,-60,2,86,42,-90,
            -88,88,-64,52,-21,75,-128,6,-110,101,
            -17,-50,3,122,-107,121,-128,118,23,0,
            -3,10,63,110,-77,56,-113,-9,-29,46,
            14,-27,65,121,-103,-70,125,61,-19,32,
            80,-65,-93,-29,-71,-50,-89,-36,-43,-63,
            -37,6,104,58,-75,26,30,0,79,-89,
            -112,-112,-66,-73,-127,86,-62,-123,-74,55,
            -90,14,-58,42,-26,0,-3,-30,-84,-31,
            64,-113,29,63,10,-24,-25,-46,-117,-128,
            58,54,61,26,-88,83,-46,7,3,-11,
            -39,-12,2,-96,55,27,-109,3,-104,116,
            106,57,96,49,-26,1,58,-11,33,96,
            -51,-58,-28,2,93,-93,-13,68,63,-44,
            47,-113,67,113,41,15,-54,-117,-14,-92,
            -68,-5,86,1,-19,18,-23,-30,58,123,
            -69,-85,-125,-73,13,80,16,-75,-62,-65,
            4,109,-8,93,-112,118,-71,-37,37,-64,
            122,-16,-71,-4,71,-91,16,8,-31,85,
            -84,-62,73,41,-45,-99,-48,-92,114,-56,
            -28,15,114,95,-60,-3,101,-18,101,-10,
            96,-109,77,-57,21,-103,-6,-71,-37,63,
            -59,-91,60,120,94,-82,121,-22,-25,-69,
            -107,3,-58,31,96,-75,-110,-16,109,-128,
            116,-34,95,7,47,51,-53,86,113,5,
            -59,-128,22,-65,-32,-73,-83,-99,99,66,
            -127,118,-93,98,17,-48,68,5,-107,127,
            2,86,-94,6,2,118,91,109,12,24,
            -69,-49,63,1,28,-66,37,112,-100,119,
            29,48,-35,48,-90,-102,71,-122,-17,87,
            -24,56,58,79,-12,-29,46,14,-49,-125,
            -14,-94,-2,42,-78,-128,46,-59,68,1,
            -19,-33,92,87,-89,-122,-79,-96,69,-86,
            -69,58,120,-101,49,-47,90,4,66,-83,
            -106,-125,-32,-20,43,52,37,116,124,3,
            84,21,-99,76,3,90,38,21,64,77,
            0,76,86,-34,7,76,83,-82,0,-57,
            64,-100,1,15,-15,69,-64,-31,-117,-63,
            61,-98,-10,63,-22,124,-118,-61,-29,82,
            30,-108,-41,-55,81,64,-53,-20,56,11,
            84,21,-119,-17,46,67,87,26,-41,-23,
            -66,14,94,122,37,-11,116,-35,110,-110,
            106,-7,8,-28,101,-109,-64,-32,-61,64,
            -21,-77,82,3,-38,73,94,-113,-33,-80,
            20,39,10,-37,-26,-124,-90,-70,18,-113,
            121,-118,67,113,121,30,31,5,2,-19,
            100,112,33,-48,-18,62,-9,-84,2,121,
            95,112,93,67,30,-82,-37,11,-17,-77,
            69,-86,-99,-128,-108,95,119,50,-112,14,
            -57,-121,1,-57,-93,-31,118,-16,102,-84,
            -11,109,-96,-116,36,-31,-3,103,1,109,
            -31,60,59,-48,-69,-67,25,7,-12,-44,
            -116,63,2,125,64,-58,122,-16,24,61,
            -17,-105,-30,-16,-72,73,-3,51,-128,-74,
            -128,-14,-30,121,-66,-54,-13,-114,-117,15,
            7,-110,84,55,29,72,-121,73,-89,-69,
            58,120,-85,-43,-69,-25,-67,-82,3,-53,
            37,-53,95,65,-32,16,-37,34,-32,80,
            -16,4,24,-11,-78,117,43,-96,-37,102,
            101,-71,20,4,-40,76,-55,2,24,61,
            111,124,-53,125,-63,99,-10,-33,-118,56,
            20,-41,-56,99,33,-27,101,107,2,70,
            -115,-74,45,6,-114,15,30,-41,65,-70,
            -36,-22,-10,17,51,-35,-51,4,-120,86,
            -56,126,-112,58,-108,62,-87,67,71,3,
            -71,-3,-82,-41,-128,-34,96,-53,69,-96,
            79,-50,-50,7,-6,-85,-98,-14,20,-121,
            -30,14,-99,0,-28,54,-30,1,119,31,
            72,29,-19,78,-57,-125,117,-6,-98,89,
            -8,-107,95,74,59,-22,-50,7,-109,-126,
            -87,16,75,-114,1,121,-95,-72,-69,59,
            6,-104,-58,60,108,-26,56,75,-50,0,
            89,76,113,117,-25,-127,73,17,-90,25,
            -20,-102,-85,-82,90,99,1,9,-30,10,
            74,2,115,35,-88,16,-97,12,4,114,
            11,42,84,-27,-81,1,-69,-90,-90,3,
            118,-53,83,-98,-30,80,-36,79,-46,-128,
            -36,92,92,65,-49,-125,-71,45,-35,-23,
            -88,45,-58,63,105,1,-30,-107,72,80,
            36,88,54,-123,10,-111,-35,6,-56,-19,
            -60,109,-9,52,-64,110,121,-6,10,82,
            -89,-125,-69,113,-78,19,-127,-36,-114,-14,
            10,10,7,-53,50,77,3,-12,84,109,
            -69,-126,-8,28,109,-37,35,38,-119,-99,
            96,-19,63,-88,16,123,38,0,121,-103,
            40,92,57,96,55,61,62,64,-90,56,
            123,102,0,121,-87,120,11,-2,53,88,
            107,94,78,-40,83,91,-66,123,-56,26,
            -13,87,32,-41,-60,37,-43,15,-28,-108,
            80,33,10,-57,2,-91,35,21,74,123,
            23,120,126,-128,-52,113,10,-89,0,-91,
            -125,120,-18,-23,11,114,-54,77,-81,114,
            -82,-71,-22,-14,125,123,-114,15,-53,22,
            -41,-35,5,31,91,-97,1,-116,21,-41,
            3,120,-90,-25,-81,106,34,-127,-25,7,
            -56,28,-89,56,28,40,87,-84,-49,3,
            -70,-3,47,-40,104,-102,-79,55,-69,-22,
            -14,125,-101,-3,-32,-35,39,62,-85,-9,
            57,96,-20,106,91,-96,110,18,-123,91,
            11,60,-1,29,100,-114,115,-75,3,80,
            55,-43,-5,10,-48,0,-99,-40,-2,-65,
            -23,-14,61,-37,71,-115,-60,-115,32,-72,
            -75,-79,117,62,-76,-15,-109,-128,-79,-118,
            -25,-128,54,70,84,46,-36,73,-123,-89,
            7,-56,28,-89,-30,5,-96,-115,105,28,
            13,104,-128,-50,71,37,-66,11,-126,-29,
            -36,-23,-14,-11,-101,-72,14,-44,106,90,
            0,-20,103,-115,-83,-78,13,49,-67,-127,
            115,-123,-56,-127,30,40,10,55,-51,73,
            -91,-57,7,-56,28,-89,19,-48,-125,98,
            -122,0,26,-96,-78,-51,77,-13,-127,-3,
            -126,59,93,-66,62,64,57,-44,114,78,
            -7,122,-117,121,-58,86,-43,-53,-99,-38,
            1,49,-31,-84,97,81,78,28,-9,-69,
            78,100,-113,15,16,-59,-119,116,34,-14,
            -24,-44,25,-48,0,85,-3,50,82,1,
            45,22,-71,-45,-27,-29,102,-19,69,45,
            -5,76,16,-81,-46,93,82,-81,-65,0,
            125,28,-81,76,51,45,27,56,90,-89,
            -99,-88,30,31,32,30,71,-37,10,-60,
            -34,102,-67,114,-127,-2,-102,88,10,-97,
            14,18,76,-65,24,-75,-90,-44,-106,1,
            42,-90,86,-40,53,-48,93,-84,15,13,
            95,12,-44,63,-15,-62,-52,87,-33,2,
            -114,86,-66,19,-123,29,114,-94,122,-52,
            -13,56,34,46,-49,99,-8,-5,64,-35,
            72,121,-122,-107,-126,-18,99,-36,-23,-14,
            113,-77,-51,-92,86,104,62,24,-14,14,
            9,-97,28,3,20,-70,-126,-10,-53,61,
            -128,99,-110,105,13,28,-2,-25,-64,-125,
            -98,-57,-111,-69,3,49,64,-5,39,39,
            0,101,44,-27,-55,-13,94,-30,78,-105,
            -113,-66,-38,-111,-8,-17,-59,108,95,-119,
            -105,-92,123,-64,-8,-73,72,-8,-4,91,
            64,30,-52,11,51,67,57,12,-40,18,
            -66,-67,-88,-70,-68,41,-18,-52,-7,58,
            -112,7,-117,-105,-90,-33,-128,-15,-90,-17,
            32,-95,43,-60,87,94,-3,-104,127,-43,
            63,94,-22,15,-126,99,36,-101,19,-55,
            -38,24,100,-120,43,-88,85,10,-112,47,
            -12,-52,3,-6,-111,-92,101,64,-1,-87,
            -37,2,-96,-1,80,93,62,-23,119,64,
            -1,-119,-14,104,-43,7,-56,23,-60,-62,
            -99,-111,-9,50,-46,33,116,-111,-50,-121,
            -41,-63,-117,22,-24,-84,-64,50,-37,114,
            4,4,-40,-125,-78,-128,36,5,-76,3,
            -119,27,108,-125,64,-15,106,18,46,37,
            0,-115,22,-28,-110,-7,2,90,31,-66,
            -35,-69,-38,-68,107,-36,100,41,30,104,
            5,-30,-99,-100,-111,-9,-38,-128,-74,32,
            113,83,-48,18,-32,-72,100,-72,78,-46,
            -19,-67,11,120,51,-100,-92,-122,54,5,
            -106,-32,-88,113,-64,49,48,87,-63,-44,
            -9,108,107,-128,60,-126,4,15,108,5,
            -76,69,-89,118,0,61,-91,-8,18,-48,
            51,-117,-70,2,125,106,-111,-45,-12,105,
            -43,-26,121,-36,-30,-117,64,-49,60,-75,
            19,-24,41,3,91,-125,-69,75,-32,92,
            -57,43,92,-41,-46,-88,9,-64,-15,-35,
            -54,117,75,-45,-99,-92,122,-37,87,-114,
            105,-119,-79,-63,26,-87,28,108,121,91,
            76,17,-109,-127,26,-109,117,25,40,55,
            -8,92,-65,-109,109,112,-62,-40,58,39,
            -98,127,-18,121,-108,81,30,-21,-99,-112,
            125,-99,117,21,40,-41,73,-121,-104,1,
            92,116,54,-8,-12,-31,117,-87,-127,21,
            56,-2,101,89,34,102,-72,25,96,69,
            25,9,-120,-98,3,-18,-56,7,-122,1,
            -83,-117,120,-51,-33,3,56,-98,63,70,
            3,86,-90,39,2,86,-82,-121,0,-121,
            -81,15,-86,-47,83,92,-98,7,-27,69,
            121,82,-34,-92,35,122,54,-72,83,37,
            62,-128,-122,-18,-85,-90,-101,-120,18,-41,
            58,-43,-36,50,-126,72,-52,-38,25,124,
            -47,-120,18,-33,119,12,-56,-7,-30,-89,
            -98,61,1,43,101,-103,78,42,-17,-7,
            109,41,-88,121,51,-25,-63,-13,20,121,
            115,-37,119,20,-56,-69,-60,0,-71,-24,
            22,-43,-72,90,-45,-53,18,-12,-73,-100,
            -11,93,119,31,28,110,27,6,24,43,
            106,9,-44,58,-30,-54,89,1,-40,57,
            -67,27,96,-41,-11,23,-127,-61,39,3,
            47,-14,-108,23,-49,-109,-14,38,29,69,
            -47,64,-83,67,58,-71,-18,17,-90,41,
            -82,-66,107,-99,-86,-33,-74,81,-93,75,
            19,16,-72,-50,-40,58,-9,-79,-35,2,
            24,43,95,14,-76,-69,111,-37,78,3,
            -49,47,35,60,-10,101,9,83,-34,-27,
            31,0,45,-63,-8,115,22,26,-96,115,
            -97,118,-79,-125,-64,-11,-90,58,109,-81,
            -87,-101,3,-15,-53,-54,-16,74,16,-43,
            -44,-40,-70,-34,-93,89,41,96,-33,-100,
            -5,16,-56,-11,46,-20,2,74,-23,-113,
            63,0,37,-1,124,6,80,10,124,-59,
            83,-34,23,118,2,-91,-12,-36,106,32,
            -121,112,-99,121,92,119,-81,-16,-37,32,
            -86,-91,-69,58,85,-9,59,-74,-89,69,
            115,44,-120,-21,44,-106,-118,-57,1,-58,
            2,-61,0,-5,109,-64,42,-64,88,-64,
            86,-32,-16,-97,1,31,-14,-108,55,-41,
            65,-70,72,-89,-72,-85,51,-22,-112,-28,
            -74,78,-43,60,64,-90,-65,-117,-23,120,
            -39,-16,71,-7,-97,44,30,-32,-119,-3,
            61,-105,-5,83,-75,-52,-25,-70,-22,60,
            -54,-9,119,44,127,120,-99,-86,-3,54,
            91,-40,27,-36,71,-69,121,55,87,-37,
            -52,-84,43,-38,84,7,-87,-90,111,-77,
            -51,63,18,-25,-105,-74,-123,63,73,-117,
            -1,-40,49,-48,-76,93,91,-68,89,87,
            -86,107,29,-36,-42,-87,-58,62,65,-12,
            19,89,-2,-25,26,18,45,104,-67,84,
            -53,125,-118,73,119,-60,-1,-55,12,-30,
            55,-65,121,-71,-3,7,114,70,-81,-40,
            -120,82,-71,38,0,0,0,90,122,84,
            88,116,67,111,109,109,101,110,116,0,
            0,120,-38,115,73,44,73,-75,82,8,
            79,77,-47,81,48,50,87,112,44,77,
            87,48,50,48,-80,80,48,-76,-76,50,
            -78,-80,50,49,81,8,112,9,-31,114,
            44,45,-55,-56,47,-78,82,112,45,-54,
            76,86,112,73,85,-16,45,-51,75,81,
            -80,73,77,76,113,-56,-84,-56,76,-52,
            -45,75,-50,-49,-75,-29,2,0,-31,23,
            21,24,123,-69,10,-77,0,0,0,0,
            73,69,78,68,-82,66,96,-126
        };
      }
}
